require "Third/sproto/print_r"
local protobuf = require "protobuf"

--[[
-- 支持格式：
-- 1、plog(基本数据类型)
-- 2、plog(元表, "注释")
--]]
plog = function(fmt, ...)
	if Global.showLog then
		loging(fmt, ...)
	end
end

function loging(fmt, ...)
	local title = ... or "NoTittle"
	if type(fmt) == "boolean" then
		print("[[---------------------------------- "..title)
		print(fmt)
		print("----------------------------------]]\n")
	elseif not fmt then
		print("[[---------------------------------- "..title)
		print("NULL")
		print("----------------------------------]]\n")
	elseif type(fmt) == "userdata" then
		print("[[---------------------------------- "..title)
		Util.Log(fmt)
		print("----------------------------------]]\n")
	elseif type(fmt) == "function" then
		print(fmt, ...)
	elseif type(fmt) == "table" then
		print_r(fmt, title)
	else
		if ... then
			local exist = string.find(fmt, "%%")
			if exist ~= nil then
				print(string.format(fmt, ...))
			else
				print(fmt, ...)
			end
		else
			print(string.format(fmt, 0))
		end
	end
end

--输出日志--
function log(str)
    Util.Log(str);
end
--打印字符串--
-- function print(str) 
-- 	Util.Log(str);
-- end

--错误日志--
function error(str, ...)
	if ... then
		str = string.format(str, ...)
	end
    Util.LogError("<From Lua Error>: " .. tostring(str) .."\n\n\n" .. debug.traceback() );
end

--警告日志--
function warn(str, ...)
	if ... then
		str = string.format(str, ...)
	end
	Util.LogWarning("<From Lua Warning>: " .. tostring(str) .."\n\n\n" .. debug.traceback() );
end

local function __TRACKBACK__(errmsg)
    local track_text = debug.traceback(tostring(errmsg), 1) or "track_text == nil";
    print("---------------------------------------- TRACKBACK ----------------------------------------");
    Util.LogError("<From Lua>: "..track_text);
    print("---------------------------------------- TRACKBACK ----------------------------------------");
    return false;
end

--[[ 
尝试调一个function 这个function可以带可变参数
如果被调用的函数有异常 返回false，退出此方法继续执行其他代码并打印出异常信息；
--]]
function TryCatch(func, ...)
    local args = { ... };
    return xpcall(function() func(unpack(args)) end, __TRACKBACK__);
end

-- 查找对象
function FindObject(str)
	return GameObject.Find(str);
end

-- 销毁对象
function DestroyObject(obj, time)
	if not obj then return end
	time = time or 0
	if time <= 0 then
		GameObject.DestroyImmediate(obj)
	else
		GameObject.Destroy(obj, time)
	end
end

-- 销毁非忽略的子对象
function DestroyChildren(obj, ignoreObjs)
	ignoreObjs = ignoreObjs or {}
	local tran = obj.transform
	for i = tran.childCount - 1, 0, -1 do
		local child = tran:GetChild(i)
		local childObj = child.gameObject
		if not IsMember(childObj, ignoreObjs) then
			DestroyObject(childObj)
		end
	end
end

-- 获取子对象
function ObtainChildren(obj, ignoreObjs)
	local children = {}
	ignoreObjs = ignoreObjs or {}
	local tran = obj.transform
	for i = 0, tran.childCount - 1 do
		local child = tran:GetChild(i)
		local childObj = child.gameObject
		if not IsMember(childObj, ignoreObjs) then
			table.insert(children, childObj)
		end
	end
	return children
end

-- 创建对象
function CreateObject(prefab)
	if prefab == nil then return nil end
	return GameObject.Instantiate(prefab)
end

-- 根据路径名创建对象
function CreateObjectByPath(name, abName)
	local prefab = Util.LoadPrefab(name, abName)
	if prefab == nil then
		warn("---->>Not Found Prefab: %s", path)
		return nil
	end
	return GameObject.Instantiate(prefab)
end

-- 挂载对象及可延时销毁
function HangObject(child, parent, time)
	if child ~= nil and parent  ~= nil then
		child.transform.parent = parent.transform
		ResetObject(child)
		if time and time > 0 then
			GameObject.Destroy(child, time)
		end
	end
end

-- 重置对象
function ResetObject(go, name, parent)
	if name then
		go.name = name
	end
	if parent then
		go.layer = parent.layer
		go.transform.parent = parent.transform
	end
	go.transform.localPosition = Vector3.zero
	go.transform.localRotation = Quaternion.identity
	go.transform.localScale = Vector3.one
end

-- 重置网格
function ResetGrid(gridObj)
	local grid = gridObj:GetComponent(UIGrid.GetClassType())
	if grid ~= nil then
		grid.enabled = true
		grid:Reposition()
--		plog("Grid reset.")
	end
end

-- 重置滚动视图
function ResetScrollView(scvObj)
	local scv = scvObj:GetComponent(UIScrollView.GetClassType())
	if scv ~= nil then
		scv:ResetPosition()
--		plog("Scroll view reset.")
	end
end

--创建面板--
function createPanel(name)
	PanelManager:CreatePanel(name);
end

function child(str)
	return transform:FindChild(str);
end

function subGet(childNode, typeName)		
	return child(childNode):GetComponent(typeName);
end

function findPanel(str) 
	local obj = FindObject(str);
	if obj == nil then
		error(str.." is null");
		return nil;
	end
	return obj:GetComponent("BaseLua");
end

function getMatchineRight(curNum,num)
	for i = 1,num do
		curNum = math.floor(curNum/2);
	end
	return curNum;
end

function getMatchineLeft(curNum,num)
	for i = 1,num do
		curNum = curNum*2;
	end
	return curNum;
end

function printTable(t,prefix)
  for k,v in pairs(t) do
    if type(v) == "string" then
      print(string.format("%s => %s", prefix .. "." .. k,v))
    else
      print(prefix .. "." .. k..type(v))
    end
    if type(v) == "table" and k ~= "_G" and k ~= "_G._G" and k ~= "package" then
      printTable(v, "  " .. prefix .. "." .. k)
    end
  end
end

-- by kenny: 判断obj是否含有指定的类型(含父类)
function typeofClass(obj, class)
	if IsEmptyTable(obj) or not class then
		return false
	end
	if obj == class then return true end
	local meta1 = getmetatable(obj)
	if not meta1 then return false end
	if meta1.cls_type == class then
		return true
	end
	local meta2 = getmetatable(meta1.cls_type)
	if not meta2 or meta2.cls_type == nil then
		return false
	end
	return typeofClass(meta1.cls_type, class)
end

--lua 字符串分割函数   
-------------------------------------------------------   
-- 参数:待分割的字符串,分割字符
-- 返回:子串表.(含有空串)   
function StrSplit(str, delimiter)       
	if str==nil or str=='' or delimiter==nil then
		return nil
	end
	
    local result = {}
    for match in (str..delimiter):gmatch("(.-)"..delimiter) do
        table.insert(result, match)
    end
    return result
end

-- 默认返回元表，StrSplit会出现nil情况
function split(str, delimiter)
	return StrSplit(str, delimiter) or {}
end

function Replace(str, pat, repl) return string.gsub(str, pat, repl) end

function Trim(s) return (string.gsub(s, "^%s*(.-)%s*$", "%1")) end

function GetGameObject(trans, path)
    if trans == nil then
        warn("GetGameObject transform == nil, path = %s", path)
        return nil
    end    

    local pathTran = trans:Find(path)
    if pathTran ~= nil then
		return pathTran.gameObject
    end
	
	return nil
end    

function GetPathComponent(trans, path, componentName)
	return GetComponent(GetGameObject(trans, path), componentName)
end    

function GetComponent(trans, compName)
    if not trans or not compName then
        error("Error: GetComponent => transform = %s, component = %s", tostring(trans), tostring(compName))
        return nil
    end

    local component = trans:GetComponent(compName)
    if not component then
        return nil
    end
    return component
end

function GetComponentInChildren(tran, classType, includeInactive)
	includeInactive = includeInactive and true or false
	return tran:GetComponentInChildren(classType, includeInactive)
end

function GetComponentsInChildren(tran, classType, includeInactive)
	includeInactive = includeInactive and true or false
	return tran:GetComponentsInChildren(classType, includeInactive)
end

function GetComponentInParent(tran, classType)
	return tran:GetComponentInParent(classType)
end

function GetComponentsInParent(tran, classType, includeInactive)
	includeInactive = includeInactive and true or false
	return tran:GetComponentsInParent(classType, includeInactive)
end

--取一个数的整数部分
function GetIntPart(x)
	if not x then return 0 end
    if x <= 0 then
       return math.ceil(x);
    end

    if math.ceil(x) == x then
       x = math.ceil(x);
    else
       x = math.ceil(x) - 1;
    end

    return x;
end

function IsNumber(v)
    local t = type(v);
    if t == "number" then
        return true;
    end

    return false;
end    

function IsString(v)
    local t = type(v);
    if t == "string" then
        return true;
    end

    return false;
end

--判断任意类型是否为空
function IsNilOrEmpty(input)
	if input==nil or input=="" then
		return true
	end
    return false
end

-- 判断任意类型是否为空
function IsEmpty(input)
	if input == nil then
		return true
	end
	local inType = type(input)
	if inType == "table" and not next(input) then
		return true
	elseif inType == "string" and Trim(input) == "" then
		return true
	end
	return false
end

-- 判断string是否为空
function IsEmptyString(input)
	if type(input) ~= "string" then
		return true
	end
	if input==nil or input=="" then
		return true
	end
    return false
end

-- 判断table是否为空
function IsEmptyTable(input)
	if type(input) ~= "table" then
		return true
	end
	if input==nil or not next(input) then
		return true
	end
    return false
end

-- 解码Json字符串
function DecodeJsonString(str)
	local tbl = nil
	tbl = cjson.decode(str)
	return tbl
end

-- 编码Json字符串
function EncodeJsonString(tbl)
	local str = nil
	str = cjson.encode(tbl)
	return str
end

-- 解码Json文件
function DecodeJsonFile(path)
	local jstr = ReadContentFromFile(path)
	return cjson.decode(jstr) or {}
end

-- 编码Json文件
function EncodeJsonFile(path, tbl)
	local src = DecodeJsonFile(path)
	if not IsEmptyTable(src) then
		AttachMapTable(tbl, src)
	else
		src = CopyTable(tbl)
	end
	local jstr = EncodeJsonString(src)
	WriteContentToFile(path, jstr)
end

-- TAG标记生成器
function TagGenerator()
	if cur_tag == nil then
		cur_tag = math.random(1000, 9999)
	end
	cur_tag = cur_tag + 1
	return cur_tag
end

-- 枚举生成器
function EnumGenerator(tbl, index)
    assert(not IsEmpty(tbl), "Enum table is nil")
    local enumtbl = {}
	local index = tonumber(index) 
	if not index then
		index = 1
	end
	local enumindex = index
    for i, v in ipairs(tbl) do
        enumtbl[v] = enumindex + i - 1
    end
    return enumtbl
end

-- 计算数组总大小包括List/Map
function CountTable(tbl, notAllowNull)
	local count = 0
	if not IsEmpty(tbl) then
		for _, child in pairs(tbl) do
			if notAllowNull then
				if not IsEmpty(child) then
					count = count + 1
				end
			else
				count = count + 1
			end
		end
	end
	return count
end

-- 按照key排序
function SortByKey(tbl)
	local keys = {}
	local datas = {}
	if not IsEmpty(tbl) then
		for key, _ in pairs(tbl) do
			table.insert(keys, key)
		end
		table.sort(keys)
		for _, key in ipairs(keys) do
			table.insert(datas, tbl[key])
		end
	end
	return keys, datas
end

-- 判断是否为表中成员, 返回boolean, serially可不传
function IsMember(val, tbl, serially)
	if IsEmpty(val) or IsEmpty(tbl) then
		return false
	end
	if serially == true then
		for k, v in ipairs(tbl) do
			if v == val then
				return true, k
			end
		end
	else
		for k, v in pairs(tbl) do
			if v == val then
				return true, k
			end
		end
	end
	return false
end

-- 判断是否为表中Key成员, 返回boolean, serially可不传
function IsMemberKey(key, tbl, serially)
	if IsEmpty(key) or IsEmpty(tbl) then
		return false
	end
	if serially == true then
		for k, v in ipairs(tbl) do
			if k == key then
				return true, k
			end
		end
	else
		for k, v in pairs(tbl) do
			if k == key then
				return true, k
			end
		end
	end
	return false
end

-- 转换字符串中的坐标
function TranPosition(strPos)
	local tbl = split(strPos, ",") or {}
	local pos = {x=0, y=0, z=0}
	pos.x = tonumber(tbl[1]) or 0
	pos.y = tonumber(tbl[2]) or 0
	pos.z = tonumber(tbl[3]) or 0
	return pos
end

-- 更新随机种子
function UpdateRandomSeed(seed)
	-- 指定种子
	if seed ~= nil then
		cur_seed = seed
		math.randomseed(seed)
		return
	end
	-- 用时间产生随机种子
	local tmp_seed = tostring(os.time()):reverse():sub(1, 6)
	while cur_seed == tmp_seed do
		tmp_seed = math.random(0, 999999)
	end
	cur_seed = tmp_seed
	math.randomseed(cur_seed)
end

-- 随机正负数
function RandomWithoutZero(num0, num1)
	if num0 > num1 then num1 = num0 end
	local zf = {-1, 1}
	return math.random(num0, num1) * zf[math.random(1, 2)]
end

-- 随机等份概率下标
function RandomIndex(tblNum, maxRand)
	local idx = 1
	if tblNum > 1 then
		local rand = math.random(1, maxRand)
		local area = maxRand/tblNum
		idx = GetIntPart(rand/area) + 1
		if idx<1 or idx>tblNum then
			idx = 1
		end
	end
	return idx
end

-- 随机连续数组成员
function RandomMember(tbl)
	if IsEmptyTable(tbl) then
		return nil
	end
	local num = table.getn(tbl)
	local idx = RandomIndex(num, 100)
	return tbl[idx], idx
end

-- r, g, b均为RGB值: 0~255; a为透明通道值: 0~255
function CreateColor(r, g, b, a)
	r = r or 0
	g = g or 0
	b = b or 0
	a = a or 0
	return Color.New(r/255, g/255, b/255, a/255)
end

-- r, g, b均为RGB值: 0~1; a为透明通道值: 0~1
function CreateColorF(r, g, b, a)
	r = r or 0
	g = g or 0
	b = b or 0
	a = a or 0
	return Color.New(r, g, b, a)
end

-- 去掉空元素
function TrimTable(tbl)
	if IsEmpty(tbl) then return {} end
	for i = #tbl, 1, -1 do
		local obj = tbl[i]
		if IsEmpty(obj) then
			table.remove(tbl, i)
		end
	end
end

function GetProtoBuffer(name, data)
	local buffer = protobuf.encode("com.ftkj.proto."..name, data)
    return Util.getLuaBufferData(buffer)
end

function GetProtoTable(name, session)
	local data = session.luaReceiveData:ReadBuffer()
	local buffer = protobuf.decode("com.ftkj.proto."..name, data)
	return buffer
end

function DecodeURI(s)
	s = string.gsub(s, '%%(%x%x)', function(h) return string.char(tonumber(h, 16)) end)
	return s
end

function EncodeURI(s)
	s = string.gsub(s, "([^%w%.%- ])", function(c) return string.format("%%%02X", string.byte(c)) end)
	return string.gsub(s, " ", "+")
end

function RequestURL(uri, callback, args)
	plog("URI = ", tostring(uri))
	if not uri then
		warn("GameObject or Uri is null.")
		return
	end
	local obj = GameObject.New()
	obj.name = "Access"
	obj:AddComponent(gamespace.Access.GetClassType())
	local access = GetComponent(obj.transform, "Access")
	access:RequestURL(uri, function(args)
		callback(args)
		DestroyObject(obj, 0.2)
	 end, args or {})
end

----debug
--function saveErrorInfo(errorInfo, traceback)
--	local errKey = Tools:getMD5(errorInfo)
--	if _errList[errKey] ~= nil then
--		return
--	end
--	_errList[errKey] = errorInfo
--	local appId = GameManager:getAppID()
--	local uuid = GameManager:getDeviceID()
--	local phone = DataArea:getInstance():getValue("PhoneNumber")
--	local model = DataArea:getInstance():getValue("PhoneModel")
--	local resVersion = GameManager:getResourceVersion()
--	local codeMD5 = LuaDecode:defaultDecode():getClassesMD5()
--	local session = LoginWnd:getLoginSession()

--	local host = "http://101.251.102.144/lcz/errinfo.jsp"
--	host = host .. "?appId=" .. encodeURI(appId)
--	host = host .. "&uuid=" .. encodeURI(uuid)
--	host = host .. "&model=" .. encodeURI(model)
--	host = host .. "&resVersion=" .. encodeURI(resVersion)
--	host = host .. "&codeMD5=" .. encodeURI(codeMD5)
--	host = host .. "&errInfo=" .. encodeURI(errorInfo)
--	host = host .. "&traceback=" .. encodeURI(traceback)
--	host = host .. "&phone=" .. encodeURI(phone)
--	host = host .. "&type=" .. encodeURI("lua")
--	if session ~= nil and session.areaName ~= nil then
--		host = host .. "&areaID=" .. encodeURI(session.areaName)
--	else
--		host = host .. "&areaID="
--	end

--	local hc = HttpConnection:getInstance()
--	local request = HttpRequest:create()
--	request:setUrl(host)
--	request:setRequestType(HttpRequest.kHttpGet)
--	hc:sendQueue(request)
--	request:release()
--end

-- 时间字符串
function Time2String(seconds)
	seconds = tonumber(seconds) or 0
	if seconds <= 0 then
		return "00:00:00"
	end
	local hour = math.modf((seconds + 0.5) / 3600)
	local min = math.modf((seconds + 0.5) / 60 % 60)
	local sec = math.modf((seconds + 0.5) % 60)

	local strHour = ""
	local strMin = ""
	local strSec = ""
	if hour < 10 then
		strHour = '0'..hour
	else
		strHour = hour
	end
	if min < 10 then
		strMin = '0'..min
	else
		strMin = min
	end
	if sec < 10 then
		strSec = '0'..sec
	else
		strSec = sec
	end

	local strTime = strHour..":"..strMin..":"..strSec
	return strTime
end

-- 日期字符串
function Date2String(seconds)
    return os.date("%Y-%m-%d %H:%M:%S", seconds)
end


--[[
	空函数
	2016-6-23 15:51:08 library
]]
function doNthFunc ()
	
end

-- 绑定Lua脚本
function BindLuaScript(luaObj, gameObj, isGlobal, createFunc, awakeFunc, startFunc, destroyFunc)
	if not createFunc then createFunc = function() end end
	if not awakeFunc then awakeFunc = function() end end
	if not startFunc then startFunc = function() end end
	if not destroyFunc then destroyFunc = function() end end
	PanelManager:BindPanelScript(luaObj, gameObj, createFunc, awakeFunc, startFunc, destroyFunc)

	if isGlobal then
		local lbs = GetComponentsInChildren(gameObj.transform, LuaBehaviour.GetClassType())
		for i = 0, lbs.Length - 1 do
			local lb = lbs[i]
			if lb.sourceLuaObj == luaObj then
				lb.isGlobal = true
			end
		end
	end
end

-- 创建数字类字符
function CreateWord(wordText, atlasPath, abName, parentObj, layerName) -- parentObj、layerName:可选
	if not atlasPath then return nil end
	if not parentObj then
		parentObj = GameObject.Find("UIRoot")
	end
	local obj = GameObject.New()
	HangObject(obj, parentObj) -- 要先挂上，才能刷Mesh
	obj.name = "WordObj"
	obj:AddComponent(MeshWord.GetClassType())
	local word = GetComponent(obj, "MeshWord")
	word.hAlignType = HorizontalAlignType.Center
	local mesh = CreateObjectByPath(atlasPath, abName)
	mesh.name = "Word"
	HangObject(mesh, obj)
	word.go = mesh
	word.uiAtlas = GetComponent(mesh.transform, "UIAtlas")
	word.fontSize = 1
	word.text = wordText
	if IsMember(layerName, LayerName) then
		ViewUtil.sortLayerOrder(obj, layerName)
	else
		ViewUtil.sortLayerOrder(obj, LayerName.UI)
	end
	
	return obj, word
end

-- 交换元表
function ExchangeTable(tbl, fromKey, toKey)
	if IsEmpty(tbl) then return end
	if IsEmpty(tbl[fromKey]) or IsEmpty(tbl[toKey]) then
		warn("Exchange key not found!")
		return
	end
	local tmp = tbl[toKey]
	tbl[toKey] = tbl[fromKey]
	tbl[fromKey] = tmp
	tmp = nil
	return tbl
end

-- 拷贝元表
function CopyTable(target)
	local ret = {}
	if not IsEmptyTable(target) then
		for key, val in pairs(target) do
			ret[key] = CopyTable(val)
		end
	else
		ret = target
	end
	return ret
end

-- 追加Map元表
function AttachMapTable(dest, src)
	local ret = src or {}
	if not IsEmpty(dest) then
		for key, val in pairs(dest) do
			ret[key] = val
		end
	end
	return ret
end

-- 追加List元表
function AttachListTable(dest, src)
	if not IsEmpty(src) then
		for i = 1, #src do
			table.insert(dest, src[i])
		end
	end
end

-- 延迟调用Lua方法
function DelayCall(func, time)
	return LuaCallManager:invokeCall(func, time)
end

-- 取消延迟调用, 一般添加在OnDestroy里
function CancelDelayCall(coroutine)
	LuaCallManager:cancelInvokeCall(coroutine)
end

-- 获取文字词语
function GetWord(code)
	local ret = nil
	local data = Config.getLangData()
	local langs = data[tostring(code)]
	if not IsEmpty(langs) then
		if Global.localLang == Language.SimpleChinese then
			ret = langs["SimpleChinese"]
		elseif Global.localLang == Language.English then
			ret = langs["English"]
		end
	end
	ret = ret or " "
	return ret
end

function GetTotalTransform(obj)
	local trans = GetComponentsInParent(obj.transform, Transform.GetClassType(), true)
	local totalPosition = Vector3.zero
	local totalRotation = Vector3.zero
	local totalScale = Vector3.one
	for i = 0, trans.Length - 1 do
		local tran = trans[i]
		totalPosition.x = totalPosition.x + tran.localPosition.x
		totalPosition.y = totalPosition.y + tran.localPosition.y
		totalPosition.z = totalPosition.z + tran.localPosition.z
		
		totalRotation.x = totalRotation.x + tran.localRotation.x
		totalRotation.y = totalRotation.y + tran.localRotation.y
		totalRotation.z = totalRotation.z + tran.localRotation.z
		
		totalScale.x = totalScale.x * tran.localScale.x
		totalScale.y = totalScale.y * tran.localScale.y
		totalScale.z = totalScale.z * tran.localScale.z
	end

	local transform = {localPosition = totalPosition, localRotation = totalRotation, localScale = totalScale}
	return transform
end

function SwitchData(data1, data2)
	local tmp = data1
	data1 = data2
	data2 = tmp
	return data1, data2
end

-- 数字转换字符：num < 100
function NumToWord(num)
	local word = ""
	if not num or num < 0 or num >= 100  then return word end
	local head = GetIntPart(num/10)
	local tail = GetIntPart(num%10)
	if head > 0 then
		if head == 1 then
			word = "0"
		else
			word = head.."0"
		end
	end
	if tail > 0 then
		word = word..tail
	end
	return word
end

-- 数字转换字符：num < 100
function NumToChar(num)
	local chars = {"一", "二", "三", "四", "五", "六", "七", "八", "九", "十"}
	local word = ""
	if not num or num < 0 or num >= 100  then return word end
	local head = GetIntPart(num/10)
	local tail = GetIntPart(num%10)
	if head > 0 then
		if head == 1 then
			word = chars[10]
		else
			word = chars[head]..chars[10]
		end
	end
	if tail > 0 then
		word = word..chars[tail]
	end
	return word
end

function SortUIGrid(gridObj, sortFunc)
	if not gridObj then return end
	local tmpObj = GameObject.New()
	ResetObject(tmpObj, "TMP", gridObj)
--	local src = {}
	local dest = ArrayList.New()
	local list = GetComponent(gridObj.transform, "UIGrid"):GetChildList()
	for i = 0, list.Count - 1 do
		local tran = list[i]
		if tran.gameObject.activeSelf then
			HangObject(tran.gameObject, tmpObj)
--			table.insert(src, tran)
			dest:Add(tran)
		end
	end
--	local dest = Util.CreateArrayList(src)
	Util.OnCustomSort(dest, sortFunc)
	for i = 0, dest.Count - 1 do
		local tran = dest[i]
		HangObject(tran.gameObject, gridObj)
	end
	DestroyObject(tmpObj)
	ResetGrid(gridObj)
end

function SortFunc(list, compFunc)
	for i = 1, #list do
		for j = 1, #list - i do
			if compFunc(list[j], list[j + 1]) == true then
				local tmp = list[j]
				list[j] = list[j + 1]
				list[j + 1] = tmp
			end
		end
	end
end

function GetRowNum(listSize, minNum)
	return math.abs(GetIntPart(listSize/minNum + (listSize%minNum > 0 and 1 or 0)))
end