--[[
-- Desc: 接入相关
-- Author: kenny
-- Date: 2016/11/09
--]]

Access = {}
local this = Access

function Access.New(data)
	return this
end

--<summary>登录</summary>
function Access.DoLogin()
	if Global.isSDKLogin then
		SDKManager:DoLogin()
	else
		local ctrl = ModuleManager.GetCtrl(ModuleName.LOGIN)
		if ctrl and ctrl.view then
			ctrl.view:createLoginDialog()
		end
	end
end

--<summary>登出</summary>
function Access.DoLogout(isSDKCallback, forceAll)
	Network.Disconnect()
	ViewUtil.closeAllViews(true, forceAll)
	if Global.isSDKLogin then
		if not isSDKCallback then
			SDKManager:DoLogout()
		end
		DelayCall(function()
			SDKManager:DoInit()
			ViewUtil.openView(ModuleName.LOGIN)
		end, 0.6)
	else
		ViewUtil.openView(ModuleName.LOGIN)
	end
end

--支付购买
function Access.DoPay(gId, price, amount, gName, gDesc)
	if Global.isSDKLogin then
		local function sendToSDK (_orderId)
			if (-1 == _orderId) then
				ViewUtil.showMsg("订单生成失败！");
				return ;
			end
		local param = PayParams.New()
			param.OrderId = _orderId
			param.GoodsId = gId
			param.TotalPrice = price * amount
			param.GoodsName = gName
			param.GoodsDes = gDesc
			param.Currency = "血钻"
			param.UserId = SDKManager:GetUserId()
			param.CustomData = "自定义拓传参数"
			
			SDKManager:DoPay(param)
		end
		Network.GetBuyOrderId(price * amount, sendToSDK);
	end
end

--<summary>支付购买</summary>
-- function Access.DoPay(gId, price, amount, gName, gDesc)
-- 	if Global.isSDKLogin then
-- 		local param = PayParams.New()
-- 		param.OrderId = Util.CreateOrderId()
-- 		param.GoodsId = gId
-- 		param.TotalPrice = price * amount
-- 		param.GoodsName = gName
-- 		param.GoodsDes = gDesc
-- 		param.Currency = "血钻"
-- 		param.UserId = SDKManager:GetUserId()
-- 		param.CustomData = "自定义拓传参数"
		
-- 		this._currency = param.Currency
-- 		this._price = price
-- 		this._amount = amount
		
-- 		SDKManager:DoPay(param)
-- 	end
-- end

function Access.DoUpdate()
	if Application.isMobilePlatform then
		SDKManager:DoUpdate()
	end
end

function Access.DoExit()
	SDKManager:DoExit()
end

--<summary>SDK提交数据</summary>
--<param name="submitType">提交类型：
--	MangoGameData.ENTER_GAME = 1;
--	MangoGameData.CREATE_ROLE = 2;
--	MangoGameData.LEVEL_UP = 3;
--	MangoGameData.EXIT = 4;
--</param>
--<param name="timeStamp">时间戳</param>
function Access.submitData(submitType, timeStamp)
	if Global.isSDKLogin then
		timeStamp = timeStamp or Util.CreateTimeStamp()
		local data = MangoGameData.New()
		data.Type = submitType
		data.RoleId = PlayerData.GetRoleTeamId() or "0"
		data.RoleName = TeamData.GetRoleName() or "Default"
		data.RoleLevel = 0
		data.ZoneId = 1
		data.ZoneName = "全球同服"
		data.Balance = TeamData.GetGem() or 0
		data.VipLevel = 0
		data.PartyName = "空"
		data.RoleCreateTime = timeStamp
		data.RoleLevelChangeTime = timeStamp
		MangoSdk.SubmitData(data)
	end
end

--<summary>SDK登录结果</summary>
--<param name="ret">是否登录成功</param>
--<param name="userId">渠道提供的用户ID, 可能为空</param>
--<param name="token">FTX平台token</param>
--<param name="timeStamp">时间戳</param>
function Access.loginCallback(ret, packageId, token, uid, exInfo)
	if ret == true then
		PlayerData.setLoginData(packageId, token, uid, exInfo)
		ViewUtil.openView(ModuleName.ENTER)
		this.GetAnnouncementInfo()
		this.GetServerInfo()
	else
		print(exInfo)
	end
end

--<summary>SDK登录退出</summary>
function Access.logoutCallback()
	this.DoLogout(true, true)
end

function Access.payCallback(ret)
	if ret == true then
		plog("[Lua]支付成功")
		if Application.isMobilePlatform then -- UMeng
			GA.Buy(this._currency, this._price, this._amount)
		end
	else
		plog("[Lua]支付失败")
	end
end

function Access.exitCallback(code)
	if code == MangoSdk.CHANNEL_EXIT then
		this.submitData(MangoGameData.EXIT)
		MangoSdk.ReleaseResource()
		Application.Quit()
	else
		if not this.exitObj then
			local uiRoot = GameObject.Find("UIRoot")
			this.exitObj = CreateObjectByPath("UI/BoardExit", AssetBundleInfo.assetBundles)
			local shade = ViewUtil.CreateShadeView()
			HangObject(shade, this.exitObj)
			HangObject(this.exitObj, uiRoot)
			ViewUtil.sortLayerOrder(this.exitObj, LayerName.Top)
			SortUIPanelDepth(this.exitObj, 10000, false)
			SortUIWidgetDepth(this.exitObj, 10000, false)
			SetCameraLayer(this.exitObj, CameraLayer.GameTop, true)
			SetCameraLayer(shade, CameraLayer.GameTop, true)

			local content = GetPathComponent(this.exitObj.transform, "Panel/Content", "UILabel")
			local cancel = GetGameObject(this.exitObj.transform, "Panel/Cancel")
			local confirm = GetGameObject(this.exitObj.transform, "Panel/Confirm")
			content.text = "你确定要离我而去吗?"

			local function doClick(obj)
				if obj == confirm then
					Application.Quit()
				end
				DestroyObject(this.exitObj)
				this.exitObj = nil
			end
			UIEventListener.Get(cancel).onClick = doClick
			UIEventListener.Get(confirm).onClick = doClick
		 end
	end
end

--获取Enter界面需要选服的服务器信息 by iorinie
function Access.GetServerInfo()
	local uri = tostring(string.format("http://121.10.118.50:8080/xgame/server/select?type=0", nil, nil, "ftx", "Android"))
	RequestURL(uri, function(data, args)
	local info = cjson.decode(data)
	if IsEmpty(info) then
		ViewUtil.showMsg("无服务器信息!")
		return
	end
	local ctrl = ModuleManager.GetCtrl(ModuleName.ENTER)
	ctrl.data = info
	ctrl.Start()
	end)
end

--获取全服公告信息 by iorinie
function Access.GetAnnouncementInfo()
	local uri = tostring(string.format("http://121.10.118.50:8080/xgame/server/tips?type=0", nil, nil, "ftx", "Android"))
	RequestURL(uri, function(data, args)
	local info = cjson.decode(data)
	if IsEmpty(info) then
		ViewUtil.showMsg("无全服公告!")
		return
	end
	local ctrl = ModuleManager.GetCtrl(ModuleName.ENTER)
	ctrl.view:ShowAnnouncement(info)
	end)
end