--道具背包处理类
--2016-6-27 14:17:35 library
require "Game/Common/List"

local const_propName = "prop_name";
local const_propShowType = "prop_type";
local const_propDesc = "description";

local const_canUseType = ",2,4,10,";--可以使用的类型
local const_propUse = ",2,4,10,";--放入道具包裹
local const_heroUse = ",1,";--放入英雄培养
local const_canGetHerosPropIds = ",1007,1008,1009,1028,1029,1030,1031,";--可以获得英雄的道具串

PropDataProxy = {
    myProps = List.new();--List<PropData>
}

local this = PropDataProxy;

--[[
    取我的道具表
]]
function PropDataProxy.getMyProps ()
    return this.myProps;
end

--[[
    设置我的背包
]]
function PropDataProxy.setMyProps (_data)
    this.myProps:clear();
    for i = 1, #_data do
        this.myProps:add(_data[i]);
    end
end

--[[
    _id number 道具的唯一ID
    return number 道具的数量
]]
function PropDataProxy.getPropNumById (_id)
    local pNum = 0;

    _id = _id or 0;

    for i = 1, #this.myProps do
        if (tonumber(_id) == tonumber(this.myProps[i].id)) then
            pNum = this.myProps[i].num;
            break ;
        end
    end

    return pNum;
end

--[[
    _id number 道具的配置ID
    return number 道具的数量
]]
function PropDataProxy.getPropNumByPropId (_propId)
    local pNum = 0;

    _id = _id or 0;

    for i = 1, #this.myProps do
        if (tonumber(_propId) == tonumber(this.myProps[i].propId)) then
            pNum = pNum + this.myProps[i].num;
        end
    end

    return pNum;
end

--[[
    取指定类型的道具
    _tp number 0，所有类型；1，道具；1，英雄培养
    return List<PropData>
]]
function PropDataProxy.getMyPropsByType (_tp)
    local propList, pData, pCfg = List.new(), nil, nil;
    local tpStr = "";
    
    if (1 == _tp) then
        tpStr = const_propUse;
    elseif (2 == _tp) then
        tpStr = const_heroUse;
    end

    for i = 1, this.myProps:getSize() do
        pData = this.myProps[i];
        
        if (0 == _tp) then
            if (pData.num > 0) then
                propList:add(pData);
            end
        else
            pCfg = PropDataProxy.getPropCfgById(pData.propId);
            if (nil ~= pCfg and (nil ~= string.find(tpStr, "," .. pCfg[const_propShowType] .. ",")) and pData.num > 0) then
                propList:add(pData);
            end
        end
    end

    return propList;
end

--[[
    根据道具ID取道具配置数据
    _pId 道具ID
    return 道具配置数据
]]
function PropDataProxy.getPropCfgById (_pId)
    local pCfg, pCfgList = nil, Config.getPropsData();

    for k, v in pairs(pCfgList) do
        if (tonumber(k) == tonumber(_pId)) then
            pCfg = v;
            break ;
        end
    end

    return pCfg;
end

--[[
    从配置表中取道具名称
    _cfg 道具配置数据
    return string 道具名称
]]
function PropDataProxy.getPropName (_cfg)
    local pName = "";

    if (nil ~= _cfg) then
        pName = _cfg[const_propName];
    end

    pName = pName or "";

    return pName;
end

--[[
    从道具配置表中取出道具描述
    _cfg 道具配置数据
    return string 道具描述
]]
function PropDataProxy.getPropDesc (_cfg)
    local pDesc = "";

    if (nil ~= _cfg) then
        pDesc = _cfg[const_propDesc];
    end

    pDesc = pDesc or "";

    return pDesc;
end

--[[
    背包变动
    _data PushPropData
    return nil
]]
function PropDataProxy.propChge (_data)
    if ("ADD" == _data.status) then
        this.addProps(_data.propList);
    elseif ("DEL" == _data.status) then
        this.removeProps(_data.propList);
    end
end

--[[
    添加道具
    _list 需要添加的道具
    return nil
]]
function PropDataProxy.addProps (_list)
    local rPData, mPData, isFind, idx = nil, nil, false, 0;
    
    for i = 1, #_list do
        rPData = _list[i];
        
        isFind = false;
        for j = 1, this.myProps:getSize() do
            mPData = this.myProps[j];
            if (rPData.id == mPData.id and rPData.propId == mPData.propId) then
                idx = j;
                isFind = true;
                break ;
            end
        end
        
        if (isFind) then
            this.myProps[idx].num = rPData.num;
            this.myProps[idx].endTime = rPData.endTime;
        else
            this.myProps:add(rPData);
        end
    end
end

--[[
    移除道具
    _list 需要移除的道具列表
    return nil
]]
function PropDataProxy.removeProps (_list)
    local rPData, mPData, idx = nil, nil, 0;

    for i = 1, #_list do
        rPData = _list[i];
        
        for j = 1, this.myProps:getSize() do
            mPData = this.myProps[j];
            if (rPData.id == mPData.id and rPData.propId == mPData.propId) then
                this.myProps[j].num = rPData.num;
                this.myProps[j].endTime = rPData.endTime;
                if (0 >= rPData.num) then
                    this.myProps:removeAt(j);
                end
                break ;
            end
        end
    end
end

--[[
    取道具类型
    _cfg PropCfg 道具配置数据
    return prop_type
]]
function PropDataProxy.getPropType (_cfg)
    local tp = 0;

    if (nil ~= _cfg) then
        tp = _cfg[const_propShowType];
    end

    tp = tp or 0;

    return tp;
end

--[[
    判断道具是否可以使用
    _id number 道具ID
    _tp number 道具类型
    return boolean
]]
function PropDataProxy.isCanUse (_id, _tp)
    local canUse = false;
    local cfg = this.getPropCfgById(_id);

    if (nil == _tp) then
        _tp = this.getPropType(cfg);
    end

    if (nil ~= _tp) then
        canUse = (nil ~= string.find(const_canUseType, "," .. _tp .. ","));
    end

    return canUse;
end

--[[
    获取可以获得英雄的道具ID串
    return string 道具ID串，格式 ,1007,1008,
]]
function PropDataProxy.getAddHeroPropIds ()
    local str = "";

    str = const_canGetHerosPropIds;

    return str;
end

--[[
    清理过期道具
    return nil
]]
function PropDataProxy.clearOutOfDateProps ()
    local pData = nil;--PropData
    local curTime = PlayerData.getCurSysTime();

    for i = this.myProps.size, 1, -1 do
        pData = this.myProps[i];
        if (pData.endTime and pData.endTime > 0 and curTime >= pData.endTime) then
            this.myProps:removeAt(i);
        end
    end
end

--[[
    根据道具唯一ID取道具数据
    _id number 道具的唯一ID
    return PropData
]]
function PropDataProxy.getMyPropById (_id)
    local pData = nil;

    for i = 1, this.myProps.size do
        pData = this.myProps[i];
        if (_id == pData.id) then
            break ;
        else
            pData = nil;
        end
    end

    return pData;
end