--定义List
List = class(BaseObject)

function List:ctor()
	local object = {}
	setmetatable(object, self)
	self.__index = self
	self.__classtype = "List"
	self.size = 0  -- 成员计数
end

function List:sort(func)
	table.sort(self, func)
end

function List:find(func) 
	for i = 1, #self do
		if func(self[i]) then
			return self[i];
		end
	end
end

function List:findAll(func) 
	local _values = List.new();
	for i = 1, #self do
		if func(self[i]) then
			_values:add(self[i]);
		end
	end				
	return _values;
end

function List:findFront(value)
	for i = 1, #self do
		if self[i] == value then
			return i
		end
	end
	return -1
end

function List:findBack(value)
	for i = #self, 1, -1 do
		if self[i] == value then
			return i
		end
	end
	return -1
end

function List:add(value)
	self.size = self.size + 1
	self[self.size] = value
end

function List:addByIndex(index, value)
	if IsEmpty(self[index]) then
		self.size = self.size + 1
	end
	self[index] = value
end

function List:values()
	local all = {}
	for i, v in pairs(self) do
		if IsNumber(i) then
			table.insert(all, v)
		end
	end
	return all
end

function List:addRange(addList)
	if(addList==nil) then
		return;
	end
	for i = 1, #addList do
		self:add(addList[i]);
	end
end

function List:getRange(index , count)
	if index > #self or index < 1 or count > #self then
		return nil;
	end

	local _values = List.new();

	local j = 1;

	for i = index, count do
		_values[j] = self[i];
		j = j+1;
	end	

	return _values;
end

function List:reverse() 
	local len = self:getSize();
	local lenLoop;

	if len % 2 == 0 then
		lenLoop = len / 2;
	else 
		lenLoop = GetIntPart(len / 2);
	end		

	for i = 1 , lenLoop do
		local tmp = self[len - i + 1];
		self[len - i + 1] = self[i];
		self[i] = tmp;
	end
end

function List:contains(value) 
	for i=1, #self do
		if self[i] == value then
			return true;
		end
	end
	return false;
end

function List:indexOf(index)
	return self[index]
end

function List:front()
	return self[1]
end

function List:back()
	return self[self.size]
end

function List:getSize()
    return self.size
end

function List:clear()
	for i = 1, #self do
		self[i] = nil
	end
	self.size = 0
end

--替换
function List:replace(index,newValue)
	self[index] = newValue
end 

function List:remove(value, formBack)
	local index = nil
	if formBack then
		index = self:findBack(value)
	else
		index = self:findFront(value)
	end
	if index ~= nil and index ~= -1 then
		self:removeAt(index)
	end	
end	

--删除
function List:removeAt(index)
	if not index or index < 1 or index >  self.size then
--		error("List error1: removeAt index:"..index..",self.size:"..self.size)
		return
	end
	for i = index, self.size do
		if index+1 <= self.size then
			self[i] = self[i+1]
		end
	end
	self[self.size] = nil
	self.size = self.size - 1
	if #self ~= self.size  then
		error("List error2: removeAt index:"..index..",self.size:"..self.size)
	end
end

function List:toString()
	local str="[List size:"..#self;
	for i=1,#self do
		if(type(self[i])~="table" and type(self[i])~="userdata") then
			str=str.."self["..i.."]="..self[i].." ";
		end
	end
	str=str.."]"
	return str
end

function List:pop(index)
	local ret = self:indexOf(index)
	self:removeAt(index)
	return ret
end
