require "Game/Common/Map"

Cookie = { 

};

local this = Cookie;

Cookie.diskCacheMap = Map.new();

Cookie.prefsCacheMap = Map.new();




--时间标识后缀
local TIME = "_T";

--内存缓存,减少IO
local cacheMemory = false;
 
--[[
/// 保存基础类型数据在本地
/// </summary>
/// <param name="key">键名</param>
/// <param name="value">数据值</param>
/// <param name="expireTime">到期时间(毫秒,小于１永久</param>
/// <param name="value">encod,字符串的编码</param>
]]
function Cookie.set(key, value, expireTime, encod)

	expireTime = expireTime or 0;

	encod = encod or nil;

	this.prefsCacheMap:put(key , WWW.EscapeURL(value, encod == nil and Encoding.UTF8 or encod));

    if expireTime > 0 then 
    	this.setExpireTime(key, expireTime);
    end	
end

--[[ 
/// 从本地读取基础类型数据
/// </summary>
/// <param name="key">键名</param>
/// <param name="defaultValue">默认数据值</param>
/// <param name="expireValue">是否返回过期数据</param>
/// <param name="now">当前时间(毫秒</param>
/// <param name="value">encod,当前字符串的编码</param>
]]
function Cookie.get(key, defaultValue, expireValue, now, encod)

	defaultValue = defaultValue or nil;
	expireValue = expireValue or false;
	now = now or 0;
	encod = encod or nil;

     
    if expireValue ~= nil and now > 0 then
    	if this.isExpire(key, now) then
    		return defaultValue;
    	end	
    end

    local rsValue = nil;

    if this.prefsCacheMap:containsKey(key) then
    	rsValue = this.prefsCacheMap[key];	
    else 
    	rsValue = defaultValue;
    end	

	return WWW.UnEscapeURL(rsValue,encod == nil and Encoding.UTF8 or encod);
end

--删除基础类型缓存
function Cookie.deleteString(key)
	
	this.prefsCacheMap:remove(key);

end

--判断数据key是否过期,now当前时间(毫秒
function Cookie.isExpire(key, now)

    local expireTime = this.getExpireTime(key);

    return (expireTime > 0) and (now > expireTime);
end

--键key数据的过期时间
function Cookie.getExpireTime(key)

    return tonumber(this.get(key .. this.TIME, "0"));
end

--保存数据过期时间
function Cookie.setExpireTime(key, time)

    this.set(key .. TIME, tostring(time));
end

--[[
/// 保存用户数据到文件
/// </summary>
/// <param name="key">键名</param>
/// <param name="data">数据</param>
/// <param name="expireTime">到期时间(毫秒,小于１永久</param>
/// <returns>成功或失败</returns>
]]

function Cookie.saveData(key, data, expireTime)

	expireTime = expireTime or 0;

    if expireTime > 0 then
    	this.setExpireTime(key, expireTime);
    end	

    this.diskCacheMap:put(key , data);

    return true;
end

--[[
/// 获取用户数据从文件
/// </summary>
/// <param name="key">键名</param>
/// <param name="expireValue">是否返回过期数据</param>
/// <param name="now">当前时间(毫秒</param>
]]
function Cookie.getData(key, expireValue, now)

	expireValue = expireValue or false;

	now = now or 0;

    if not expireValue and now > 0 then
    	if this.isExpire(key, now) then
    		return nil;
    	end
    end		

    return this.diskCacheMap[key];
end

--删除对象缓存
function Cookie.deleteData(key)
	this.diskCacheMap:remove(key);
end

-- 清所有缓存
function Cookie.clearCache()
	this.diskCacheMap:clear();
end