--逻辑工具类
--2016-7-4 10:16:50 library

LogicUtil = {};


local this = LogicUtil;

--[[
    将公用英雄选择排序面板的显示类型转换为英雄的品质Quality
    _showType num 0，显示全部；1，闪金；2，金色；3，紫色；4，蓝色；5，绿色；6，白色
    return num
]]
function LogicUtil.showTypeToQua (_showType)
    local qua = 0;

    if (0 == _showType) then
        qua = 0;
    elseif (1 == _showType) then
        qua = 6;
    elseif (2 == _showType) then
        qua = 5;
    elseif (3 == _showType) then
        qua = 4;
    elseif (4 == _showType) then
        qua = 3;
    elseif (5 == _showType) then
        qua = 2;
    elseif (6 == _showType) then
        qua = 1;
    end

    return qua;
end

--[[
    将英雄的品质Quality转换为公用英雄选择排序面板的显示类型
    _qua num 0，所有品质；1，白色；2，绿色；3，蓝色；4，紫色；5，金色；6，闪金
    return num
]]
function LogicUtil.quaToShowType (_qua)
    local showType = 0;

    if (0 == _qua) then
        showType = 0;
    elseif (1 == _qua) then
        showType = 6;
    elseif (2 == _qua) then
        showType = 5;
    elseif (3 == _qua) then
        showType = 4;
    elseif (4 == _qua) then
        showType = 3;
    elseif (5 == _qua) then
        showType = 2;
    elseif (6 == _qua) then
        showType = 1;
    end

    return showType;
end

--[[
    将显示的数据格式化为k、m、b、t，对应thousend、million、billion、Trillion
    _number number 
    return string 对应的串
]]
function LogicUtil.getShowNum (_number)
    local numStr, base, suffix = "", 1, "";
    
    _number = tonumber(_number) or 0;

    if (_number >= 1000) then
        base = 1000;
        suffix = "K";
    end
    
    if (_number >= 1000000) then 
        base = 1000000;
        suffix = "M";
    end
    
    if (_number >= 1000000000) then
        base = 1000000000;
        suffix = "B";
    end
    
    if (_number >= 1000000000000) then
        base = 1000000000000;
        suffix = "T";
    end
    
    numStr = _number / base;
--    numStr = string.format("%.2f", numStr); -- 取消四舍五入，否则会导致副本章节开启问题

    if (string.find(numStr, "%p")) then
	    numStr = string.sub(numStr, 1, string.find(numStr, "%p") + 2);
        if ("0" == string.sub(numStr, -1, -1)) then
            numStr = string.sub(numStr, 1, -2);
            if ("0" == string.sub(numStr, -1, -1)) then
                numStr = string.sub(numStr, 1, -3);
            end
        end
    end
    numStr = numStr .. suffix;
    return numStr;
end

--[[
    解析配置串
    分隔符层级：{":", ",", "|", "?"}
    _str string 配置串
    return table, layer 配置表，层级
]]
function LogicUtil.splitStrCfgs (_str)
    local cfgTab = nil;
    local delimiters = {":", ",", "|", "?"};
    local delimiter = "";
    local layer = 0;

    _str = _str or "";

    for i = 1, #delimiters do
        if (string.find(_str, delimiters[i])) then
            layer = i;
        end
    end
    
    local function _delimit (_str, _layer) 
        local _delimiter = delimiters[_layer];
        local _tab, rsltTab = nil, nil;
        
        _layer = _layer - 1;

        if (_layer >= 0) then
            rsltTab = {};
            _tab = StrSplit(_str, _delimiter);
            for i = 1, #_tab do
                local tmpStr = nil;
                
                tmpStr = _tab[i];
                if ("" ~= Trim(tmpStr)) then
                    table.insert(rsltTab, tmpStr);
                    rsltTab[#rsltTab] = _delimit(tmpStr, _layer);
                end
            end
        else
            rsltTab = _str;
        end
        
        return rsltTab;
    end

    cfgTab = _delimit(_str, layer);
    
    return cfgTab, layer;
end
--[[
    解析配置串
    分隔符层级：{",", ":", ";", "="}
    _str string 配置串
    return table, layer 配置表，层级
]]
function LogicUtil.nSplitStrCfgs (_str)
    local cfgTab = nil;
    local delimiters = {",", ":", ";", "="};
    local delimiter = "";
    local layer = 0;

    _str = _str or "";

    for i = 1, #delimiters do
        if (string.find(_str, delimiters[i])) then
            layer = i;
        end
    end

    local function _delimit (_str, _idx) 
        local _delimiter = delimiters[_idx];
        local _tab = nil;
    	local rsltTab = {};
        
        if (_idx <= layer) then
        	_tab = StrSplit(_str, _delimiter);
    		_idx = _idx + 1;
            for i = 1, #_tab do
                local tmpStr = nil;
                
                tmpStr = _tab[i];
                if ("" ~= Trim(tmpStr)) then
                    table.insert(rsltTab, tmpStr);
                    rsltTab[#rsltTab] = _delimit(tmpStr, _idx);
                end
            end
        else
        	rsltTab = _str;
        end
        
        return rsltTab;
    end
	
    cfgTab = _delimit(_str, 1);
    
    return cfgTab, layer;
end

--[[
    压平表，将表压至仅剩2层
    _tarTab table 压平后的表，如{{{{1001,1},{1002,2}},{{1001,1},{1002,2}}},{{{1001,1},{1002,2}},{{1001,1},{1002,2}}}}
    _srcTab table 被压平的表，{{1001,1},{1002,2},{1001,1},{1002,2},{1001,1},{1002,2},{1001,1},{1002,2}}
    _layer number 需要压平的层级
]]
function LogicUtil.planishRwd (_tarTab, _srcTab, _layer)
    local tmpTab = {};

    _layer = _layer - 1;
    if (_layer >= 1) then
        for i = 1, #_srcTab do
            tmpTab = _srcTab[i];
            if (1 == _layer) then
                table.insert(_tarTab, tmpTab);
            else
                LogicUtil.planishRwd(_tarTab, tmpTab, _layer);
            end
        end
    end
end

--[[
    _time number 计时时间，毫秒
    return string 时间串xx:xx:xx
]]
function LogicUtil.getCountTimeStr (_time)
    local timeStr = "";
    local hour, minute, second = 0, 0, 0;

    _time = tonumber(_time) or 0;
    _time = GetIntPart(_time / 1000);

    hour = GetIntPart(_time / 3600);
    minute = GetIntPart((_time - hour * 3600) / 60);
    second = _time % 60;

    if (0 <= hour and hour < 10) then
        timeStr = "0" .. hour .. ":";
    else
        timeStr = hour .. ":";
    end

    if (0 <= minute and minute < 10) then
        timeStr = timeStr .. "0" .. minute .. ":";
    else
        timeStr = timeStr .. minute .. ":";
    end

    if (0 <= second and second < 10) then
        timeStr = timeStr .. "0" .. second;
    else
        timeStr = timeStr .. second;
    end

    return timeStr;
end

--[[

]]
function LogicUtil.getJsonConfigVal (_jsonCfg, _key)
    local val = nil;

    for k, v in pairs(_jsonCfg) do
        if (tonumber(k) == tonumber(_key)) then
            val = v;
            break ;
        end
    end

    return val;
end