
ViewManager = {
};

local this = ViewManager;
this.VIEW_BASE = "base";
this.VIEW_MODULE = "module";
this.VIEW_POP = "pop";
this.VIEW_BACKGROUND = "backGround";
this.VIEW_TOP = "top";
this.VIEW_NOTICEPANE = "noticePane";
this.baseLayer = nil;
this.moduleLayer = nil;
this.popLayer = nil;
this.backGround = nil;
this.topLayer = nil;
this.noticePane = nil;
this.currentPanelDepth = 0;
this.currentScene = nil;
this.lastScene = nil;
this.changeing = false;			-- 切换中
this.LoadingSence = nil;			-- 加载场景状态对象
this.loadingView = nil;				-- 当前加载的模块
this.loadingViewABS = nil;
this.viewStack = List.new();
this.viewQueue = List.new();
this.navigation = List.new();
--this.openSoundnot = List.new();
this.root = nil;
this.uiScale = 0;
this.realWidth = 0;
this.realHeight = 0;
this.lastModule = nil
this.currentModule = nil
this.previousModule = nil
this.topModule = nil
this.maxOrder = 0
this.maxDepth = 0


-- 常在层(不关闭之前视图)
ViewManager.constViews = {
	ModuleName.COM,
    ModuleName.GUIDE
}

-- 广播层
ViewManager.broadcastViews = {
	ModuleName.COM,
	ModuleName.FAIL,
	ModuleName.WIN,
}

-- 置顶层, 不会自动回收, 需手动关闭
ViewManager.topViews = {
	ModuleName.LOAD,
	ModuleName.GM,
    ModuleName.GUIDE
}

-- 不再返回的视图
ViewManager.notBackViews = {
	ModuleName.COM,
	ModuleName.LOAD,
	ModuleName.FAIL,
	ModuleName.WIN,
	ModuleName.FIGHT,
	ModuleName.GM,
	ModuleName.SR,
	ModuleName.HP,
    ModuleName.GUIDE,
    ModuleName.SC,
    ModuleName.PROP,
    ModuleName.FriendNotice,
    ModuleName.MONTHCARD,
}

-- 忽略层级(不参与层级排序)
ViewManager.ignoreLayerViews = {
	ModuleName.FIGHT,
}

-- 不带COM模块的视图
ViewManager.withoutComViews = {
	ModuleName.COM,
	ModuleName.LOAD,
	ModuleName.FAIL,
	ModuleName.WIN,
	ModuleName.FIGHT,
	ModuleName.GM,
	ModuleName.LOGIN,
	ModuleName.REG,
	ModuleName.ENTER,
	ModuleName.SR,
	ModuleName.SC,
    ModuleName.PROP,
    ModuleName.MONTHCARD,
	ModuleName.SETTING,
	ModuleName.FriendNotice,
}


-- 初始化
function ViewManager.init()
	this.root = ViewUtil.getRootView()
	this.uiScale = this.root.activeHeight / Screen.height;
 	this.realWidth = Screen.width * this.uiScale + 0.5;
	this.realHeight = Screen.height * this.uiScale + 0.5;
end

function ViewManager.isEmptyViewList()
	if this.viewQueue and this.viewQueue.size > 0 then
		return false
	end
	return true
end

function ViewManager.isExistView(name)
	if not name then return false end
	for i = this.viewQueue.size, 1, -1 do
    	if this.viewQueue[i].name == name then
    		return true
    	end
    end
    return false
end

-- 打开视图
function ViewManager.OpenView(moduleName, keepCurView, callback, params)
	if not IsMember(moduleName, ViewManager.withoutComViews) then
		this.OpenModule(moduleName, false, true, function(openParam)
			this.OpenModule(ModuleName.COM, false, true, function(openParam)
				if callback ~= nil then
					callback(openParam)
				end
			end, true, true, {}, true)
		end, true, not keepCurView, params, true)
	else
		this.OpenModule(moduleName, false, true, callback or function(openParam) end, true, not keepCurView, params, true)
	end
end

function ViewManager.Goto(moduleName, keepCurView, callback, params)
	this.OpenView(moduleName, keepCurView, callback, params)

--	if Global.isLocal then
--		this.OpenView(moduleName, keepCurView, callback, params)
--	else
--		if moduleName == this.currentModule then
--			if callback ~= nil then
--				callback(params)
--			end
--		else
--			this.OpenView(moduleName, keepCurView, function()
--				this.OpenView(ModuleName.LOAD, true, callback, params)
--			end)
--		end
--	end
end

--[[
/// <summary> 
/// 打开模块
/// </summary>
/// <param name="name">模块名</param>
/// <param name="hasScene">是否有场景</param>
/// <param name="async">是否异步实例化内容</param>
/// <param name="callBack">打开模块后回调</param>
/// <param name="cloneEable">是否复制新的主预设到场景</param>
/// <param name="closeCurView">是否关闭当前模块</param>
/// <param name="openParam">打开模块后回调参数</param>
/// <param name="clearStack">是否清理模块的返回堆栈,禁用模块返回</param>
--]]
function ViewManager.OpenModule(name, hasScene, async, callBack, cloneEable, closeCurView, openParam, clearStack)
--	if not this.openSoundnot:contains(name) then
--		SoundManager:playClickEffect("interface01")
--	end
	if not IsMember(name, ModuleName) then
		return
	end
    
	if IsMember(name, this.constViews) then
		closeCurView = false
	end
    
	if this.changeing then
		return
	end

--	if hasScene and (name == this.currentScene) then
--		return
--	end
	
	local isExist = false
	if this.isExistView(name) then
		isExist = true
	end

	local vd = this.getViewData(name, hasScene, async, callBack, cloneEable, closeCurView, openParam)
	vd.pushBack = true
	
	if this.topModule and vd.name ~= this.topModule then
		if not IsMember(vd.name, ViewManager.constViews) then
			local ctrl = ModuleManager.GetCtrl(this.topModule)
			if ctrl and ctrl.view and ctrl.view.LostFocus ~= nil then
				ctrl.view:LostFocus()
			end
		end
	end

	if isExist then
		plog("It's a exist view: ".. tostring(name))
		this.opened(vd, true)
		return
	end

	if clearStack then
		this.clearViewStack()
	end	

	if hasScene then
		this.viewStack:clear()
		this.clearViewQueue()
		AssetManager:changeScene()
	end

	this._OpenModul(vd)
end

function ViewManager.OpenCurModule(callback, params)
	if this.currentModule ~= nil then
		this.Goto(this.currentModule, false, callback, params)
	end
end

function ViewManager.OpenLastModule(callback, params)
	this.lastModule = ViewManager.GetLastModule(this.currentModule)
	if this.lastModule ~= nil then
		this.Goto(this.lastModule, false, callback, params)
	end
end

--[[
/// <summary>
/// 关闭模块
/// </summary>
/// <param name="name"></param>
--]]
function ViewManager.CloseModule(name, openLast)
	if not name then return end
	openLast = openLast and true or false
	local viewQueue = this.viewQueue
	local viewStack = this.viewStack
	local ccv = nil
	for i = 1, viewQueue:getSize() do
		local cv = viewQueue[i]
		if cv.name ~= nil and name == cv.name then
			ccv = cv
			break
		end
	end
	plog("CloseModule: name=" .. name .. ", OpentLast=" .. tostring(openLast) .. ", NotFound=" .. tostring(ccv==nil))

	if ccv ~= nil then
		viewQueue:remove(ccv)
		AssetManager:DestroyModule(name)
		ccv = nil
		
		if openLast then
			ViewUtil.openView(this.lastNavigation(name))
		else
			local backView = this.viewQueue:back() -- 不一定为LastView
			if backView ~= nil then
				local name = backView.name
				ViewUtil.openView(name)
			end
		end
	end
end

-- 关闭当前模块
function ViewManager.CloseTopModule(openLast)
	openLast = openLast and true or false
	this.CloseModule(this.topModule, openLast)
end

-- 关闭非当前模块
function ViewManager.CloseOthersModules(name)
	if clearStack then
		this.clearViewStack()
	end
	local viewQueue = this.viewQueue
	for i = viewQueue:getSize(), 1, -1 do
		local cv = viewQueue[i]
		if not IsMember(cv.name, this.topViews) then
			if name ~= cv.name then
				viewQueue:remove(cv)
				AssetManager:DestroyModule(cv.name)
			end
		end
	end
	
	this.maxOrder = 0
	this.maxDepth = 0
end

-- 关闭所有模块
function ViewManager.CloseAllModules(clearStack, forceAll)
	if clearStack then
		this.clearViewStack()
	end
	local viewQueue = this.viewQueue
	for i = viewQueue:getSize(), 1, -1 do
		local cv = viewQueue[i]
		if not IsMember(cv.name, this.topViews) or forceAll then
			viewQueue:remove(cv)
			AssetManager:DestroyModule(cv.name)
		end
	end
	
	this.maxOrder = 0
	this.maxDepth = 0
end

function ViewManager.OpenModuleNoSceneNoCallbask(name, clearStack)
    if this.changeing then
    	return;
    end

    if name == this.currentScene or name == this.currentModule then
    	return;
    end

    local vd = this.getViewData(name, false, true, null, false, false, nil);

    vd.pushBack = true;

    if clearStack then
		this.clearViewStack();
    end

    this._OpenModul(vd);
end

function ViewManager._OpenModul(vd)
	this.changeing = true;
	local bm = AssetManager:getOftenModule(vd.name);
	if bm ~= nil then
		bm:showModule(false);
		this.opened(vd);
		return;
	end
    AssetManager:loadModule(vd.name, this.onLoadedModule, vd);
end

function ViewManager.GetLastModule(modName)
	return this.lastNavigation(modName)
end

function ViewManager.GetCurrentModule()
	return this.currentModule
end
function ViewManager.SetCurrentModule(name)
	if name ~= nil and name ~= this.currentModule then
		if not IsMember(this.currentModule, this.constViews) then
			this.previousModule = this.currentModule
		end
		this.currentModule = name
	end
end

function ViewManager.GetTopModule()
	return this.topModule
end
function ViewManager.SetTopModule(name)
	if not IsMember(name, this.notBackViews) then
		this.topModule = name
	end
end

function ViewManager.lastNavigation(mod)
	local value = nil
	local idx = this.navigation:findBack(mod)
	if idx == -1 then
		for i = this.navigation.size, 1, -1 do
			value = this.navigation:indexOf(i)
			if value ~= nil
			and value ~= mod
			and not IsMember(value, this.notBackViews)
			then
				break
			end
		end
	else
		for i = idx - 1, 1, -1 do
			value = this.navigation:indexOf(i)
			if value ~= nil
			and value ~= mod
			and not IsMember(value, this.notBackViews)
			then
				break
			end
		end
	end
	return value
end
function ViewManager.pushNavigation(modName)
	if not IsMember(modName, this.notBackViews) then
		this.navigation:add(modName)
	end
end

function ViewManager.opened(vd, isExist)
	local ctrl = ModuleManager.GetCtrl(vd.name)
	if ctrl then
		if vd.name ~= this.topModule then
--		plog("#vd.name = %s, this.topModule = %s, vd.closeCurView = %s", vd.name, this.topModule, tostring(vd.closeCurView))
			if vd.closeCurView then
				this.CloseTopModule(false) -- 手动维护
--				this.CloseOthersModules(vd.name) -- 会误删
			end
		end
		this.pushNavigation(vd.name)
	end

	if vd.hasScene then
		this.lastScene = this.currentScene
		this.currentScene = vd.name
	end
	this.changeing = false

	if not isExist then
		this.viewQueue:add(vd)
	end

--	plog("@PreviousModule = %s,  CurrentModule = %s", tostring(this.previousModule), tostring(vd.name))
	this.SortingModuleView(ctrl.gameObject, vd.name)
	this.SetCurrentModule(vd.name)
	this.SetTopModule(vd.name)

	-- 不能提前，否则GetGameObject为空
	if ctrl.view and ctrl.view.OnFocus ~= nil then
--		plog("@Ctrl.name = %s, Ctrl.view = %s", tostring(ctrl.view.name), tostring(ctrl.view))
		ctrl.view:OnFocus(this.previousModule) -- 保持在Update前执行
	end

--	if vd.pushBack then
--		this.viewStack:add(vd:copyTo(this.getViewData()))
--	end
	if vd.openCallback ~= nil then -- 注意"."和":"
		vd.openCallback(vd.openParam)
	end

	-- 其他非视图管理逻辑
--    ViewUtil.showNewHeros();
end

function ViewManager.onLoadedModule(csPanel, ds, vd)
    if vd.hasScene then
		this.loadingView = vd;
		this.loadingViewABS = ds;
		if vd.async then
			this.LoadingSence = Application.LoadLevelAsync(vd.name);
		else
			Application.LoadLevel(vd.name);
			--InitMainAsset(datas, vd);//还没有实体
			this.addLuaBehaviour(vd.name);
		end
	else
		this.InitMainAsset(ds, vd);
	end
end

function ViewManager.addLuaBehaviour(name) 
	local _name = name .. "Main";
    csPanel = GameObject.Find(_name); --在根路径中寻找
    if csPanel == nil then
    	csPanel = GameObject.Find(_name .. "(Clone)"); 
    end

    if csPanel == nil then
    	csPanel = GameObject.Find("UIRoot/" .. _name .. "(Clone)"); 
    end

    if csPanel ~= nil then
        local lb = csPanel:AddComponent(LuaBehaviour.GetClassType());
        lb:OnInit(nil,nil);
        lb.luaEventAccepterName = name .. "Controler";
    end
end

function ViewManager.checkLoadScene()
	if this.loadingView ~= nil then
		if this.LoadingSence ~= nil then
			if not this.LoadingSence.isDone then
                return;
			end
		end
		this.addLuaBehaviour(this.loadingView.name);
        local mainAsset = this.checkMainAsset(this.loadingViewABS, this.loadingView);
        if mainAsset ~= nil then
            this.changeScene(mainAsset);
        end
    end
end

function ViewManager.changeScene(mainAsset)
	this.InitMainAsset(this.loadingViewABS, this.loadingView, mainAsset);
end

function ViewManager.clearChangeState()
	this.LoadingSence = nil;
	this.loadingViewABS = nil;
	this.loadingView = nil;
end

function ViewManager.initSceneLayer(mainAsset)
	local tf = mainAsset.transform:Find(ViewManager.VIEW_BASE);
	if tf ~= nil then
		this.baseLayer = tf.gameObject;
	end

	tf = mainAsset.transform:Find(ViewManager.VIEW_MODULE);
	if tf ~= nil then
		this.moduleLayer = tf.gameObject;
	end
		
	tf = mainAsset.transform:Find(ViewManager.VIEW_POP);
	if tf ~= nil then
		this.popLayer = tf.gameObject;
	end	

	tf = mainAsset.transform:FindChild(ViewManager.VIEW_BACKGROUND);
	if tf ~= nil then
		this.backGround = tf.gameObject;
	end

	tf = mainAsset.transform:FindChild(ViewManager.VIEW_TOP);
	if tf ~= nil then
		this.topLayer = tf.gameObject;
	end
		
	tf = mainAsset.transform:FindChild(ViewManager.VIEW_NOTICEPANE);
	if tf ~= nil then
		this.noticePane = tf.gameObject;
	end
end

-- 获得球员动画时主页可见信息
function ViewManager.setGetPlayerAnimationMainShow(show)
	this.backGround:SetActive (show);
	this.moduleLayer:SetActive (show);
	this.topLayer:SetActive (show);
	this.noticePane:SetActive (show);
end

function ViewManager.checkMainAsset(ws, vd)
	local mainAsset = nil;
	if ws ~= nil and ws.Count > 0 then
		mainAsset = AssetManager:getMainFromBundles(ws, vd.name);
		if mainAsset ~= nil then
            --if (!Global.isRelease) Logger.setTimeGap("checkMainAsset.start");
            mainAsset = CreateObject(mainAsset);
           -- if (!Global.isRelease) Logger.setTimeGap("checkMainAsset.end");
        end
	end
 
	if mainAsset == nil then
		mainAsset = AssetManager:getMainScene(vd.name);
       -- Logger.Debug("checkMainAsset.getMainScene,mainAsset=" + mainAsset+",module="+
            --GameObject.Find(vd.name + "Main") + ",hasScene=" + vd.hasScene);
 
		if mainAsset == nil then
			mainAsset = AssetManager:getMainFromAssets(vd.name);
	 
			if mainAsset ~= nil then
                --if (!Global.isRelease) Logger.setTimeGap("checkMainAsset.start");
                mainAsset = CreateObject(mainAsset);
                --if (!Global.isRelease) Logger.setTimeGap("checkMainAsset.end");
            end
		end
	end
	return mainAsset;
end

function ViewManager.InitMainAsset(ws, vd, mainAsset)
	if mainAsset == nil then
		local name = vd.name.."Main(Clone)"
		local root = GameObject.Find("UIRoot")
		local go = nil
		if root ~= nil then
			local tran = root.transform:Find(name)
			if tran ~= nil then
				go = tran.gameObject
			end
		end
		if go ~= nil then
--			local root = GameObject.Find("UIRoot")
--			HangObject(go, root)
--			go.transform.parent = root.transform
			go:SetActive(true)
			mainAsset = go
		else
			mainAsset = this.checkMainAsset(ws, vd);
		end
	end
	this.clearChangeState();

	if mainAsset ~= nil then
		local luaBehaviour = GetComponent(mainAsset.transform,"LuaBehaviour");
		if luaBehaviour == nil then
			this.addLuaBehaviour(vd.name);
			luaBehaviour = GetComponent(mainAsset.transform,"LuaBehaviour");
		end
		
		luaBehaviour:CallMethod("OnCreate", mainAsset, vd.name)

		local bm = mainAsset:GetComponent("BaseModule");
        if bm == nil then
            bm = mainAsset:AddComponent(BaseModule.GetClassType());
			AssetManager:AddModule(vd.name, bm);
        else
			AssetManager:AddModule(vd.name, bm);
		end

		bm:initModule(vd.name , mainAsset);
		bm:showModule(true);

		if vd.hasScene then
			this.initSceneLayer(mainAsset);
			this.lastScene = this.currentScene;
			this.currentScene = vd.name;
		else
			if this.moduleLayer and Util.getMainAssetUIRoot(mainAsset) == nil then
				ViewUtil.AddChild(moduleLayer, mainAsset);
				initLayer(moduleLayer);
			end
		end

		luaBehaviour:CallMethod("Awake") -- 待用
	else
		Logger.Error("ViewManager.InitMainAsset(),没有主视图");                
	end

    this.opened(vd)
end

function ViewManager.SortOrder(target)
	this.maxOrder = this.maxOrder + 1
	SortRendererOrder(target, this.maxOrder, false)
	local max = GetMaxPanelOrder(target)
	if max > this.maxOrder then
		this.maxOrder = max
	end
end

function ViewManager.SortDepth(target, _addDepth)
    _addDepth = tonumber(_addDepth) or 0;
	this.maxDepth = this.maxDepth + 1 + _addDepth;
    if (target and target.transform) then
	    SortUIPanelDepth(target, this.maxDepth, false)
	    SortUIWidgetDepth(target, this.maxDepth, false)
    end
	local max = GetMaxPanelDepth(target)
	if max > this.maxDepth then
		this.maxDepth = max
	end
--	plog("[%s]SortDepth = %d", target.name, this.maxDepth)
end

function ViewManager.SortOrderOnly(target, father)
    local maxOrder = this.maxOrder
	if father ~= nil then
		maxOrder = GetMaxPanelOrder(father)
	end
	maxOrder = maxOrder + 1
	SortRendererOrder(target, maxOrder, false)
end

function ViewManager.SortDepthOnly(target, father, _addDepth)
    local maxDepth = this.maxDepth
	if father ~= nil then
		maxDepth = GetMaxPanelDepth(father)
	end
    _addDepth = tonumber(_addDepth) or 0;
	maxDepth = maxDepth + 1 + _addDepth;
    if (target and target.transform) then
	    SortUIPanelDepth(target, maxDepth, false)
	    SortUIWidgetDepth(target, maxDepth, false)
    end
--	plog("[%s]SortDepthOnly = %d", target.name, maxDepth)
end

function ViewManager.GuideSortDepth(target)
    local guideDepth = this.maxDepth + 1;
    
	SortUIPanelDepth(target, guideDepth, false);
	SortUIWidgetDepth(target, guideDepth, false);
end

-- 阴影遮挡
function ViewManager.SortingShade(moduleName)
	local ctrl = ModuleManager.GetCtrl(moduleName)
	if ctrl and ctrl.view and ctrl.gameObject then
		if ctrl.view.showShade == true then
			for i = this.viewQueue.size, 1, -1 do
				local modName = this.viewQueue[i].name
				local mod = ModuleManager.GetCtrl(modName)
				local target = mod.gameObject
				if target == ctrl.gameObject then
					local shade = GetGameObject(mod.transform, "ShadeView")
					if shade ~= nil then
						DestroyObject(shade)
						shade = nil
					end
					shade = ViewUtil.CreateShadeView()
					if shade ~= nil then
						HangObject(shade, target)
						local sprite = GetPathComponent(shade.transform, "Panel/Collider/Sprite", "UISprite")
						if not mod.view.shadeOpacity then
							-- 若UISprite自带BoxCollider,alpha为0则穿透，建议UISprite为BoxCollider的子集
							sprite.color = CreateColorF(sprite.color.r, sprite.color.g, sprite.color.b, 0)
						else
							sprite.color = CreateColorF(sprite.color.r, sprite.color.g, sprite.color.b, 1)
						end
					end
				else
					if mod.view.showShade == true then	
						local shade = GetGameObject(mod.transform, "ShadeView")
						if shade ~= nil then 
							DestroyObject(shade)
							shade = nil
						end
    				end
				end
			end
		end
	end
end

 -- 管理有序视图层级 by kenny
function ViewManager.SortingModuleView(gameObj, moduleName)
	local mod = ModuleManager.GetCtrl(moduleName)
	local target = gameObj
	if not mod or not target then
		return
	end
	local updateConst = false
	this.SortingShade(moduleName)
--	plog("#shade moduleName = ", moduleName)

	if not IsMember(moduleName, this.ignoreLayerViews) then
		if IsMember(moduleName, this.topViews) then
			ViewUtil.sortLayerOrder(target, LayerName.Top)
		elseif mod.view and mod.view.shadeOpacity then
			ViewUtil.sortLayerOrder(target, LayerName.Dialog)
		elseif IsMember(moduleName, this.broadcastViews) then
			ViewUtil.sortLayerOrder(target, LayerName.Broadcast)
		else
			ViewUtil.sortLayerOrder(target, LayerName.UI)
			updateConst = true
		end
	end
	
	this.SortOrder(target)
	this.SortDepth(target)

	if updateConst then
		for _, name in ipairs(this.constViews) do
			local mod = ModuleManager.GetCtrl(name)
			if mod and mod.gameObject then
				this.SortOrder(mod.gameObject)
				this.SortDepth(mod.gameObject)
			end
		end
	end

	local loading = ModuleManager.GetCtrl(ModuleName.LOAD)
	if loading.gameObject then
		this.SortOrder(loading.gameObject)
		this.SortDepth(loading.gameObject)
	end
end

function ViewManager.findLastSceneInStack()
	for i = this.viewStack:getSize() , 0 , -1 do
		if this.viewStack[i].hasScene then
			return this.viewStack[i];	
		end
	end
	return nil;
end

--清除视图栈
function ViewManager.clearViewStack()
	local bases = (not Global.isRelease) and 1 or 2;
	while this.viewStack:getSize() > bases do
		this.viewStack:removeAt(this.viewStack:getSize());
	end
end

function ViewManager.popStackView(name)
	name = name or nil;
	local stackLen = this.viewStack:getSize();
	if stackLen > 0 then
		local remind = stackLen;
		if name ~= nil then
			for i = 1 , stackLen do
				if this.viewStack[i].name == name then
					remind = i;
					break;
				end
			end
		end

		local openV = nil;
		while this.viewStack.size > remind do
			openV = this.viewStack[remind];
			this.viewStack:removeAt(remind);
		end

		return openV;
	end

	return nil;
end

function ViewManager.clearViewQueue()
	--alex todo 这里有对象池处理：  foreach (ViewData cv in new List<ViewData>(viewQueue)) ObjectPool.ReturnInstance(cv);
	this.viewQueue:clear();
end

function ViewManager.getViewData(name, hasScene, async, callBack, cloneEable, closeCurView, openParam)
	name = name or nil
	if hasScene == nil then
		hasScene = true
	end
	if async == nil then
		async = true
	end
	if cloneEable == nil then
		cloneEable = true
	end	
	if closeCurView == nil then
		closeCurView = true
	end
	callBack = callBack or function( ) end
	openParam = openParam or nil

	local vd = ViewData.New()
	vd:init(name, hasScene, async, callBack, cloneEable, closeCurView, openParam)
	return vd
end

-- 适应UIRoot by kenny
function ViewManager.AdaptUIRoot(objRoot)
	local uiRoot = GetComponent(objRoot, "UIRoot")
	Util.AdaptUIRoot(uiRoot, true)
end

-- 动态适配UI: 若参数resetPos为true, 则锚点Pivot需在中心, 对应控件将自动重置为相对于中心原点的位置
function ViewManager.AdaptUISize(gameObj, resetPos, deltaTop, deltaBottom, deltaLeft, deltaRight) -- by kenny
	local obj = gameObj
	if not obj then return end
	local realHeight = ViewUtil.getRealHeight()
	local realWidth = 640
	local paddingTop = Constants.PaddingTop + deltaTop
	local paddingBottom = Constants.PaddingBottom + deltaBottom
	local paddingLeft = Constants.PaddingLeft + deltaLeft
	local paddingRight = Constants.PaddingRight + deltaRight

	local topPos = realHeight/2 - paddingTop
	local bottomPos = -realHeight/2 + paddingBottom
	local leftPos = realWidth/2 - paddingLeft
	local rightPos = -realWidth/2 + paddingRight

	local sizeW = leftPos - rightPos
	local sizeH = topPos - bottomPos
	local centerPos = Vector3.New(leftPos - sizeW/2, topPos - sizeH/2)

	local uipanel = GetComponent(obj.transform, "UIPanel")
	if uipanel then
		uipanel.baseClipRegion = Vector4.New(0, 0, sizeW, sizeH)
	end

	local spr = GetComponent(obj.transform, "UISprite")
	if spr then
		spr.width = sizeW
		spr.height = sizeH
	end

	local collider = GetComponent(obj.transform, "BoxCollider2D")
	if collider then
		collider.size = Vector2.New(sizeW, sizeH)
	end
	
	if resetPos then
		local total = GetTotalTransform(obj)
		local parentPos = total.localPosition - obj.transform.localPosition
		obj.transform.localPosition = centerPos - parentPos 
		obj.transform.localRotation = Quaternion.identity
		obj.transform.localScale = Vector3.one
	end
end

--弹出对象
function ViewManager.pop(go)
	if this.popLayer ~= nil then
		ViewUtil.AddChild(this.popLayer,go);
	end
end

function ViewManager.Init()
	this.changeing = false;
end

function ViewManager.Step()
	this.checkLoadScene();
end

function ViewManager.Clear()
	this.clearViewQueue();
	Util.clearAssetBundleDic(loadingViewABS);
	this.currentModule = nil;
	this.currentScene = nil;
	this.lastScene = nil;
	this.clearChangeState();
	this.clearPK();
end
 
function ViewManager.getTextureAsync(path, cb)
    AssetManager.getAssetAsync(AssetGetType.Texture, AssetConst.texture .. "/" .. path, cb);
end
