
ViewUtil = {
    isOpeningHP = false;
    tipsSortOrder = 1000;
    easyGuideInfo = {};--一级元素个数代表引导的步长；二级元素中，元素一表示对话内容，元素二代表GO的路径
    easyGuideIndex = 1;
}

local this = ViewUtil


-- 给go所有的子wiget的深度+depth
function ViewUtil.AddDepth(depth, go)

	local allUIWidget = Util.GetAllUIWidgetInChildren(go);

	for i = 1 , #allUIWidget do

		local w = allUIWidget[i];

		w.depth = w.depth + depth;

	end	
end

function ViewUtil.AddChild(parent, child, x , y, undo)

	child = child or nil;

	x = x or int.MinValue;

	y = y or int.MinValue;

	undo = undo or true;

    local go = child == nil and GameObject.New() or child;

    if parent ~= nil then
    
        local t = go.transform;

        t.parent = parent.transform;

        if x ~= int.MinValue or y ~= int.MinValue then

        	t.localPosition = Vector3(x, y, 0);

        end	

        t.localRotation = Quaternion.identity;

        t.localScale = Vector3.one;

        go.layer = parent.layer;
    end

    return go;
end

--添加纹理图到go
function ViewUtil.addTexture2d(go, texture, depth  , w,  h ,  x ,  y )

	depth = depth or int.MinValue;

	w = w or int.MinValue;

	h = h or int.MinValue;

	x = x or int.MinValue;

	y = y or int.MinValue;

    if go == nil or texture == nil then
    	return;
    end	

    local tt = go.GetComponent("UITexture");

    if tt == nil then
    	tt = go.AddComponent("UITexture");
    end
    	
    if depth ~= int.MinValue then 
    	tt.depth = depth;
    end	

    local s = int.MinValue;

    if w ~= int.MinValue then
        s = w * 1.0 / texture.width;
    end

    if h ~= int.MinValue then
    
        local ss = h * 1.0 / texture.height;
        if ss < s then
        	s = ss;
        end	
    end

    if s ~= int.MinValue then
    
        go.transform.localScale = Vector3(s, s, 1);
    end

    if x ~= int.MinValue or y ~= int.MinValue then
    
        local v = go.transform.localPosition;
        if x ~= int.MinValue then
        	v.x = x;
        end
        	
        if y ~= int.MinValue then
        	v.y = y;
        end
        go.transform.localPosition = v;
    end
    tt.mainTexture = texture;
end

--在parent中找名称为name的GO
function ViewUtil.findGOForName(name, parent)

    local go = nil;

    local tansArr = Util.getAllChildrenTransform(parent);

    for i = 1 , #tansArr do
    	 if tansArr[i].name == name then
                
	        go = tansArr[i].gameObject;

	        return true;

         end 

         return false;      
    end	

    return go;
end

--[[
/// <summary>
/// 返回相对source变换的target本地位置,
/// </summary>
/// <param name="target">变换目标</param>
/// <param name="source">相对变换源</param>
/// <param name="offsetX">相对变换源偏移X</param>
/// <param name="offsetY">相对变换源偏移Y</param>
/// minX,maxX,minY,maxY ,偏移边界
]]
function ViewUtil.getRelativePos(target, source, offsetX, offsetY, minY, maxY , minX , maxX)

	offsetX = offsetX or 0;

	offsetY = offsetY or 0;

	minY = minY or int.MinValue;

	maxY = maxY or int.MaxValue;

	minX = minX or int.MinValue;

	maxX = maxX or int.MaxValue;

    local pos = source.parent:TransformPoint(source.localPosition);
    --Logger.Debug("getRelativePos" + pos + ":" + source.localPosition);
    pos = target.parent:InverseTransformPoint(pos);
    --Logger.Debug("getRelativePos----" + pos + ":" + offsetX);
    pos.x = pos.x + offsetX;
    pos.y = pos.y + offsetY;

    if pos.x < minX then
    	pos.x = minX;
    end
    	
    if pos.x > maxX then
    	pos.x = maxX;
    end
    	
    if pos.y < minY then
    	pos.y = minY;
    end
    	
    if pos.y > maxY then
    	pos.y = maxY;
    end
    	
    return pos;
end

--[[
/// <summary>
/// 变换target本地位置,相对source变换
/// </summary>
/// <param name="target">变换目标</param>
/// <param name="source">相对变换源</param>
/// <param name="offsetX">相对变换源偏移X</param>
/// <param name="offsetY">相对变换源偏移Y</param>
/// minX,maxX,minY,maxY ,偏移边界
--]]
function ViewUtil.Translate(target, source, offsetX, offsetY, minY, maxY, minX , maxX)

	minY = minY or int.MinValue;

	maxY = maxY or int.MaxValue;

	minX = minX or int.MinValue;

	maxX = maxX or int.MaxValue;

    target.localPosition = this.getRelativePos(target,source,offsetX,offsetY,minY,maxY,minX,maxX);
    
end

--向上到parent(null为root) 累加位置
function ViewUtil.addsPosToParent(tf, parent)

	parent = parent or nil;

    local pos = Vector3.zero;

    if parent ~= nil then 
    	parent = tf.root;
    end	

    while tf ~= parent do
    
        pos = pos + tf.transform.localPosition;

        tf = tf.parent;
    end

    return pos;
end

--按原来图片的高宽比例缩放sp,会纠正Widget的高,
-- Widget的宽w,0为默认宽
function ViewUtil.adjustHeight(sp,w)

	w = w or 0;

	if sp == nil or sp.mainTexture == nil then
		return;
	end
		
	local s = 1;

	if sp.GetClassType() == UITexture.GetClassType() then
	
		if w < 1 then 
			w = sp.mainTexture.width;
		end	

		sp.width = w;

		s = w * 1 / sp.mainTexture.width;

		sp.height = Mathf.FloorToInt(sp.mainTexture.height * s);
	
	elseif sp.GetClassType() == UISprite.GetClassType() then
	
		local uisp = sp;

		local spd = uisp:GetAtlasSprite();

		if spd ~= nil then
		
			if w < 1 then
				w = spd.width;
			end	

			sp.width = w;

			s = w * 1 / spd.width;

			sp.height = Mathf.FloorToInt(spd.height * s);
		end

	end
end

--按原来图片的高宽比例缩放sp,会纠正Widget的宽,
-- Widget的高h,0为默认高
function ViewUtil.adjustWidth(sp,h)

	h = h or 0;

	if sp == nil or sp.mainTexture == nil then
		return;
	end	

	local s = 1;

	if sp.GetClassType() == UITexture.GetClassType() then
	
		if h < 1 then 
			h = sp.mainTexture.height;
		end	

		sp.height = h;

		s = h * 1 / sp.mainTexture.height;

		sp.width = Mathf.FloorToInt(sp.mainTexture.width * s);
	
	elseif sp.GetClassType() == UISprite.GetClassType() then
	
		local uisp = sp;

		local spd = uisp:GetAtlasSprite();

		if spd ~= nil then
		
			if h < 1 then 
				h = spd.height;
			end
				
			sp.height = h;

			s = h * 1 / spd.height;

			sp.width = Mathf.FloorToInt(spd.width * s);
		end
	end
end


--[[
/// <summary>
/// 添加按钮点击事件
/// </summary>
/// <param name="btn">按钮对象</param>
/// <param name="cb">回调方法</param>
/// <param name="forceClick">强制用点击事件</param>
--]]
function ViewUtil.AddButtonClickEvent(btn,cb,forceClick,sound)
--	local delegate = DelegateFactory.EventDelegate(cb);--将lua的function封装成Mono的EventDelegate

	local delegate = DelegateFactory.EventDelegate_Callback(cb)
	forceClick = forceClick or false;
	sound = sound or true;
	Util.AddButtonClickEvent(btn,delegate,forceClick,sound)
end

function ViewUtil.AddScaleButtonClickEvent(target,msgTarget,methodName,param,sound)

	local ub = target.GetComponent("ScaleButton");

	if ub == nil then			
		target.AddComponent("ScaleButton");
	end

	ub = target.GetComponent("ScaleButton");

	ub.target = target;

	ub.clickTarget = msgTarget;

	ub.clickMethod = methodName;

	ub.clickParam =param;

	ub.sound = sound;
end
		
--[[		
/// <summary>
/// 添加GameObject点击事件触发器
/// </summary>
/// <param name="go">目标对象</param>
/// <param name="cb">回调方法</param>
]]
function ViewUtil.AddReleaseEventTrigger(go, cb,sound)

	local delegate = DelegateFactory.EventDelegate(cb);--将lua的function封装成Mono的EventDelegate

	Util.AddReleaseEventTrigger(go , delegate , sound);
end

--[[
/// <summary>
/// 添加GameObject点击事件触发器
/// </summary>
/// <param name="go">目标对象</param>
/// <param name="cb">回调方法</param>
]]
function ViewUtil.AddClickEventTrigger(go,  cb, sound)
    local delegate = DelegateFactory.EventDelegate(cb);--将lua的function封装成Mono的EventDelegate

	return Util.AddClickEventTrigger(go , delegate , sound);
end

function ViewUtil.clickEffect()

    Util.clickEffect();
end

-- 添加GameObject点击事件触发器
function ViewUtil.addBoxCollider(go)

	if go.GetComponent("BoxCollider") == nil then
	
		local wd = go.GetComponent("UIWidget");
		if wd == nil then
		
			wd = Util.GetUIWidgetInChildren(go);

			go = wd.gameObject;
		end

		local b = go.AddComponent("BoxCollider");

		local vs = b.size;

		vs:Set(wd.width, wd.height, 0);

		b.size = vs;

		wd.autoResizeBoxCollider = true;
	end
end

--[[
/// <summary>
/// 删除GameObject点击事件触发器
/// </summary>
/// <param name="go">目标对象</param>
/// <param name="cb">回调方法</param>
]]
function ViewUtil.RemoveClickEventTrigger(go)

    Util.RemoveClickEventTrigger(go);
end

--[[
/// <summary>
/// 删除按钮点击事件
/// </summary>
/// <param name="btn"></param>
]]
function ViewUtil.RemoveButtonClickEvent(btn)

	Util.RemoveButtonClickEvent(btn);
end

         

--uilable 文字的真实宽度
function ViewUtil.getUILableW(lb)
    return Util.getUILableW(lb);
end

--uilable 文字的真实宽度
function ViewUtil.getUILableH(lb)

    return (lb.fontSize + lb.spacingY) * lb.text.Length;
end

-- 激活物体上的点击事件

function ViewUtil.enableClick(go, able)

    Util.enableClick(go , able);
end
         
--设置ScrollView的位置
function ViewUtil.setScrollViewPos(sv, x , y )

	x = x or int.MinValue;

	y = y or int.MinValue;

    local tf = sv.transform;

    local v = tf.localPosition;

    if x ~= int.MinValue then
    	v.x = x;
    end	

    if y ~= int.MinValue then
    	v.y = y;
    end	

    tf.localPosition = v;
end



--[[
/// <summary>
/// 设置缓动信息
/// </summary>
/// <param name="tweenType">position 位置缓动 alpha 透明度 </param>
/// <param name="tweenGame">缓动物体</param>
/// <param name="from">初始位置（位置缓动使用）</param>
/// <param name="to">结束位置（位置缓动使用）</param>
/// <param name="durate">缓动时长</param>
/// <param name="fromA">开始透明度（透明度缓动使用）</param>
/// <param name="toA">结束透明度（透明度缓动使用）</param>
/// <param name="callback">缓动结束回调</param>
/// <param name="delay">缓动延时时间</param>
]]
function ViewUtil.TweenAct(tweenType,tweenGame,from,to,durate,fromA,toA ,callback,delay, method)

	fromA = fromA or 1;

	toA = toA or 1;

	callback = callback or nil;

	delay = delay or 0;

	method = method or UITweener.Method.Linear


	if tweenType == "position" or tweenType == "p" then

		tweenGame.transform.localPosition = from;

		local tp = tweenGame:GetComponent("TweenPosition");

		if tp ~= nil then
			GameObject.Destroy (tp);
		end	

		tp = tweenGame:AddComponent("TweenPosition");

		tp.method = method;

		tp.from = from; 

		tp.to = to;

		tp.duration = durate; 

		tp.enabled = true;

		tp.delay = delay;

		Util.addUITweenerFinish(tp , DelegateFactory.EventDelegate(function() 

				GameObject.Destroy (tp);
				if callback ~= nil then
					callback ();
				end		
			end			

		));
	 elseif tweenType == "alpha" or tweenType == "a"  then

		if tweenGame:GetComponent("UIWidget") == nil then
			tweenGame:AddComponent("UIWidget");
		end		

		tweenGame:GetComponent("UIWidget").alpha = fromA;

		local ta = tweenGame:GetComponent("TweenAlpha");

		if ta ~= nil then
			GameObject.Destroy (ta);-------未销毁第二次再使用会无效--
		end	

		ta = tweenGame:AddComponent("TweenAlpha");

		ta.method = method;

		ta.from = fromA;

		ta.to = toA;

		ta.duration = durate;

		ta.enabled = true;

		ta.delay = delay;


		Util.addUITweenerFinish(ta , DelegateFactory.EventDelegate(function() 

				GameObject.Destroy (ta);
				if callback ~= nil then
					callback ();
				end		
			end			

		));

		 
	 elseif tweenType == "scale" or tweenType == "s" then

		tweenGame.transform.localScale = from;

		local ts = tweenGame:GetComponent("TweenScale");

		if ts ~= nil then
			GameObject.Destroy (ts);
		end

		ts = tweenGame:AddComponent("TweenScale");

		ts.method = method;

		ts.from = from;

		ts.to = to;

		ts.duration = durate;

		ts.delay = delay;

		ts.enabled = true;

		Util.addUITweenerFinish(ts , DelegateFactory.EventDelegate(function() 

				GameObject.Destroy (ts);
				if callback ~= nil then
					callback ();
				end		
			end			

		));

	 elseif tweenType == "rotation" or tweenType == "r" then 

		tweenGame.transform.localRotation = Quaternion.Euler(from);

		local tr = tweenGame.GetComponent("TweenRotation");

		if tr ~= nil then
			GameObject.Destroy (tr);
		end
			
		tr = tweenGame.AddComponent("TweenRotation");

		tr.from = from;

		tr.to = to;

		tr.duration = durate;

		tr.enabled = true;

		tr.method = method;

		tr.delay = delay;

		Util.addUITweenerFinish(tr , DelegateFactory.EventDelegate(function() 

				GameObject.Destroy (tr);
				if callback ~= nil then
					callback ();
				end		
			end			

		));
	end
end	

--[[
/// <summary>
/// 设置字体缓动效果
/// </summary>
/// <param name="label">字体对象</param>
/// <param name="startNum">开始值</param>
/// <param name="maxNum">结束值</param>
/// <param name="position">字体当前位置</param>
/// <param name="otherText">字体其他的文字 xx{1}xx</param>
/// <param name="changeNum">数值转动次数</param>
/// <param name="changeTime">每次转动时间</param>
/// <param name="smallTime">变大后缩小时间</param>
/// <param name="scaleBase">变大倍数</param>

]]
function ViewUtil.setNumChange(label,startNum,maxNum,position,otherText ,changeNum ,changeTime,smallTime,scaleBase,tweenPosition) 

	otherText = otherText or "";

	changeNum = changeNum or 10.0;

	changeTime = changeTime or 0.1;

	smallTime = smallTime or 0.5;
	
	scaleBase = scaleBase or 1.2;

	tweenPosition = tweenPosition or true;

	label.text = not String.IsNullOrEmpty(otherText) and string.gsub(otherText,"{1}",startNum .. "") or (startNum .. "");		

	if maxNum == startNum then
		return;
	end
		
	if Mathf.Abs(float.Parse((maxNum - startNum) .. "")) < changeNum then

		changeNum = Mathf.Abs(float.Parse((maxNum - startNum) .. ""));

	end
						
	local upNum = Mathf.CeilToInt ((maxNum - startNum) / changeNum);

	local curNum = startNum;

	this.TweenAct ("scale", label.gameObject, Vector3.one, Vector3 (scaleBase, scaleBase, 1), changeNum * changeTime/2, 1, 1,

		function() 

			this.TweenAct("scale",label.gameObject,label.transform.localScale,Vector3.one,smallTime,1,1,nil,changeNum * changeTime/2);

        	if tweenPosition then
        		this.TweenAct("position", label.gameObject, label.transform.localPosition, position, smallTime, 1, 1, nil, changeNum * changeTime / 2);
			end
       	end		

	);

    if tweenPosition then
    	ViewUtil.TweenAct("position", label.gameObject, position, Vector3(position.x + label.width * (scaleBase - 1) / 2.0, position.y + label.height * (scaleBase - 1) / 2, position.z), changeNum * changeTime / 2, 1, 1);
    end	
	
	local func = function()

			if upNum > 0 and (curNum + upNum) >= maxNum or upNum < 0  and (curNum + upNum) <= maxNum then
				label.text = not String.IsNullOrEmpty(otherText) and string.gsub(otherText , "{1}" , maxNum .. "") or (maxNum .. "");
			
			else
				curNum = curNum+upNum;

				label.text = not String.IsNullOrEmpty(otherText) and string.gsub(otherText , "{1}" , curNum .. "") or (curNum .. "");
			end	
		end;
	
	func ();

	Timer.invoke(DelegateFactory.Action(func),changeTime,Mathf.CeilToInt(changeNum));
end

function ViewUtil.getRootView()
	return GameObject.Find("UIRoot"):GetComponent(UIRoot.GetClassType())
 end

function ViewUtil.isExistView(moduleName)
	return ViewManager.isExistView(moduleName)
end

function ViewUtil.goto(moduleName, keepCurView, callback, params)
	ViewManager.Goto(moduleName, keepCurView, callback, params)
end

function ViewUtil.openView(moduleName, keepCurView, callback, params)
	ViewManager.OpenView(moduleName, keepCurView, callback, params)
end

function ViewUtil.openCurView(callback, params)
	ViewManager.OpenCurModule(callback, params)
end

function ViewUtil.openLastView(callback, params)
	ViewManager.OpenLastModule(callback, params)
end

function ViewUtil.closeView(moduleName, openLastView)
	ViewManager.CloseModule(moduleName, openLastView)
end

function ViewUtil.closeAllViews(clearStack, forceAll)
	ViewManager.CloseAllModules(clearStack, forceAll)
end

-- 关闭Loading
function ViewUtil.closeLoading()
    ViewUtil.closeView(ModuleName.LOAD)
    ViewUtil.showNewHeros();
end

function ViewUtil.isCurModule(name)
	if name and name == ViewManager.GetCurrentModule() then
		return true
	end
	return false
end

function ViewUtil.isTopModule(name)
	if name and name == ViewManager.GetTopModule() then
		return true
	end
	return false
end

function ViewUtil.getLastModule(modName)
	return ViewManager.GetLastModule(modName)
end

function ViewUtil.getTopModule()
	return ViewManager.GetTopModule()
end

function ViewUtil.getCurrentModule()
	return ViewManager.GetCurrentModule()
end

function ViewUtil.getTopView()
	local mod = ViewManager.GetTopModule()
	local view = nil
	local ctrl = ModuleManager.GetCtrl(mod)
	if ctrl ~= nil then
		view = ctrl.view
	end
	return view
end

function ViewUtil.getCurrentView()
	local mod = ViewManager.GetCurrentModule()
	local view = nil
	local ctrl = ModuleManager.GetCtrl(mod)
	if ctrl ~= nil then
		view = ctrl.view
	end
	return view
end

function ViewUtil.isTopView(name)
	return this.isTopModule(name)
end

function ViewUtil.getTopViewObject()
	local view = this.getTopView()
	if not view then return nil end
	return view.gameObject
end

function ViewUtil.getUIRoot()
	return GameObject.Find("UIRoot")
end

function ViewUtil.getRealWidth()
	return ViewManager.realWidth
end

function ViewUtil.getRealHeight()
	return ViewManager.realHeight
end

function ViewUtil.adaptUIRoot(objRoot)
	ViewManager.AdaptUIRoot(objRoot)
end

function ViewUtil.adaptUISize(gameObj, resetPos, deltaTop, deltaBottom, deltaLeft, deltaRight)
	ViewManager.AdaptUISize(gameObj, resetPos, deltaTop, deltaBottom, deltaLeft, deltaRight)
end

function ViewUtil.adaptScrollViewHeight(scvBGSprite, scvPanel, delta)
	local saveX = scvPanel.baseClipRegion.x
	local saveY = scvPanel.baseClipRegion.y
	local saveWidth = scvPanel.baseClipRegion.z
	local saveHeight = scvPanel.baseClipRegion.w
	local realHeight = ViewUtil.getRealHeight() - Constants.PaddingTop - Constants.PaddingBottom - delta
	local deltaHeight = realHeight - saveHeight
	scvBGSprite.width = scvBGSprite.width
	scvBGSprite.height = scvBGSprite.height + deltaHeight
	scvPanel.baseClipRegion = Vector4.New(saveX, saveY - deltaHeight/2, saveWidth, realHeight)
end

--<summary>渲染层级次序</summary>
--<param name="gameObj">GameObject</param>
--<param name="layerName">层级名称</param>
--<param name="isTemp">是否永久修改</param>
function ViewUtil.sortLayerOrder(gameObj, layerName, isTemp, _addDepth)
    local tmpDepth = 0;
    _addDepth = tonumber(_addDepth) or 0;
	if IsMember(layerName, LayerName) then
		SortingLayer(gameObj, layerName)
	end
	if not isTemp then
		ViewManager.SortOrder(gameObj)
		ViewManager.SortDepth(gameObj, _addDepth)
	else
		ViewManager.SortOrderOnly(gameObj)
		ViewManager.SortDepthOnly(gameObj, nil, _addDepth)
	end
    
	if this.boardLost ~= nil then
        if (GetMaxPanelDepth(this.boardLost) < ViewManager.maxDepth + 1) then
		    if not isTemp then
			    ViewManager.SortDepth(this.boardLost, _addDepth)
		    else
			    ViewManager.SortDepthOnly(this.boardLost, nil, _addDepth)
		    end
        end
		ViewManager.SortOrderOnly(this.boardLost);
        SetCameraLayer(this.boardLost, CameraLayer.GameTop, true);
	end
end

function ViewUtil.addDragScrollView(gameObj, scrollObj)
	if not gameObj then return end
	local drag = gameObj:AddComponent(UIDragScrollView.GetClassType())
	if scrollObj ~= nil then
		local com = GetComponent(scrollObj.transform,"UIScrollView")
		if com ~= nil then
			drag.scrollView = com
		end
	end
end
function ViewUtil.removeDragScrollView(gameObj)
	if not gameObj then return end
	local drags = GetComponentsInChildren(gameObj.transform, UIDragScrollView.GetClassType())
	for i = 0, drags.Length - 1 do
		local comp = drags[i]
		if comp ~= nil then
			DestroyObject(comp)
		end
	end
end

function ViewUtil.enableDragScrollView(gameObj, scrollObj)
	if not gameObj then return end
	local drags = GetComponentsInChildren(gameObj.transform, UIDragScrollView.GetClassType())
	for i = 0, drags.Length - 1 do
		local comp = drags[i]
		if comp ~= nil then
			if not scrollObj then
				comp.enabled = true
			elseif comp.scrollView == scrollObj then
				comp.enabled = true
			end
		end
	end
end
function ViewUtil.disableDragScrollView(gameObj, scrollObj)
	if not gameObj then return end
	local drags = GetComponentsInChildren(gameObj.transform, UIDragScrollView.GetClassType())
	for i = 0, drags.Length - 1 do
		local comp = drags[i]
		if comp ~= nil then
			if not scrollObj then
				comp.enabled = false
			elseif comp.scrollView == scrollObj then
				comp.enabled = false
			end
		end
	end
end

function ViewUtil.addDragCamera(gameObj, cameraObj)
	local com = GetComponent(gameObj.transform,"UIDragCamera")
	if com == nil then
		gameObj:AddComponent(UIDragCamera.GetClassType())
	end
	if cameraObj then
		com = GetComponent(gameObj.transform,"UIDragCamera")
		local cam = GetComponent(cameraObj.transform,"UIDraggableCamera")
		if cam then com.draggableCamera = cam end
	end
end

function ViewUtil.removeDragCamera(gameObj)
	local com = GetComponent(gameObj.transform,"UIDragCamera")
	if com ~= nil then
		DestroyObject(com)
	end
end

function ViewUtil.initData(ctrl, data)
	ViewUtil.closeLoading()
	ctrl.data = nil
	if not IsEmpty(data) then
		ctrl.data = data
		if ctrl.view ~= nil then
			ctrl.view:initData(ctrl.data)
		end
	end
end

--[[
    在屏幕上飘过显示消息
    _msg string 需要显示的提示
    return nil
]]
function ViewUtil.showMsg (_msg)
    local prefab = Util.LoadPrefab("UI/TipsMsg", AssetBundleInfo.assetBundles);
    local tmpTips = CreateObject(prefab);
    local msg = GetPathComponent(tmpTips.transform, "msg", "UILabel");
    local root = ViewUtil.getRootView();
    local tipsPos = tmpTips.transform.localPosition;

    _msg = _msg or "";
    msg.text = _msg;
    HangObject(tmpTips, root);
    SortingLayer(tmpTips, LayerName.Top);
    tmpTips.transform.localPosition = Vector3.New(0, -120, 0);
    if (ViewManager.maxOrder > this.tipsSortOrder) then 
        this.tipsSortOrder = ViewManager.maxOrder;
    end
    this.tipsSortOrder = this.tipsSortOrder + 1;
    GetComponent(tmpTips, "UIPanel").sortingOrder = this.tipsSortOrder;

    iTween.MoveTo(tmpTips, iTween.Hash(
        "position", Vector3.New(tipsPos.x, 120, tipsPos.z),
        "islocal", true,
        "time", 0.75,
        "easeType", iTween.EaseType.easeOutQuad,
        "oncomplete", function () 
                DestroyObject(tmpTips, 1);
            end
    ));
end

--<summary>事件触发器</summary>
--<param name="go">GameObject</param>
--<param name="eventType">事件类型</param>
--<param name="cbFunc">回调方法</param>
--<param name="param">回调参数</param>
--<param name="noSound">是否无声</param>
function ViewUtil.AddEventTrigger(go, eventType, cbFunc, param, noSound)
	if not go then return end
	if type(param) ~= "table" then
		param = {param}
	end
	local function _doEvent(tbl)
		if eventType == EventConst.onClick then
			if not noSound then
				SimpleAudio.PlayEffect("tap")
			end
		end
		if cbFunc ~= nil then
			cbFunc(tbl)
		end
	end
	return Util.AddEventTrigger(go, eventType, _doEvent, param)
end

--<summary>点击事件</summary>
--<param name="go">GameObject</param>
--<param name="cbFunc">回调方法</param>
--<param name="noSound">是否无声</param>
function ViewUtil.AddClickEvent(go, cbFunc, noSound)
	if not go then return end
	local function _doClick(obj)
		if not noSound then
			SimpleAudio.PlayEffect("tap")
		end
		if cbFunc ~= nil then
			cbFunc(obj)
		end
	end
	UIEventListener.Get(go).onClick = _doClick
end

--[[
    添加遮罩面板
    _lPos localPosition
    _lScl localScale
    _callBack
    _tp 0，遮罩面板添加事件；1，遮罩面板不添加事件
]]
function ViewUtil.showScreenMask (_lPos, _lScl, _callBack, _tp, _name)
    local prefab, sMask, render, uiLbl = nil, nil, nil, nil;
    
    _lPos = _lPos or Vector3.New(0, 0, 0);
    _lScl = _lScl or Vector3.New(1, 1, 1);
    prefab = Util.LoadPrefab("UI/ScreenMask", AssetBundleInfo.assetBundles);
    sMask = CreateObject(prefab);
	if (nil ~= _name) then
		sMask.name = "ScreenMask_" .. _name;
	end
    HangObject(sMask, this.getRootView());
    SortingLayer(sMask, LayerName.Top);
    sMask.transform.localPosition = _lPos;
    sMask.transform.localScale = _lScl;
    render = GetGameObject(sMask.transform, "BG"):AddComponent(MeshRenderer.GetClassType());
    render.material.color.a = 0.38;
    
    uiLbl = GetComponent(sMask.transform, "UIPanel");
    uiLbl.depth = 999;
    
    if (0 == _tp) then
        -- UIEventListener.Get(sMask).onClick = function () 
        --     DestroyObject(sMask); 
        --     if (nil ~= _callBack) then
        --         _callBack();
        --     end
        -- end;
        ViewUtil.AddClickEvent(sMask, function () 
            DestroyObject(sMask); 
            if (nil ~= _callBack) then
                _callBack();
            end
        end)
    end
    return sMask;
end

--[[
    二次确认框
    _cfmWords string 确认内容
    _okFunc function 确认按钮事件
    _cancelFunc function 取消按钮事件
    _okWords string 确认按钮内容
    _cancelWords string 取消按钮内容
    return nil
]]
function ViewUtil.comCfmPanel (_cfmWords, _okFunc, _cancelFunc, _okWords, _cancelWords)
    local cfmPrefab, cfmObj, cfmPanel, root, sMask = nil, nil, nil, nil, nil;
    local okFunc, cancelFunc = nil, nil;

    okFunc = function () 
        if (nil ~= sMask) then
            DestroyObject(sMask);
        end
        if (nil ~= _okFunc) then
            _okFunc();
        end
        DestroyObject(cfmObj);
    end
    
    cancelFunc = function () 
        if (nil ~= sMask) then
            DestroyObject(sMask);
        end
        if (nil ~= _cancelFunc) then
            _cancelFunc();
        end
        DestroyObject(cfmObj);
    end

    cfmPrefab = Util.LoadPrefab("UI/CfmPanel", AssetBundleInfo.assetBundles);
    cfmObj = CreateObject(cfmPrefab);
    
    sMask = ViewUtil.showScreenMask();
    GetComponent(sMask, "UIPanel").depth = 0;
    ViewUtil.sortLayerOrder(sMask, LayerName.Dialog);
    
    root = ViewUtil.getRootView();
    cfmPanel = ComCfmPanel.new();
    cfmPanel:Awake(cfmObj);
    cfmPanel:initInfo(_cfmWords, okFunc, cancelFunc, _okWords, _cancelWords);
    PanelManager:BindPanelScript(cfmPanel, cfmObj, doNthFunc, doNthFunc, doNthFunc, doNthFunc);
    HangObject(cfmObj, root);

    cfmObj.transform.localPosition = Vector3.New(-1, 128, 0);
    ViewUtil.sortLayerOrder(cfmObj, LayerName.Top);
end

--[[
    二次确认框，带有可变数量框
    _cfmWords string 确认内容
    _okFunc function 确认按钮事件
    _cancelFunc function 取消按钮事件
    _numChgeFunc function 数字改变事件
    _okWords string 确认按钮内容
    _cancelWords string 取消按钮内容
    return nil
]]
function ViewUtil.comNumChgePanel (_cfmWords, _okFunc, _cancelFunc, _numChgeFunc, _okWords, _cancelWords)
    local cncPanel, root, sMask, cfmObj = nil, nil, nil, nil;
    local okFunc, cancelFunc = nil, nil;

    okFunc = function (_num) 
        if (nil ~= sMask) then
            DestroyObject(sMask);
        end
        if (nil ~= _okFunc) then
            _okFunc(_num);
        end
    end
    
    cancelFunc = function () 
        if (nil ~= sMask) then
            DestroyObject(sMask);
        end
        if (nil ~= _cancelFunc) then
            _cancelFunc();
        end
    end

    sMask = ViewUtil.showScreenMask();
    GetComponent(sMask, "UIPanel").depth = 0;
    ViewUtil.sortLayerOrder(sMask, LayerName.Dialog);
    
    root = ViewUtil.getRootView();
    cncPanel = CfmNumChgePanel.new();
    cncPanel:Awake(_cfmWords, okFunc, cancelFunc, _numChgeFunc, _okWords, _cancelWords);
    cfmObj = cncPanel.gameObject;
    HangObject(cfmObj, root);

    cfmObj.transform.localPosition = Vector3.New(-1, 128, 0);
    ViewUtil.sortLayerOrder(cfmObj, LayerName.Top);
end

-- 创建UI底部阴影遮挡
function ViewUtil.CreateShadeView(callbackFunc, params)
	local shade = CreateObjectByPath("UI/ShadeView", AssetBundleInfo.assetBundles)
	if not shade then return nil end
	shade.name = "ShadeView"
	SortRendererOrder(shade, -100, false)
	SortUIPanelDepth(shade, -100, false)
	SortUIWidgetDepth(shade, -100, false)
	local spr = GetPathComponent(shade.transform, "Panel/Collider/Sprite", "UISprite")
	spr.width = ViewManager.realWidth + 20
	spr.height = ViewManager.realHeight + 20
	local collide = GetGameObject(shade.transform, "Panel/Collider")
	local function _callback(args)
		local event = args.eType
		if event == EventConst.onClick then
			if callbackFunc ~= nil then
				callbackFunc(params)
			end
		end
	end
	ViewUtil.AddEventTrigger(collide, EventConst.onClick, _callback, {eType=EventConst.onClick})
	return shade
end

--[[
    获得的英雄动画显示
    _hDataList List<HeroData>
    return GameObject 
]]
function ViewUtil.ShowHeros (_hDataList)
    local shade, hPane, grid, uiGrid = nil, nil, nil;
    local hData, hCfg, heroTmp, baseHero = nil, nil, nil, nil;
    local goList = List.new();
    local coroutine = nil;
    local luckyCtl = nil;

    luckyCtl = ModuleManager.GetCtrl(ModuleName.LUCK);
    if (luckyCtl.animating) then
        --抽卡时，不再显示这里的动画
        return ;
    end

    shade = ViewUtil.showScreenMask(nil, nil, function () 
        if (nil ~= coroutine) then
            LuaCallManager:cancelInvokeCall(coroutine);
        end
        DestroyObject(hPane);
    end, 0, "Push");
    hPane = CreateObject(Util.LoadPrefab("UI/PaneContainer", AssetBundleInfo.assetBundles));

    HangObject(hPane, shade);
    ViewManager.SortDepthOnly(shade, nil, 1000);
    -- SetCameraLayer(shade, CameraLayer.GameMask, true)
    grid = GetGameObject(hPane.transform, "Grid");
    uiGrid = GetComponent(hPane, "UIGrid");
    
    for i = 1, #_hDataList do
        hData = _hDataList[i];
        hCfg = HeroDataProxy.getHeroCfgById(hData.heroId);
        baseHero = BaseHero.new();
        baseHero:setData(hCfg);
        heroTmp = baseHero:createHero("UI/Hero");
        HangObject(heroTmp, grid);
        goList:add({heroTmp});
        heroTmp:SetActive(false);
    end
    HangObject(shade, this.getRootView());
    SortingLayer(hPane, LayerName.Top);
    
    coroutine = LuaCallManager:invokeCall(function () 
        local endPos, beginPos = nil, Vector3.New(300, 600, 0);
        for i = 1, goList:getSize() do
            heroTmp = goList[i][1];
            goList[i][2] = heroTmp.transform.localPosition;
            heroTmp.transform.localPosition = beginPos;
        end
        
        local idx = 1;
        local function anmtiTween ()
            if (idx > goList:getSize()) then
                ViewUtil.showNewHeros();
                return ;
            end
            _go = goList[idx][1];
            _go:SetActive(true);
            iTween.MoveTo(_go, iTween.Hash(
                "position", goList[idx][2],
                "islocal", true,
                "time", 0.4,
                "easeType", iTween.EaseType.easeOutQuad,
                "oncomplete", function () 
                    idx = idx + 1;
                    anmtiTween();
                end
            ));
        end
        anmtiTween();
    end, 0.1);
end

--[[
    新的英雄的显示--引导至详细页
]]
function ViewUtil.showNewHeros ()
--    error(HeroDataProxy.newHeroList.size .. "|" .. tostring(ViewManager.changeing) .. "|" .. tostring(Network.goPK) .. "|" .. tostring(ViewManager.isExistView(ModuleName.FIGHT)) .. "|"  .. tostring(ViewManager.isExistView(ModuleName.LOAD)));
    local luckyCtl = ModuleManager.GetCtrl(ModuleName.LUCK);
    local hpCtl = ModuleManager.GetCtrl(ModuleName.HP);
    
    if (ViewManager.changeing or this.isOpeningHP or (nil ~= luckyCtl and luckyCtl.animating)) then
        return ;
    end

    if (Network.goPK or ViewManager.isExistView(ModuleName.FIGHT) or ViewManager.isExistView(ModuleName.LOAD)) then
        return ;
    end

    if (HeroDataProxy.newHeroList.size > 0) then
        this.isOpeningHP = true;
        ModuleManager.GetCtrl(ModuleName.HP).data = HeroDataProxy.getHeroCfgById(HeroDataProxy.newHeroList[HeroDataProxy.newHeroList.size].heroId);
        HeroDataProxy.newHeroList:removeAt(HeroDataProxy.newHeroList.size);
        
        if (ViewManager.isExistView(ModuleName.PROP)) then
            local ctl = ModuleManager.GetCtrl(ModuleName.HP);
            ViewUtil.closeView(ModuleName.PROP, false);
            ctl.openProp = true;
        end
        
        if (not ViewManager.isExistView(ModuleName.HP)) then
            ViewUtil.openView(ModuleName.HP, true);
        else
            hpCtl.Start();
            this.isOpeningHP = false;
        end
    end
end

--[[
    缓动动画，逐渐消失
]]
function ViewUtil.showItem (_go)
    local time = 0.75;
    iTween.ScaleTo(_go, Vector3.New(0.1, 0.1, 1), time);
    iTween.MoveTo(_go, iTween.Hash(
        "y", _go.transform.localPosition.y - 120,
        "islocal", true,
        "time", time + 0.1,
        "oncomplete", function () 
                DestroyObject(_go);
            end
    ));
end

--[[
    设置游戏是否响应事件，修改UICamera的EventMask值
    _isEnable boolean true，响应事件；false，不响应事件
]]
function ViewUtil.gameEventEnabled (_isEnable)
    local layerMaskVal = -1;
    local uiCamera, layerMask = nil, nil;--UICamera, LayerMask

    uiCamera = GetComponent(GameObject.Find("UICamera").transform, "UICamera");
    if (nil ~= _isEnable) then
        layerMaskVal = _isEnable and -1 or 0;
        layerMask = LayerMask.New();
        layerMask.value = layerMaskVal;
        uiCamera.eventReceiverMask = layerMask;
    else    
        layerMaskVal = uiCamera.eventReceiverMask.value;

        return layerMaskVal;
    end
end

----[[
--    添加道具tips事件
--    _propId number 道具ID
--    _pgo GameObject 需要添加tips的GO
--]]
--function ViewUtil.showPropTips (_propId, _pgo)
--    local pCfg = PropDataProxy.getPropCfgById(_propId);

--    UIEventListener.Get(_pgo).onPress = function (_go, _isPress) showPropTipsTmp(_go, _isPress, pCfg); end

--    local function showPropTipsTmp (_go, _isPress, _pCfg)
--        if (_isPress) then
--            local pTipsGO = CreateObject(Util.LoadPrefab("UI/TipsProp"));
--            local superWPos, superLPos = _pgo.transform.position, _pgo.transform.localPosition;
--            local superSPos = Camera.main:WorldToScreenPoint(superWPos);
--            local cSPos = Vector3.New(superSPos.x + 50, superSPos.y + 50, superSPos.z);
--            local cWPos = Camera.main:ScreenToWorldPoint(cSPos);

--            GetPathComponent(pTipsGO.transform, "pName", "UILabel").text = PropDataProxy.getPropName(_pCfg);
--            GetPathComponent(pTipsGO.transform, "pDesc", "UILabel").text = PropDataProxy.getPropDesc(_pCfg);
--            HangObject(pTipsGO, ViewUtil.getRootView());
--            pTipsGO.transform.position = cWPos;
--            ViewUtil.constTips = pTipsGO;

--            ViewUtil.sortLayerOrder(pTipsGO, LayerName.Top);
--        else
--            if (nil ~= ViewUtil.constTips) then
--                DestroyObject(ViewUtil.constTips);
--                ViewUtil.constTips = nil;
--            end
--        end
--    end
--end

function ViewUtil.showSkillTipsEvent(_go, _isPress, _skillId)
	if not _go or IsEmpty(_skillId) then return end
    local sCfg = SkillDataProxy.getSkillCfgById(_skillId);
    if (_isPress) then
        local pTipsGO = CreateObject(Util.LoadPrefab("UI/TipsProp", AssetBundleInfo.assetBundles));
--        local superWPos = _go.transform.position;
--        local superSPos = Camera.main:WorldToScreenPoint(superWPos);
--        local cSPos = Vector3.New(superSPos.x + 100, superSPos.y + 50, superSPos.z);
--        local cWPos = Camera.main:ScreenToWorldPoint(cSPos);
        local mousePos = Input.mousePosition;
        local mouseToWorldPos = Camera.main:ScreenToWorldPoint(mousePos);
        local worldToLocPos = _go.transform:InverseTransformPoint(mouseToWorldPos);
        local realLocPos = Vector3.New(worldToLocPos.x + 170, worldToLocPos.y + 120, worldToLocPos.z);
        local cWPos = _go.transform:TransformPoint(realLocPos);
            
        GetPathComponent(pTipsGO.transform, "pName", "UILabel").text = SkillDataProxy.getSkillName(sCfg);
        GetPathComponent(pTipsGO.transform, "pDesc", "UILabel").text = SkillDataProxy.getSkillDesc(sCfg);
        HangObject(pTipsGO, ViewUtil.getRootView());
        
        pTipsGO.transform.position = cWPos;
        ViewUtil.constTips = pTipsGO;

        ViewUtil.sortLayerOrder(pTipsGO, LayerName.Top);
    else
        if (nil ~= ViewUtil.constTips) then
            DestroyObject(ViewUtil.constTips);
            ViewUtil.constTips = nil;
        end
    end
end

--[[
    显示技能tips
    _skillId number 技能ID
    _sgo GameObject 需要添加tips的技能GO
]]
function ViewUtil.showSkillTips(_skillId, _sgo)
    UIEventListener.Get(_sgo).onPress = function(_go, _isPress)
		this.showSkillTipsEvent(_go, _isPress, _skillId);
	end
end

function ViewUtil.showHerosTips(go, skillId, isPress)
	if not go or IsEmpty(skillId) then return end
	if isPress then
		if this.skillTips then
			DestroyObject(this.skillTips)
			this.skillTips = nil
		end
		this.skillTips = CreateObjectByPath("UI/TipsSkill", AssetBundleInfo.assetBundles)
		
        local curPos = UICamera.currentTouch.pos
        local touchPos = Camera.main:ScreenToWorldPoint(curPos)
        local srcPos = go.transform:InverseTransformPoint(touchPos)
        local tmpPos = Vector3.New(srcPos.x + 170, srcPos.y + 310, srcPos.z)
        local destPos = go.transform:TransformPoint(tmpPos)        

		local cfgS = SkillDataProxy.getSkillCfgById(skillId)
		local heroIds = split(cfgS.hero_id, ":")
		local count = 0
	
		local gridObj = GetGameObject(this.skillTips.transform, "BG/Grid")
		for i = 0, #heroIds do
			local heroId = tonumber(heroIds[i]) or 0
			if heroId > 10000 then
				count = count + 1
				local cfgH = HeroDataProxy.getHeroCfgById(heroId)
				local tipsClass = TipsClass.new()
				tipsClass:init(cfgH, heroId)
				HangObject(tipsClass:createSkillHero(), gridObj)
			end
		end
		if count > 1 then
			ViewUtil.sortLayerOrder(this.skillTips, LayerName.UI)
			HangObject(this.skillTips, go)
			this.skillTips.transform.position = destPos;
		else
			DestroyObject(this.skillTips)
			this.skillTips = nil
		end
	else
		if this.skillTips then
			DestroyObject(this.skillTips)
			this.skillTips = nil
		end
	end
end

-- 创建道具和英雄掉落的Item
function ViewUtil.CreateDropItem(propId, num, parentObj)
	local item = nil
	if GetIntPart(tonumber(propId)/10000) > 0 then
		local hCfg = HeroDataProxy.getHeroCfgById(propId)
		local baseHero = BaseHero.new()
		baseHero:setData(hCfg)
		item = baseHero:createHero("UI/ItemHero")
		HangObject(item, parentObj)
	else
		item = CreateObjectByPath("UI/ItemDrop", AssetBundleInfo.assetBundles)
		local sprComp = GetPathComponent(item.transform, "Sprite", "UISprite")
		local numComp = GetPathComponent(item.transform, "Num", "UILabel")
		sprComp.spriteName = propId
		numComp.text = num
		HangObject(item, parentObj)
	end
	return item
end

function ViewUtil.CreateLostBoard(kind)
	if this.boardLost ~= nil then return end
	local uiRoot = ViewUtil.getUIRoot()
	local boardLost = CreateObjectByPath("UI/BoardLost", AssetBundleInfo.assetBundles)
	local shade = ViewUtil.CreateShadeView()
	HangObject(shade, boardLost)
	HangObject(boardLost, uiRoot)
    
	ViewUtil.sortLayerOrder(boardLost, LayerName.Top, true, 100)
	this.boardLost = boardLost
    SetCameraLayer(this.boardLost, CameraLayer.GameTop, true);

	local content = GetPathComponent(boardLost.transform, "Panel/Content", "UILabel")
	local btnOK = GetGameObject(boardLost.transform, "Panel/BtnOK")
	local btnCancel = GetGameObject(boardLost.transform, "Panel/BtnCancel")

	if kind == 0 then
		content.text = "服务器网络已断开！"
	elseif kind == 1 then
		content.text = "您的账号已在其他地方登陆！"
	end

	local function doClick(obj)
		if obj == btnOK then
            if (not GuideControler.isGuideShowing()) then
			    EnterControler.Login(true, false);
            else
                GuideControler.callViewGoResume();
                GuideControler.showCurModule(false);
			    ViewUtil.closeAllViews(true, true)
			    ViewUtil.openView(ModuleName.LOGIN)
            end
		elseif obj == btnCancel then
            if (not GuideControler.isGuideShowing()) then
			    ViewUtil.closeAllViews()
			    ViewUtil.openView(ModuleName.LOGIN)
            else
                GuideControler.callViewGoResume();
                GuideControler.showCurModule(false);
                ViewUtil.closeAllViews(true, true)
			    ViewUtil.openView(ModuleName.LOGIN)
            end
		end
		DestroyObject(this.boardLost)
		this.boardLost = nil
	end
	-- UIEventListener.Get(btnOK).onClick = doClick
	-- UIEventListener.Get(btnCancel).onClick = doClick
	ViewUtil.AddClickEvent(btnOK, doClick)
	ViewUtil.AddClickEvent(btnCancel, doClick)

	Network.Disconnect()
end


--[[
    显示tips
]]
function ViewUtil.showCommonTips (_go, _ruleStr)
    local bg, label = nil, nil;
    local mousePos = Input.mousePosition;
    local mouseToWorldPos = Camera.main:ScreenToWorldPoint(mousePos);
    local worldToLocPos = _go.transform:InverseTransformPoint(mouseToWorldPos);
    local realLocPos, cWPos = nil, nil;

    this.comTips = CreateObject(Util.LoadPrefab("UI/ComTips", AssetBundleInfo.assetBundles));
    HangObject(this.comTips, ViewUtil.getRootView());
    label = GetPathComponent(this.comTips.transform, "label", "UILabel");
    label.text = _ruleStr;
    bg = GetPathComponent(this.comTips.transform, "BG", "UISprite");
    bg.width = label.width + 40;
    bg.height = label.height + 40;
    realLocPos = Vector3.New(worldToLocPos.x - bg.width / 2 - 10, worldToLocPos.y - bg.height / 2 - 10, worldToLocPos.z);
    cWPos = _go.transform:TransformPoint(realLocPos);
    this.comTips.transform.position = cWPos;
    this.sortLayerOrder(this.comTips, LayerName.Top);
end

--[[
    删除tips
]]
function ViewUtil.removeTips ()
    if (nil ~= this.comTips) then
        DestroyObject(this.comTips);
        this.comTips = nil;
    end
end

--[[
    设置_go的SortingLayer、SortingOrder
    _go GameObject 
    _layerName string
    _order number
    _tp number 0，覆盖原来的sortingOrder；1，不覆盖
]]
function ViewUtil.setAnmtSortingLayerOrder (_go, _layerName, _order, _tp)
    local transTmp, len = nil, 0;
    local spriteRender = nil;
    
    if (nil == _go) then
        return ;
    end
    _layerName = _layerName or LayerName.UI;
    _order = _order or 0;
    len = _go.transform.childCount;
    _tp = _tp or 0;
    
    for i = 0, len - 1 do
        local lenTmp = 0;
        transTmp = _go.transform:GetChild(i);
        spriteRender = GetComponent(transTmp, "SpriteRenderer");
        if (nil ~= spriteRender) then
            spriteRender.sortingLayerName = _layerName;
            if (1 == _tp) then
                spriteRender.sortingOrder = spriteRender.sortingOrder + _order;
            else
                spriteRender.sortingOrder = _order;
            end
        end
        lenTmp = transTmp.childCount;
        if (lenTmp > 0) then
            this.setAnmtSortingLayerOrder(transTmp.gameObject, _layerName, _order, _tp);
        end
    end
end

function ViewUtil.CreateFriendHeroes(propId, parentObj)
	local item = nil
	local hCfg = HeroDataProxy.getHeroCfgById(propId)
	local baseHero = BaseHero.new()
	baseHero:setData(hCfg)
	item = baseHero:createHero("UI/ItemHero")
	HangObject(item, parentObj)
	return item
end

--[[
    显示简易引导
    _tp number 简易引导的类型
    return nil
]]
function ViewUtil.showEasyGuide (_tp)
    local eGGO = nil;

    this.easyGuideIndex = 1;
    this.easyGuideInfo = {}
    if (_tp == ConstantNames.EasyGuide_FBPKFail) then
        this.easyGuideInfo = {
            {GuideConst.easyGuideWordInfo.const_0, GuideConst.easyGuideWordInfo.const_1}
        };
    elseif (_tp == ConstantNames.EasyGuide_FBMidCard) then
        this.easyGuideInfo = {
            {GuideConst.easyGuideWordInfo.const_2, GuideConst.easyGuideWordInfo.const_3}
        };
    elseif (_tp == ConstantNames.EasyGuide_FBHardCard) then
        this.easyGuideInfo = {
            {GuideConst.easyGuideWordInfo.const_4, GuideConst.easyGuideWordInfo.const_5}
        };
    elseif (_tp == ConstantNames.EasyGuide_Explore) then
        this.easyGuideInfo = {
            {GuideConst.easyGuideWordInfo.const_6, GuideConst.easyGuideWordInfo.const_7}, 
            {GuideConst.easyGuideWordInfo.const_8, GuideConst.easyGuideWordInfo.const_9}
        };
    end

    eGGO = GameObject.Find("EasyGuideGO");
    if (not eGGO) then
        eGGO = CreateObject(Util.LoadPrefab("UI/EasyGuide", AssetBundleInfo.assetBundles));
        eGGO.name = "EasyGuideGO";
        HangObject(eGGO, GameObject.Find("UIRoot"));
        this.sortLayerOrder(eGGO, LayerName.Top, true);
    end

    local function easyGuideNext () 
        if (this.easyGuideIndex > #this.easyGuideInfo) then
            DestroyObject(eGGO);
            this.easyGuideIndex = 1;
            this.easyguideInfo = {};
            return ;
        end

        local npcWord = GetGameObject(eGGO.transform, "Container/TalkPanel/guideWord");
        local npcWordLabel = GetComponent(npcWord, "UILabel");
        local circleEff = GetGameObject(eGGO.transform, "Container/CircleEff");
        local container = GetGameObject(eGGO.transform, "Container");

        npcWordLabel.text = this.easyGuideInfo[this.easyGuideIndex][1];
        
        if (this.easyGuideInfo[this.easyGuideIndex][2]) then
            local tarGO = GameObject.Find("UIRoot/" .. this.easyGuideInfo[this.easyGuideIndex][2]);

            if (circleEff and tarGO) then
                circleEff:SetActive(true);
                circleEff.transform.position = tarGO.transform.position;
            else
                circleEff:SetActive(false);
            end
        else
            circleEff:SetActive(false);
        end
        
        if (container) then
            ViewUtil.AddClickEvent(container, function (_go) 
                this.easyGuideIndex = this.easyGuideIndex + 1;
                easyGuideNext();
            end);
        end
    end

    easyGuideNext();
end

function ViewUtil.comAnnouncePanel (_cfmWords, _okFunc, _okWords)
    local AnnouncePrefab, AnnounceObj, AnnouncePanel, root, sMask = nil, nil, nil, nil, nil;
    local okFunc = nil;

    okFunc = function () 
        if (nil ~= sMask) then
            DestroyObject(sMask);
        end
        if (nil ~= _okFunc) then
            _okFunc();
        end
        DestroyObject(AnnounceObj);
    end

    sMask = ViewUtil.showScreenMask();
    GetComponent(sMask, "UIPanel").depth = 0;
    ViewUtil.sortLayerOrder(sMask, LayerName.Dialog);

    AnnouncePrefab = Util.LoadPrefab("UI/AnnouncePanel", AssetBundleInfo.assetBundles);
    AnnounceObj = CreateObject(AnnouncePrefab);
    root = ViewUtil.getRootView();
    HangObject(AnnounceObj, root);
    AnnounceObj.transform.localPosition = Vector3.New(-1, -50, 0);
    ViewUtil.sortLayerOrder(AnnounceObj, LayerName.Top);
    
    AnnouncePanel = ComAnnouncePanel.new();
    AnnouncePanel:Awake(AnnounceObj);
    AnnouncePanel:initInfo(_cfmWords, okFunc, _okWords);
    PanelManager:BindPanelScript(AnnouncePanel, AnnounceObj, doNthFunc, doNthFunc, doNthFunc, doNthFunc);
end