require "Game/FB/View/HeroDrop"

FBView = class(BaseView)

local g_localTime = 0
local g_deltaTime = 0 -- 离开到返回的时间
local g_hero_pos = 250
local g_boss_pos = 250
local g_height_pos = -40
local g_dest_pos = Vector3.New(320, 100, 0)
local g_boss_space = 60
local g_hero_space = 50
local g_hero_space2 = 120
local g_boss_speed = 100
local g_hero_speed = 60

function FBView:ctor()
end  

function FBView:Awake(obj)
	self.gameObject = obj
	self.transform = obj.transform
	self.chestCount = 0
	self.preTime = 0
	self.curTime = 0
	self.hasSend = true
	self.heroProps = Config.getHerosData()
	self.bossData = Config.getBossData()
	self.gatesData = Config.getGatesData()

	self.bossList = {}
	self.runedBoss = {}
	self.staying1 = false
	self.sumTime1 = 0
	self.tmpTime1 = 0

	self.heroList = {}
	self.runedHero = List.new()
	self.staying2 = false
	self.sumTime2 = 0
	self.tmpTime2 = 0
	
	self.bossObj = nil
	self.bossSkeleton = nil
	self.heroObj = nil
	self.heroSkeleton = nil

	self:initPanel()
end

function FBView:initLayers()
	ViewUtil.sortLayerOrder(self.expObj, LayerName.UI)
	ViewUtil.sortLayerOrder(self.fbFG, LayerName.UI)
	ViewUtil.sortLayerOrder(self.tpPanel, LayerName.UI)
	ViewUtil.sortLayerOrder(self.fbPanel, LayerName.UI)
	ViewUtil.sortLayerOrder(self.btPanel, LayerName.UI)
end

function FBView:OnFocus(preMod)
	self.preTime = 0
	self.curTime = 0
	self:initBosses()
	self:initHeros()
	self:initLayers()

	if preMod ~= ModuleName.SPACE then
		FBData.ResetHangUpGate()
	end

	local msgBox = GetGameObject(self.gameObject.transform, "FightMsgBox")
	if msgBox then DestroyObject(msgBox) end

	FBControler.request()
	UpdateHelper.addUpdate(FBControler.Update)

	if AudioConst.GetBgmName() ~= "main" then
        SimpleAudio.StopMusic(doNthFunc)
        SimpleAudio.PlayMusic("main")
        AudioConst.SetBgmName("main")
    end
end

function FBView:Start()
end

function FBView:LostFocus()
	UpdateHelper.removeUpdate(FBControler.Update)
	self:clearBosses()
	self:clearHeros()
end

function FBView:Update()
	self.curTime = Time.timeSinceLevelLoad
	if not self.preTime or self.preTime <= 0 then
		self.preTime = self.curTime
	end
	local deltaTime = self.curTime - self.preTime
	self.preTime = self.curTime

	local hu = FBData.GetHangUpGate()
	local gateId = FBData.GetGateId(hu.chapterId, hu.houseId)
	local hardLevel = hu.hardLevel
	local isPassed = hu.isPassed
	do
		if isPassed then
			FBData.SetHangGateTime(0)
		else
			local gt = FBData.GetHangGateTime() - deltaTime
			FBData.SetHangGateTime(gt)
			if FBData.GetHangGateTime() <= 0 then self:updateFightTitle() end
		end
		self.gateTime.text = Time2String(FBData.GetHangGateTime())
		local gtRate, btRate = FBData.GetHangTimeProcess()
		self.process.barSize = gtRate
	end
	do
		if self.chestCount < 3 then
			self.chestTime.text = Time2String(FBData.GetHangBoxTime())
			if self.hasSend ~= true then
				self.chestProcess.value = FBData.GetChestTimeProcess(0)
			else
				self.chestProcess.value = FBData.GetChestTimeProcess(1)
			end			
		else
			self.chestTime.text = Time2String(FBData.GetCfgBoxTime()) --这里需要确认策划想要什么效果 by iorinie
			self.chestProcess.value = 1
		end
		local bt = FBData.GetHangBoxTime() - deltaTime
		FBData.SetHangBoxTime(bt)
		if bt <= 0 and not self.hasSend then
			self.hasSend = true
			self:sendGate(gateId, hardLevel)
		end
		
	end
	
	self:updateBoss(deltaTime)
	self:updateHero(deltaTime)
end

function FBView:isLessHero()
	local n = 0
	for _, obj in ipairs(self.heroList) do
		if obj ~= nil and obj.activeSelf == true then
			n = n + 1
		end
	end
	if n < 5 then
		return true
	end
	return false
end

function FBView:updateBoss(deltaTime)
	if not self.staying1 and self.heroObj then
		self.sumTime1 = self.sumTime1 + deltaTime
		local delta = self.sumTime1 - self.tmpTime1

		for i = 1, #self.bossList do
			local obj = self.bossList[i]
			local _id = 0
			local lb = GetComponent(obj, "LuaBehaviour")
			if lb then
				local tbl = lb.sourceLuaObj
				_id = tbl.id or 0
			end

			local deltaS = g_boss_speed * deltaTime
--			if delta > i and not IsMember(_id, self.runedBoss) then
			if obj.activeSelf == true then
				if not self.bossObj then
					obj.transform.localPosition = Vector3.New(obj.transform.localPosition.x - deltaS, g_height_pos, 0)
				end
				if obj.transform.localPosition.x < 100 then
					self.staying1 = true
					self.bossObj = obj
					self.bossSkeleton = GetPathComponent(obj.transform, "body", "SkeletonAnimation")
				end
			end
		end
		if #self.bossList == #self.runedBoss then
			self.tmpTime1 = self.sumTime1
			self.runedBoss = {}
			self.bossObj = nil
			self.bossSkeleton = nil
		end
	end
end

function FBView:updateHero(deltaTime)
	if not self.staying2 then
		self.sumTime2 = self.sumTime2 + deltaTime
		local delta = self.sumTime2 - self.tmpTime2

		for i = 1, #self.heroList do
			local obj = self.heroList[i]
			local _id = 0
			local lb = GetComponent(obj, "LuaBehaviour")
			if lb then
				local tbl = lb.sourceLuaObj
				_id = tbl.id or 0
			end
			local deltaS = g_hero_speed * deltaTime
			local bool = true
			if bool == true and obj.activeSelf == true then
				if not self:isLessHero() or not self.heroObj then
					obj.transform.localPosition = Vector3.New(obj.transform.localPosition.x + deltaS, g_height_pos, 0)
				end
				if obj.transform.localPosition.x > -160 then
					self.animator1.speed = 1
					self.animator2.speed = 1
					self.animator3.speed = 1
				end
				if obj.transform.localPosition.x > 20 and not self.runedHero:contains(_id) then
					self.expObj:SetActive(true)
					self.staying2 = true
					self.heroObj = obj
					self.heroSkeleton = GetPathComponent(obj.transform, "body", "SkeletonAnimation")
				end
			end
		end
	end

	if not self.skilling then
		if self.staying1 and self.staying2 then
			if self.bossObj ~= nil and self.heroObj ~= nil then
				self.skilling = true
				if not self.hitEffect then
					self.hitEffect = CreateObjectByPath("Effects/FBDH/fuben_hit", AssetBundleInfo.assetBundles)
					HangObject(self.hitEffect, self.heroObj)
					ViewUtil.sortLayerOrder(self.hitEffect, LayerName.UI, true)
					self.hitEffect.transform.localPosition = Vector3.New(-30, 40, 0)
				end
				self.hitEffect:SetActive(true)
				
				if not self.actorFly then
					self.actorFly = CreateObjectByPath("Effects/FBDH/ActorFly", AssetBundleInfo.assetBundles)
					self.place = GetGameObject(self.actorFly.transform, "ActorFly/yueqi")
					self.animator = GetPathComponent(self.actorFly.transform, "ActorFly", "Animator")
					self.cycle = self.animator:GetCurrentAnimatorStateInfo(0).length
				end

				self.delay2 = DelayCall(function()
					local deathEff = CreateObjectByPath("Effects/FBDH/DeathEff", AssetBundleInfo.assetBundles)
					HangObject(deathEff, self.heroPanel, 1)
					ViewUtil.sortLayerOrder(deathEff, LayerName.UI, true)
					deathEff.transform.localPosition = Vector3.New(100, 20, 0)

					HangObject(self.bossObj, self.place)
					HangObject(self.actorFly, self.heroPanel)
					self.animator:Play(0)
				end, 0.1)
				
				local _bossObj = self.bossObj
				self.delay1 = DelayCall(function()
					if self.hitEffect then
						self.hitEffect:SetActive(false)
					end
					
					if _bossObj ~= nil then
						local lb = GetComponent(_bossObj, "LuaBehaviour")
						local tbl = lb.sourceLuaObj
						table.insert(self.runedBoss, tbl.id)
						HangObject(_bossObj, self.heroPanel)
						self.bossSkeleton.AnimationName = AnimationName.IDLE
--						local lastId = tbl.id - 1
--						if lastId < 1 then
--							lastId = #self.bossList
--						end
						local function getLastPos()
							local lastPos = nil
							for i, obj in ipairs(self.bossList) do
								if obj and obj.activeSelf then
									local pos = obj.transform.localPosition.x
									if lastPos == nil then
										lastPos = pos
									else
										if pos > lastPos then
											lastPos = pos
										end
									end
								end
							end
							return lastPos
						end
						local lastPos = getLastPos() -- self.bossList[lastId].transform.localPosition.x
						_bossObj.transform.localPosition = Vector3.New(lastPos + g_boss_space, g_height_pos, 0)
						_bossObj.transform.localRotation = Quaternion.identity
						_bossObj.transform.localScale = Vector3.New(-1, 1, 1)
														
						local _heroObj = self.heroObj
						local _heroSkeleton = self.heroSkeleton
						_heroSkeleton.AnimationName = AnimationName.IDLE
						local lb = GetComponent(_heroObj, "LuaBehaviour")
						local tbl = lb.sourceLuaObj
						self.runedHero:add(tbl.id)
						local function _onupdate(luaObj, value)
							local render = _heroSkeleton
							if render ~= nil then
								render.skeleton.a = value
							end
						end
						local from = 1
						local to = 0
						if self:isLessHero() then
							to = 1
						end
						iTween.ValueTo(_heroObj, iTween.Hash(
							"from", from,
							"to", to,
							"time", 0.2,
							"easeType", iTween.EaseType.linear,
							"onupdate", _onupdate,
							"oncomplete", function()
								local function getLastPos()
									local lastPos = nil
									for i, obj in ipairs(self.heroList) do
										if obj and obj.activeSelf then
											local pos = obj.transform.localPosition.x
											if lastPos == nil then
												lastPos = pos
											else
												if pos < lastPos then
													lastPos = pos
												end
											end
										end
									end
									return lastPos
								end

								local lastPos = getLastPos() -- heroObj.transform.localPosition.x
								if not self:isLessHero() then
									_heroObj.transform.localPosition = Vector3.New(lastPos - g_hero_space, g_height_pos, 0)
								end
								_heroSkeleton.skeleton.a = 1
										
								-- 技能英雄next
								if not self:isLessHero() then
									local id = tbl.id + 1
									if id > #self.heroList then id = 1	end
									if self.hitEffect then
										HangObject(self.hitEffect, self.heroList[id])
										self.hitEffect.transform.localPosition = Vector3.New(-30, 40, 0)
										ViewUtil.sortLayerOrder(self.hitEffect, LayerName.UI, true)
									end
								end

								self.skilling = false
								self.runedHero:remove(tbl.id, true)
							end,
							"luaobj", self))

						self.staying1 = false
						self.bossObj.transform.localRotation = Quaternion.identity
						self.bossObj = nil

						self.staying2 = false
						if not self:isLessHero() then
							self.heroObj = nil
						end
						self.expObj:SetActive(false)
					end
				end, self.cycle)
			end
		end
	end
end

function FBView:initBosses(resetPos)
	if resetPos then
		for i = 1, #self.bossList do
			local heroObj = self.bossList[i]
			local lb = GetComponent(heroObj, "LuaBehaviour")
			local tbl = lb.sourceLuaObj
			local id = tonumber(tbl.id) or 1
			heroObj.transform.localPosition = Vector3.New(g_boss_pos + g_boss_space * (id - 1), g_height_pos, 0)
		end
	else
		self:clearBosses()
		local i = 0
		for m = 1, 5 do
			local gateId = FBData.GetGateId(FBData.GetCurChapterId(), m)
			local info = self.gatesData[tostring(gateId)]
			local npcId = tostring(info[FBData.GetCurHardLevel()]["npc_id"])
			local data = self.bossData[npcId]
			local heroIds = split(data["heros"], ",")
			for n = 1, #heroIds do
				i = i + 1
				local heroProp = self.heroProps[tostring(heroIds[n])]
				if not IsEmpty(heroProp) then
					local skinName = heroProp["#prefab_name"]					
					local heroObj = nil
					for _, obj in ipairs(self.bossList) do
						if not obj.activeSelf then
							local _, name, _ = GetActorPrefab(skinName)
							if obj.name == name then
								local lb = GetComponent(obj, "LuaBehaviour")
								local src = lb.sourceLuaObj
								src.id = i
								src.pn = skinName
								RefreshActor(obj, skinName)
								obj:SetActive(true)
								heroObj = obj
								break
							end
						end
					end
					if heroObj == nil then
						local go = GetActorObject(skinName)
						if go ~= nil then
							heroObj = go
						else
							local prefab, name = CreateActorPrefab(skinName)
							if not prefab then 
								error("Not Found PrefabName = %s", skinName)
								prefab = CreateActorPrefab("wangzhaojun")
							end
							heroObj = CreateObject(prefab)
							heroObj.name = name
						end
						BindLuaScript({id = i, name = skinName}, heroObj)
						table.insert(self.bossList, heroObj)
					end

					local body = GetGameObject(heroObj.transform, "body")
					local skeletonAnimation = GetComponent(body.transform, "SkeletonAnimation")
					skeletonAnimation.loop = true
					skeletonAnimation.AnimationName = AnimationName.IDLE

					HangObject(heroObj, self.heroPanel)
					heroObj.transform.localPosition = Vector3.New(g_boss_pos + g_boss_space * (i - 1), g_height_pos, 0)
					heroObj.transform.localScale = Vector3.New(-1, 1, 1)
					ViewUtil.sortLayerOrder(heroObj, LayerName.UI, true)
				end
			end
		end
	end
end

function FBView:initHeros(resetPos)
	if resetPos then
		for i = 1, #self.heroList do
			local heroObj = self.heroList[i]
			local lb = GetComponent(heroObj, "LuaBehaviour")
			local tbl = lb.sourceLuaObj
			local id = tonumber(tbl.id) or 1
			heroObj.transform.localPosition = Vector3.New(-g_hero_pos - g_hero_space * (id - 1), g_height_pos, 0)
		end
	else
		self:clearHeros()
		local heros = TeamData.GetHerosByIndex(TeamData.GetCurPageIndex())
		for i = 1, #heros do
			local hero = heros[i]
			local heroProp = self.heroProps[tostring(hero.heroId)]
			if not IsEmpty(heroProp) then
				local skinName = heroProp["#prefab_name"]
				local heroObj = nil
				for _, obj in ipairs(self.heroList) do
					if not obj.activeSelf then
						local _, name, _ = GetActorPrefab(skinName)
						if obj.name == name then
							local lb = GetComponent(obj, "LuaBehaviour")
							local src = lb.sourceLuaObj
							src.id = i
							src.pn = skinName
							RefreshActor(obj, skinName)
							obj:SetActive(true)
							heroObj = obj
							break
						end
					end
				end
				if heroObj == nil then
					local go = GetActorObject(skinName)
					if go ~= nil then
						heroObj = go
					else
						local prefab, name = CreateActorPrefab(skinName)
						if not prefab then 
							error("Not Found PrefabName = %s", skinName)
							prefab = CreateActorPrefab("wangzhaojun")
						end
						heroObj = CreateObject(prefab)
						heroObj.name = name
					end
					BindLuaScript({id = i, name = skinName}, heroObj)
					table.insert(self.heroList, heroObj)
				end

				local body = GetGameObject(heroObj.transform, "body")
				local skeletonAnimation = GetComponent(body.transform, "SkeletonAnimation")
				skeletonAnimation.loop = true
				skeletonAnimation.AnimationName = AnimationName.IDLE
			
				HangObject(heroObj, self.heroPanel)
				heroObj.transform.localPosition = Vector3.New(-g_hero_pos - g_hero_space * (i - 1), g_height_pos, 0)
				ViewUtil.sortLayerOrder(heroObj, LayerName.UI, true)
			end
		end
	end
end

function FBView:resetFBDH()
	self.preTime = 0
	self.curTime = 0
	
	self.sumTime1 = 0
	self.tmpTime1 = 0
	self.staying1 = false
	self.bossObj = nil
	self.bossSkeleton = nil

	CancelDelayCall(self.delay1)
	CancelDelayCall(self.delay2)
	self.sumTime2 = 0
	self.tmpTime2 = 0
	self.staying2 = false
	self.heroObj = nil
	self.heroSkeleton = nil
	self.animator1.speed = 0
	self.animator2.speed = 0
	self.animator3.speed = 0
	self.skilling = false
	self.expObj:SetActive(false)
	DestroyObject(self.hitEffect)
	self.hitEffect = nil

	self:initBosses(true)
	self:initHeros(true)
	self:initLayers()
end

function FBView:clearBosses()
	for i = #self.bossList, 1, -1 do
		local hero = self.bossList[i]
		hero:SetActive(false)
	end
	self.runedBoss = {}
	self.sumTime1 = 0
	self.tmpTime1 = 0
	self.staying1 = false
	self.bossObj = nil
	self.bossSkeleton = nil
end

function FBView:clearHeros()
	CancelDelayCall(self.delay1)
	CancelDelayCall(self.delay2)

	for i = #self.heroList, 1, -1 do
		local hero = self.heroList[i]
		hero:SetActive(false)
	end
	self.runedHero = List.new()
	self.sumTime2 = 0
	self.tmpTime2 = 0
	self.staying2 = false
	self.heroObj = nil
	self.heroSkeleton = nil

	self.animator1.speed = 0
	self.animator2.speed = 0
	self.animator3.speed = 0
	self.skilling = false
	
	self.expObj:SetActive(false)
	DestroyObject(self.hitEffect)
	self.hitEffect = nil
end

function FBView:initPanel()
 	self.top = GetPathComponent(self.transform, "Mode/Top", "UIWidget")
 	self.bottom = GetPathComponent(self.transform, "Mode/Bottom", "UIWidget")
	self.top:SetAnchor(self.root.transform)
	self.bottom:SetAnchor(self.root.transform)

	local fbPath = "Mode/Top/Panel/FBInfo"
 	self.fbName = GetPathComponent(self.transform, fbPath.."/GateTitle", "UILabel")
	
	self.btnsLevel = {}
	self.btnsLevel[1] = GetGameObject(self.transform, fbPath.."/CallBG/BtnSoldier")
	self.btnsLevel[2] = GetGameObject(self.transform, fbPath.."/CallBG/BtnGeneral")
	self.btnsLevel[3] = GetGameObject(self.transform, fbPath.."/CallBG/BtnKing")
	
	self.dropIcons = {}
	self.dropIcons[1] = GetGameObject(self.transform, fbPath.."/CallBG/DropPanel/DropIcon1")
	self.dropIcons[2] = GetGameObject(self.transform, fbPath.."/CallBG/DropPanel/DropIcon2")
	self.dropIcons[3] = GetGameObject(self.transform, fbPath.."/CallBG/DropPanel/DropIcon3")

 	self.gateNum = GetPathComponent(self.transform, fbPath.."/nameBG/gateNum", "UILabel")
 	self.gateName = GetPathComponent(self.transform, fbPath.."/nameBG/gateName", "UILabel")
 	self.gateTime = GetPathComponent(self.transform, fbPath.."/nameBG/gateTime", "UILabel")
 	self.goldNum = GetPathComponent(self.transform, fbPath.."/GoldNum", "UILabel")
 	self.expNum = GetPathComponent(self.transform, fbPath.."/ExpNum", "UILabel")
 	self.chestNumObj = GetPathComponent(self.transform, fbPath.."/Chest/ChestNum", "UILabel")
 	self.chestTime = GetPathComponent(self.transform, fbPath.."/Chest/ChestTime", "UILabel")
	self.chestBox = GetGameObject(self.transform, fbPath.."/Chest/Box")
	self.chestProcess = GetPathComponent(self.transform, fbPath.."/Chest/process", "UIProgressBar")
	self.chestProcess.value = 0

	self.process = GetPathComponent(self.transform, fbPath.."/nameBG/process", "UIScrollBar")
	self.process.barSize = 0
	
	self.fbBG = GetGameObject(self.transform, "Mode/Center/BGPanel")
	self.heroPanel = GetGameObject(self.fbBG.transform, "Heros")
	self.fbFG = GetGameObject(self.transform, "Mode/Center/FGPanel")
	self.fbPanel = GetGameObject(self.transform, "Mode/Center/Panel")
	self.tpPanel = GetGameObject(self.transform, "Mode/Top")
	self.btPanel = GetGameObject(self.transform, "Mode/Bottom")
		
	self.fbDH = GetGameObject(self.transform, "Mode/Center/BGPanel/FBDH")
	self.expObj = GetGameObject(self.fbDH.transform, "diaoluowupin")
	self.expObj:SetActive(false)

	self.animator1 = GetPathComponent(self.fbDH.transform, "runmap1", "Animator")
	self.animator2 = GetPathComponent(self.fbDH.transform, "runmap2", "Animator")
	self.animator3 = GetPathComponent(self.fbDH.transform, "runmap3", "Animator")
	
	local centerPath = "Mode/Center/Panel"
 	self.fightBtn = GetGameObject(self.transform, centerPath.."/FightBtn")
 	self.fightNow = GetGameObject(self.transform, centerPath.."/FightBtn/FightNow")
 	self.fightNext = GetGameObject(self.transform, centerPath.."/FightBtn/FightNext")
 	self.fightReady = GetGameObject(self.transform, centerPath.."/FightBtn/FightReady")
	self:updateFightTitle()
	
	self.btnsGate = {}
 	self.btnsGate[1] = GetGameObject(self.transform, centerPath.."/fightBG/house1")
 	self.btnsGate[2] = GetGameObject(self.transform, centerPath.."/fightBG/house2")
 	self.btnsGate[3] = GetGameObject(self.transform, centerPath.."/fightBG/house3")
 	self.btnsGate[4] = GetGameObject(self.transform, centerPath.."/fightBG/house4")
 	self.btnsGate[5] = GetGameObject(self.transform, centerPath.."/fightBG/house5")
	for i = 1, 5 do
		local btn = self.btnsGate[i]
		Util.EnableButton(btn, ButtonState.Disabled, false)
	end

	self.iconsGate = {}
 	self.iconsGate[1] = GetPathComponent(self.transform, centerPath.."/fightBG/house1", "UIButton")
 	self.iconsGate[2] = GetPathComponent(self.transform, centerPath.."/fightBG/house2", "UIButton")
 	self.iconsGate[3] = GetPathComponent(self.transform, centerPath.."/fightBG/house3", "UIButton")
 	self.iconsGate[4] = GetPathComponent(self.transform, centerPath.."/fightBG/house4", "UIButton")
 	self.iconsGate[5] = GetPathComponent(self.transform, centerPath.."/fightBG/house5", "UIButton")

	local listPath = "Mode/Bottom/Panel/ListBG"
 	self.space = GetGameObject(self.transform, listPath.."/Space")
 	self.package = GetGameObject(self.transform, listPath.."/Package")
 	self.intensify = GetGameObject(self.transform, listPath.."/Intensify")
 	self.hintPoint = GetGameObject(self.transform, listPath.."/Space/HintPoint")
	self:showHintPoint()
	
	local function doClick(obj)
		if obj == self.fightBtn then
			if FBData.GetCurHardLevel() <= FBData.GetLastHardLevelRPC() then
				local passed = FBData.IsPassedCurGate()
				if not passed then
					if FBData.GetHangGateTime() <= 0 then
						self:createFightMsgBox()
						if (PlayerData.guideNotEnd()) then
							Facade:sendNotification(MVCConst.GUIDE_NEXT, {});
						end
					else
						ViewUtil.showMsg("倒计时未结束!")
					end
					local hu = FBData.GetHangUpGate()
					if hu.chapterId ~= FBData.GetCurChapterId() or hu.hardLevel ~= FBData.GetCurHardLevel() then
						FBData.ResetHangUpGate()
						FBControler.request()
					end
				else
					if FBData.GetCurHouseId() >= 5 and FBData.IsPassedChapter(FBData.GetCurChapterId()) then
						if FBData.checkPower(FBData.GetCurChapterIdRPC() + 1) then
							self.isNewChapter = true -- 必须置顶

                            if (Global.openGuide) then
                                if (
                                    10105 == FBData.GetCurGateId() and FBData.IsPassedGate(10105, 1) and PlayerData.getMyGuideStep() < 330 or 
                                    10205 == FBData.GetCurGateId() and FBData.IsPassedGate(10205, 1) and PlayerData.getMyGuideStep() < 450 or 
                                    10305 == FBData.GetCurGateId() and FBData.IsPassedGate(10305, 1) and PlayerData.getMyGuideStep() < 500
                                    ) then
                                    --防止1-5、2-5、3-5之后，快速操作引起的引导出现的问题
                                    return ;
                                end
                            end
							FBData.IncChapterId()
							FBControler.request()
                            --add by library 2016-11-16 12:16:25 更新挂机关卡
                            FBData.guideSetCurHangHouseInfo(FBData.guideCurChapterId + 1,(FBData.guideCurChapterId + 1) * 100 + 1, 1);
						end
						self:resetFBDH()
					else
						FBData.ResetHangUpGate()
						self:refreshTabLevel()
						FBData.IncCurHouseId()
                        if (Global.openGuide) then
                            if (
                                10104 == FBData.GetCurGateId() and FBData.IsPassedGate(10104, 1) and PlayerData.getMyGuideStep() < 250 or 
                                10202 == FBData.GetCurGateId() and FBData.IsPassedGate(10202, 1) and PlayerData.getMyGuideStep() < 390
                                ) then
                                --防止1-4、2-2之后，快速操作引起的引导出现的问题
                                return ;
                            end
                        end
						self:slideTabGate(FBData:GetCurHouseId(), true, true)
					end
				end
			else
				ViewUtil.showMsg("请先通过之前难度")
			end

		elseif obj == self.space then
			ViewUtil.openView(ModuleName.SPACE)
            if (PlayerData.guideNotEnd()) then
                Facade:sendNotification(MVCConst.GUIDE_NEXT, {});
            end

		elseif obj == self.package then
            ViewUtil.openView(ModuleName.PROP, true);
            if (PlayerData.guideNotEnd()) then
                Facade:sendNotification(MVCConst.GUIDE_NEXT, {});
            end

		elseif obj == self.intensify then
			-- 直接传方法执行会有一部分变量没赋值，导致显示异常。 by iorinie
			ViewUtil.openView(ModuleName.HU, false, function(args)
				-- local ctrl = ModuleManager.GetCtrl(ModuleName.HU)
				-- ctrl.slideTab(1)
				if (PlayerData.guideNotEnd()) then
	                Facade:sendNotification(MVCConst.GUIDE_NEXT, {});
	            end
	        end)
		else
			do
				local b1, k1 = IsMember(obj, self.btnsLevel)
				if b1 then
					if k1 > FBData.GetLastHardLevelRPC() then
						ViewUtil.showMsg("请先通过当前难度")
					else
						local fbId = FBData.GetCurChapterIdRPC()
						local hardLevel = FBData.CheckHardLevel(fbId, k1)
						self:refreshData(fbId, hardLevel)
					end
				end
			end
			do
				local b2, k2 = IsMember(obj, self.btnsGate)
				if b2 then
					self:slideTabGate(k2, true, true)
				end
			end
		end
    end

	ViewUtil.AddClickEvent(self.fightBtn, doClick)
	ViewUtil.AddClickEvent(self.space, doClick)
	ViewUtil.AddClickEvent(self.package, doClick)
	ViewUtil.AddClickEvent(self.intensify, doClick)
	for _, btn in ipairs(self.btnsLevel) do
		ViewUtil.AddClickEvent(btn, doClick)
	end
	for _, btn in ipairs(self.btnsGate) do
		ViewUtil.AddClickEvent(btn, doClick)
	end
	
	local function doClickChesk(obj)
		local function _call(session, tbl)
			local mo = GetProtoTable("DefaultData", session)
			local gifts = split(mo.msg, ",")
			if IsEmptyTable(gifts) then
				ViewUtil.showMsg("还没有奖励")
			else
				self.chestCount = CountTable(gifts)
				FBData.SetRewardsCount(self.chestNumObj, self.chestCount)

				local paneAward = CreateObjectByPath("UI/FBAwardPane", AssetBundleInfo.assetBundles)
				HangObject(ViewUtil.CreateShadeView(function(args)
					DestroyObject(paneAward)
				end), paneAward)
				HangObject(paneAward, self.gameObject)
				ViewUtil.sortLayerOrder(paneAward, LayerName.Dialog, true)

				local centerPath = "Mode/Center/Panel"
				local scvObj = GetGameObject(paneAward.transform, centerPath.."/ScrollViewBG/ScrollView")
				local gridObj = GetGameObject(scvObj.transform, "Panel/Grid")
				local btnClose = GetGameObject(paneAward.transform, centerPath.."/BtnClose")
				local btnGain = GetGameObject(paneAward.transform, centerPath.."/BtnGain")
			
				local function _doClose(obj)
					if obj == btnClose then
						DestroyObject(paneAward)
					elseif obj == btnGain then
						Network.SendMoData(ServerID.GainRewards, {})
						ViewUtil.showMsg("领取成功")
						self.chestCount = 0
						FBData.SetRewardsCount(self.chestNumObj, self.chestCount)
						DestroyObject(paneAward)
						self:sendGate(FBData.GetCurGateId(), FBData.GetCurHardLevel())
					end
					if (PlayerData.guideNotEnd()) then
						Facade:sendNotification(MVCConst.GUIDE_NEXT, {});
					end
				end
				-- UIEventListener.Get(btnClose).onClick = _doClose
				-- UIEventListener.Get(btnGain).onClick = _doClose
				ViewUtil.AddClickEvent(btnClose, _doClose)
				ViewUtil.AddClickEvent(btnGain, _doClose)

				for _, data in pairs(gifts) do
					local info = split(data, ":")
					if not IsEmpty(info) then
						local id = tostring(info[1])
						local num = tonumber(info[2])
						ViewUtil.CreateDropItem(id, num, gridObj)
--						local item = CreateObjectByPath("UI/ItemDrop")
--						local sprComp = GetPathComponent(item.transform, "Sprite", "UISprite")
--						local numComp = GetPathComponent(item.transform, "Num", "UILabel")
--						sprComp.spriteName = id
--						numComp.text = num
--						HangObject(item, gridObj)
					end
				end
			end

            if (PlayerData.guideNotEnd()) then
                Facade:sendNotification(MVCConst.GUIDE_NEXT, {});
            end
		end
		Network.SendMoData(ServerID.ChestRewards, {}, _call)
	end
	-- UIEventListener.Get(self.chestBox).onClick = doClickChesk
	ViewUtil.AddClickEvent(self.chestBox, doClickChesk)
end

function FBView:refreshData(fbId, hardLevel)
--	plog("[RefreshData] fbId = %s, hardLevel = %s", fbId, hardLevel)
	FBData.SetCurHardLevel(hardLevel)
	Network.SendMoData(ServerID.FB, {fbId, hardLevel}, function(session, params)
		local serviceId = tonumber(session.serviceId)
		local data = GetProtoTable("TeamDungeonsData", session)
		local dataList = data.infoDataList
		local fbId = FBData.GetChapterIdRPC(data.curIndex)
		FBData.InitTimeList(dataList, hardLevel)
		self:resetData(data, true, true)
		self:slideTabLevel(hardLevel)
	end, {})
end

function FBView:updateFightTitle()
	local state = 1
	if FBData.IsPassedCurGate() then
		state = 2
	elseif FBData.GetHangTimeProcess() >= 1 then
		state = 1
	else
		state = 0
	end
	if state == 0 then
		NGUITools.SetActive(self.fightNow, false)
		NGUITools.SetActive(self.fightNext, false)
		NGUITools.SetActive(self.fightReady, true)
	elseif state == 1 then
		NGUITools.SetActive(self.fightNow, true)
		NGUITools.SetActive(self.fightNext, false)
		NGUITools.SetActive(self.fightReady, false)
	else
		NGUITools.SetActive(self.fightNow, false)
		NGUITools.SetActive(self.fightNext, true)
		NGUITools.SetActive(self.fightReady, false)
	end
end

function FBView:showHintPoint()
	local newChapter = PlayerData.playerData.status.chapter
	if newChapter then
		NGUITools.SetActive(self.hintPoint, true)
	else
		NGUITools.SetActive(self.hintPoint, false)
	end
end

function FBView:createFightMsgBox()
	local msgBox = CreateObjectByPath("UI/FightMsgBox", AssetBundleInfo.assetBundles)
	msgBox.name = "FightMsgBox"
	local shade =  ViewUtil.CreateShadeView(function(args)
--		DestroyObject(msgBox)
	end)
	HangObject(shade, msgBox)
	HangObject(msgBox, self.gameObject)
	ViewUtil.sortLayerOrder(msgBox, LayerName.Dialog, true)
	
	local fight = GetGameObject(msgBox.transform, "Panel/BtnFight")
	local back = GetGameObject(msgBox.transform, "Panel/BtnBack")
	local story = GetPathComponent(msgBox.transform, "Panel/Content", "UILabel")
	local content = FBData.GetFBStory(FBData.GetCurGateId())
	story.text = content
	local story = GetPathComponent(msgBox.transform, "Panel/Content", "UILabel")
	local content = FBData.GetFBStory(FBData.GetCurGateId())
	local title = GetPathComponent(msgBox.transform, "Panel/GateTitle", "UILabel")
	title.text = FBData.GetCurGateTitle()

	local function doClick(obj)
		if obj == fight then
			if Application.isMobilePlatform then -- UMeng
				GA.StartLevel(FBData.GetCurGateTitle())
			end

			Network.ReqFightFB()
			if (PlayerData.guideNotEnd()) then
				Facade:sendNotification(MVCConst.GUIDE_SAVEGUIDE, {})
            end
		end
		DestroyObject(msgBox)
	end
	ViewUtil.AddClickEvent(fight, doClick)
	ViewUtil.AddClickEvent(back, doClick)
end

function FBView:refreshTabLevel()
	local curLevel = FBData.GetCurHardLevel()
	local fbId = FBData.GetCurChapterIdRPC()
	local maxLevel = FBData.GetMaxHardLevel(fbId)
--	plog("##fbId = ", fbId, ", maxLevel = ", maxLevel, ", curLevel = ", curLevel)
	
	for i = 1, 3 do
		local obj = self.btnsLevel[i]
		local fg1 = GetGameObject(obj.transform, "FG1")
		local fg2 = GetGameObject(obj.transform, "FG2")
		if i == curLevel then
			NGUITools.SetActive(obj, true)
			Util.EnableButton(obj, ButtonState.Disabled, false)
			fg1:SetActive(true)
			fg2:SetActive(false)
		elseif i <= maxLevel then
			NGUITools.SetActive(obj, true)
			Util.EnableButton(obj, ButtonState.Normal, true)
			fg1:SetActive(false)
			fg2:SetActive(true)
		else
			NGUITools.SetActive(obj, false)
			Util.EnableButton(obj, ButtonState.Disabled, false)
		end
	end
end

-- 难度
function FBView:slideTabLevel(sw, guideClick)
	self:refreshTabLevel()
	local hu = FBData.GetHangUpGate()
	local curChapterId = FBData.GetCurChapterId()
	local houseId = 0
	if curChapterId == hu.chapterId and sw == hu.hardLevel then
		houseId = hu.houseId
	end
	self:slideTabGate(houseId, guideClick)
end

-- 关卡
function FBView:slideTabGate(sw, guideClick, forceClick)
	local sw = tonumber(sw) or 0
	local curChapter = FBData.GetCurChapterId()
	local curHardLevel = FBData.GetCurHardLevel()
	local rpcHardLevel = FBData.GetLastHardLevelRPC()
	local lastHouse = FBData.GetLastHouseId()
	
	FBData.SetCurHouseId(sw)
	FBData.SetHangUpGate()

    --add by library 2016-11-16 11:59:15 同步新手引导用到的副本数据
    if (forceClick) then
        --手动点击关卡，当前挂机关卡改变，同步新手数据
        FBData.guideSetCurHangHouseInfo(FBData.GetChapterIdRPC2(curChapter), FBData.GetChapterIdRPC2(curChapter) * 100 + sw, curHardLevel);
    end
--	plog("##sw = %d, curChapter = %d, curHardLevel = %d, rpcHardLevel = %d, lastHouse = %d", sw, curChapter, curHardLevel, rpcHardLevel, lastHouse)
	
	self.chestCount = 0
	for i = 1, 5 do
		local btn = self.btnsGate[i]
		Util.ResetButton(btn)
		self.iconsGate[i].normalSprite = "gate"..FBData.GetCurChapterId()
		self.iconsGate[i].pressedSprite = "gate"..FBData.GetCurChapterId()
		if curHardLevel == rpcHardLevel then
			if i <= lastHouse then
				if i == sw then
					self.gateNum.text = FBData.GetGateTitle(curChapter, i)
					self.gateName.text =  FBData.GetGateName(FBData.GetCurGateId(), FBData.GetCurHardLevel())
					Util.EnableButton(btn, ButtonState.Pressed, true)
					self:sendGate(FBData.GetGateIdRPC(curChapter, i), curHardLevel, guideClick)
				else
					Util.EnableButton(btn, ButtonState.Normal, true)
				end
			else
				Util.EnableButton(btn, ButtonState.Disabled, false)
			end

		elseif curHardLevel < rpcHardLevel then
			if i == sw then
				self.gateNum.text = FBData.GetGateTitle(curChapter, i)
				self.gateName.text = FBData.GetGateName(FBData.GetCurGateId(), FBData.GetCurHardLevel())
				Util.EnableButton(btn, ButtonState.Pressed, true)
				self:sendGate(FBData.GetGateId(curChapter, i), curHardLevel, guideClick)
			else
				Util.EnableButton(btn, ButtonState.Normal, true)
			end
		else
			Util.EnableButton(btn, ButtonState.Disabled, false)
		end
	end

	self:updateGateInfo()

	if forceClick then
		self:resetFBDH()
	end
end

function FBView:updateRewardsCount(callback)
--	FBData.SetRewardsCount(self.chestNumObj, self.chestCount) -- 不清缓存
	Network.SendMoData(ServerID.ChestRewards, {}, function(session, tbl)
		local mo = GetProtoTable("DefaultData", session)
		local gifts = split(mo.msg, ",")
		local count = CountTable(gifts)
		self.chestCount = count
		local code = tonumber(mo.code)
		FBData.SetRewardsCount(self.chestNumObj, self.chestCount)
		if callback ~= nil then
			callback(code)
		end
	end)
end

function FBView:updateGateInfo()
	local check = FBData.checkPower(FBData.GetLastChapterIdRPC(), true) 
--	plog("@GetPower = %d, CheckPower = %s", TeamData.GetPower(), tostring(check))
	if check then
		local selId = FBData.GetLastSelectedChapterId()
		if FBData.IsPassedChapter(selId) and selId < FBData.GetLastChapterId() then
			FBData.showChapterTips(true)
		end
	else
		FBData.showChapterTips(false)
	end
	if FBData.GetCurHouseId() > 0 then
		self.goldNum.text = tonumber(FBData.GetCurGateGold()) or 0
		self.expNum.text = tonumber(FBData.GetCurGateExp()) or 0
		self.fbName.text = FBData.GetCurChapterName()
		self.gateNum.text = FBData.GetCurGateTitle()
		self.gateName.text = FBData.GetCurGateName()
	else
		local hu = FBData.GetHangUpGate()
		self.goldNum.text = tonumber(FBData.GetGateGold(hu.chapterId, hu.hardLevel, hu.houseId)) or 0
		self.expNum.text = tonumber(FBData.GetGateExp(hu.chapterId, hu.hardLevel, hu.houseId)) or 0
		local fbId = FBData.GetChapterIdRPC2(hu.chapterId)
		self.fbName.text = FBData.GetChapterName(fbId)
		self.gateNum.text = FBData.GetGateTitle(hu.chapterId, hu.houseId)
		self.gateName.text = FBData.GetGateName(FBData.GetGateId(hu.chapterId, hu.houseId), hu.hardLevel)
	end
	for i = 1, #self.btnsGate do
		local gateText = GetPathComponent(self.btnsGate[i].transform, "gateNum", "UILabel")
		gateText.text =  FBData.GetGateTitle(FBData.GetCurChapterId(), i)
	end
	self:updateRewardsCount()
	self:updateFightTitle()
end

-- 通知后台关卡信息
function FBView:sendGate(gateId, hardLvl, guideClick)
	local mo = {gateId, hardLvl}
	if not IsEmptyTable(mo) then
--		plog("SendGate = {%s, %s}", gateId, hardLvl)
		Network.SendMoData(ServerID.Gate, mo, function(session, params)
			local serviceId = tonumber(session.serviceId)
			local mo = GetProtoTable("DefaultData", session)
--			plog("GateTime mo = ", mo.msg)
			local tbl = split(mo.msg, ",")
			local seconds = tonumber(tbl[1]) or 0
			local chestNum = tonumber(tbl[2]) or 0		-- 累计宝箱次数
			self.chestCount = tonumber(tbl[3]) or 0		-- 当前次数
			FBData.SetRewardsCount(self.chestNumObj, self.chestCount)
--			plog("$$chestNum = %d, hasSend = %s, chestCount = %d, seconds = %d", chestNum, tostring(self.hasSend), self.chestCount, seconds)
			local bt = 0
			if self.hasSend == true then
				if chestNum == 0 then
					bt = FBData.GetCfgFirstBoxTime()
					if seconds > 0 then
						bt = bt - seconds
						if bt <= 0 then
							bt = FBData.GetCfgFirstBoxTime()
						end
					end
				else
					bt = FBData.GetCfgBoxTime()
--					plog("GateId = ", FBData.GetCurGateId(), ", CFGTime = ", cfg, ", GateTime = ", tbl[1])
					if seconds > 0 then
						bt = bt - seconds
						if bt <= 0 then
							bt = FBData.GetCfgBoxTime()
						end
					end
				end
				FBData.SetHangBoxTime(bt)
				if bt > 0 then
					self.hasSend = false
				end
			end
		end)
        
		if guideClick then
			if (PlayerData.guideNotEnd()) then
				LuaCallManager:invokeCall(function () 
					Facade:sendNotification(MVCConst.GUIDE_NEXT, {});
				end, 0.15);
			end
		end
	end
end

function FBView:resetData(data, withoutLevel, notHangUp)
	local dataList = data.infoDataList
	local chapterId = FBData.GetChapterId(data.curIndex)
	local lastLevel = tonumber(data.curLeveL)	or 1				-- 注意后台大小写
	local lastHouse = FBData.GetHouseId(data.curIndex)
	local fbId = FBData.GetChapterIdRPC(data.curIndex)
	local curGateLevel = data.curGateLevel
	if self.isNewChapter then -- 弥补后台新章节第一次数据异常
		self.isNewChapter = false
		curGateLevel = 1
	end
	local curLevel = FBData.CheckHardLevel(fbId, curGateLevel)
	local cfgLevel = FBData.CheckHardLevel(fbId, lastLevel)
--	plog("[ResetData] chapterId = %d, lastLevel = %d, lastHouse = %d, fbId = %d, curGateLevel = %d, curLevel = %d, cfgLevel = %d",
--	 chapterId, lastLevel, lastHouse, fbId, curGateLevel, curLevel, cfgLevel)
	
	FBData.InitTimeList(dataList, curLevel)
	FBData.SetLastGateIdRPC(data.curIndex)
	if not withoutLevel then
		FBData.SetCurHardLevel(curLevel)
		FBData.SetLastHardLevel(cfgLevel)
		FBData.SetLastHardLevelRPC(lastLevel)
	end
	FBData.SetLastHouseId(lastHouse)
	FBData.SetCurDataList(dataList)
	FBData.UpdateRPCInfo(chapterId, lastLevel, lastHouse)

	-- 校验实际难度
	if FBData.GetCurHardLevel() > FBData.GetLastHardLevelRPC() then
		FBData.SetCurHardLevel(FBData.GetLastHardLevelRPC())
	end
	
	local hu = FBData.GetHangUpGate()
	if not FBData.IsPassedChapter(hu.chapterId, hu.hardLevelRPC, hu.houseId2)
	and not FBData.checkPower(hu.chapterIdRPC, true)
	then
--		FBData.DecChapterId(hu.chapterId)
	else
		if not notHangUp then
			FBData.SetHangUpGate()
		end
	end
	
--	plog("@[FBView] CurGateLevel = ", curGateLevel)
--	plog("@[FBView] CurChapterId = ", FBData.GetCurChapterId())
--	plog("@[FBView] CurHardLevel = ", FBData.GetCurHardLevel())
--	plog("@[FBView] CurHouseId = ", FBData.GetCurHouseId())
--	plog("@[FBView] LastChapterId = ", chapterId)
--	plog("@[FBView] LastHardLevel = ", lastLevel)
--	plog("@[FBView] LastHouseId = ", lastHouse)
end

function FBView:initData(data)
	if IsEmpty(data) then return end
	self:resetData(data)
	self:slideTabLevel(FBData.GetCurHardLevel(), false)
	self.fbName.text = FBData.GetCurChapterName()

	local function _clickDropHero(obj)
		local lb = GetComponent(obj, "LuaBehaviour")
		local src = lb.sourceLuaObj
		local conf = src.conf
        HeroPropControler.accept(conf)
        HeroPropControler.openBefore = true;
        ViewUtil.openView(ModuleName.HP, false)
	end
	local fbData = Config.getFBData()
	local fbId = FBData.GetCurChapterIdRPC()
	local charData = fbData[tostring(fbId)]
	if not IsEmptyTable(charData) then
		local heroIds = split(charData.heros, ",")
		for i = 1, #heroIds do
			if i > 0 and i < 4 then
				local ids = split(heroIds[i], ":")
				local heroId = ids[1]
				if not IsEmptyString(heroId) then
--					local heroObj = self.dropIcons[i]
--					if heroObj ~= nil then
--						local lb = GetComponent(heroObj.transform, "LuaBehaviour")
--						local info = lb.sourceLuaObj
--						info:refresh(heroId)
--					else
--					end
					local cls = HeroDrop.new()
					cls:init(heroId)
					local hero = cls:createHero()
					BindLuaScript(cls, hero)
					ViewUtil.AddClickEvent(hero, _clickDropHero)
					DestroyChildren(self.dropIcons[i])
					HangObject(hero, self.dropIcons[i])
				end
			end
		end
	end

    -- if (PlayerData.guideNotEnd()) then
    -- 	Facade:sendNotification(MVCConst.GUIDE_NEXT, {});
    -- end
end

--[[
    新手引导，引导升级之后，修改升级按钮的depth，使其可见
]]
function FBView:sortLvlupBtnDepth ()
    if (self.intensify) then
        ViewManager.SortDepth(self.intensify);
    end
end