--[[
-- Desc: 特效
-- Author: kenny
-- Date: 2016
--]]
EffectAction = class(BaseAction)

function EffectAction:ctor()
	self.startPositionList = List:new()
end

function EffectAction:onBegin()
	self.super.onBegin(self)
	self:doAction()
end

function EffectAction:createEffect(target, pos)
	if not target then return end
	self.isSkillName = false
	self.skillName = self.actionExtra
--	local idx = string.find(self.actionExtra, "_01mz")
--	if idx and idx > 0 then
--		self.isSkillName = true
--		self.actionExtra = "skills_name_01mz"
--	end

	local len = string.len(self.actionExtra)
	local sub = string.sub(self.actionExtra, len - 1, len)
	if sub == "mz" then
		self.isSkillName = true
		self.isSkillGroup = false
		local skillsData = Config.getSkillsData()
		local skillInfo = skillsData[self.skillId] or {}
		local skillHeroIds = split(skillInfo[g_skills_hero_id], ":")
		self.skillTitle = skillInfo[g_skills_file_title] or "无名"
		if #skillHeroIds > 1 then -- 合体技
			self.isSkillGroup = true
			self.actionExtra = "skills_group"
		else
			self.isSkillGroup = false
			self.actionExtra = "skills_single"
		end
	end

	local effpath = "Effects/Skills/"..self.actionExtra
	local effectRes = Util.LoadPrefab(effpath, AssetBundleInfo.assetBundles)
--	plog("Effect File = %s", self.actionExtra)
	if effectRes ~= nil then
		local effectObj = CreateObject(effectRes)
		effectObj.transform.parent = target.transform
		effectObj.transform.localPosition = pos
		effectObj.transform.localRotation = Quaternion.Euler(Vector3.zero)
		effectObj.transform.localScale = Vector3.one

		if self.isSkillName then
--			local particle = GetPathComponent(effectObj.transform, "Particle/Name", "Renderer")
--			local tex2d = Resources.Load("UI/SkillName/"..self.skillName)
--			if tex2d ~= nil then
--				particle.material.mainTexture = tex2d
--			end
			local nameObj = GetGameObject(effectObj.transform, "Name")
			local lbl = GetPathComponent(nameObj.transform, "Label", "UILabel")
			if not self.isSkillGroup then
				effectObj.transform.localPosition = Vector3.New(-141, 216, 0)
				lbl.text = self.skillTitle
			else
				local lbl2 = GetPathComponent(nameObj.transform, "Label2", "UILabel")
				lbl.text = self.skillTitle
				lbl2.text = self.skillTitle
			end
			if self.teamSide == TeamSide.Right then
				nameObj.transform.localScale = Vector3.New(-1, 1, 1)
			else
				nameObj.transform.localScale = Vector3.New(1, 1, 1)
			end
		end

		self.effectObj = effectObj
		self:scaleParticleSize(effectObj)
		self:addActionObject(effectObj)
		self:onSequence(effectObj)
	else
		warn("Not found effect file = %s", self.actionExtra)
	end
end

function EffectAction:doAction()
    self.super.doAction(self)

	if self.startPositionType==PositionType.Custom then
		for i = 1, self.startPositionList.size do
			local pos = self.startPositionList[i]
			local target = FightControler.getCampObject(self.teamSide)
			self:createEffect(target, pos)
		end
	else
		local actorList = self:getActors()		-- 行为主体不为空时，只有相对坐标
		if not IsEmpty(actorList) then
			for _, actor in ipairs(actorList) do
				local pos = Vector3(self.startPosition.x, self.startPosition.y, 0)
				local target = actor:getActorObject()
				self:createEffect(target, pos)
			end
		else
			local pos = Vector3(self.startPosition.x, self.startPosition.y, 0)
			local target = FightControler.getCampObject(self.teamSide)
			self:createEffect(target, pos)
		end
	end
end

-- 序列动画
function EffectAction:onSequence(target)
	if self.lastTime <= 0 then
		local time = Util.GetParticleTime(target)
		if not time or time <= 0 or time > 15 then
			return
		end
		self.endCond[2] = time
		self.lastTime = time
	end
	if IsMember(self.aimPositionType, PositionType) then
		local aimPosX = self.aimPosition.x
		local aimPosY = self.aimPosition.y
		if self.aimPositionType == PositionType.Relative then
			local deltaPos = Vector3(aimPosX, aimPosY, 0)
			local pos = self:getPositionRelative(target, deltaPos)
			aimPosX = pos.x
			aimPosY = pos.y
		end
		
		iTween.MoveTo(target,
			iTween.Hash(
				"position", Vector3(aimPosX, aimPosY, target.transform.localPosition.z),
				"islocal", true,
				"time", self.lastTime,
				"easeType", iTween.EaseType.linear))
	end
end

function EffectAction:onEnd()
    self.super.onEnd(self)
	local list = self:getActionObjects()
	if not IsEmpty(list) then
		for i = #list, 1, -1 do
			local obj = list[i]
			DestroyObject(obj)
			list:removeAt(i)
		end
	end
end

function EffectAction:initPosition()
	if self.startPositionType == PositionType.Custom then
		local standType = math.abs(tonumber(self.startPosition.x) or StandType.HeroArea)
		local destAeraPosDic = GeneratePositions(standType, self.teamSide, #self.skillOnlyIds)
--		plog(destAeraPosDic, "destAeraPosDic2")
	
		local posIndex = tonumber(self.startPosition.y) or 0
		local count = CountTable(destAeraPosDic)
		if not posIndex or posIndex < 1 or posIndex > count then
			for p = 1, count do
				local data = destAeraPosDic[p]
				local pos = Vector3(data.x, data.y, 0)
				self.startPositionList:add(pos)
			end
		else
			local data = destAeraPosDic[posIndex]
			local pos = Vector3(data.x, data.y, 0)
			self.startPositionList:add(pos)
		end
	end

	if self.aimPositionType == PositionType.Custom then
		local standType = math.abs(tonumber(self.aimPosition.x) or StandType.HeroArea)
		local destAeraPosDic = GeneratePositions(standType, self.teamSide, #self.skillOnlyIds)
	
		local posIndex = tonumber(self.aimPosition.y) or 0
		if self.aimPosition.y == 0 then 
			posIndex = 1
		end
		if not posIndex or posIndex < 1 or posIndex > CountTable(destAeraPosDic) then
			posIndex = 1
		end
		local destPos = destAeraPosDic[posIndex]
		if destPos ~= nil then
			self.aimPosition.x = destPos.x
			self.aimPosition.y = destPos.y
		end
	end
end
