--[[
-- Desc: 管理类
-- Author: kenny
-- Date: 2016
--]]
ActionManager = class()
 
function ActionManager:ctor()
	self.actionMaps = Map.new()
	self.beganQueue = List:new()
	self.endedQueue = List:new()
	self.curMapIndex = 0					-- 当前actionMaps的index
	self.insertIndex = 0						-- 插入下标
	self.curCount = 0							-- 当前actionMaps的actions队列计数
	self.curRound = 0						-- 当前回合
	self.isOver = false
end

function ActionManager:init(data)
	self.skillsData = Config.getSkillsData()
	self.actionsData = Config.getActionsData()
	self:makePrepareAction()
	self:makeActions(data)
end

function ActionManager:getActionMaps()
	return self.actionMaps
end

function ActionManager:getActionByIndexId(actionId, camp, round, indexId)
	local key = FightControler.getActionMapKey(actionId, camp, round)
	local actions = self.actionMaps[key]
	if not IsEmpty(actions) and actions:size() > 0 then
		local keys = actions:keys()
		for k = 1, #keys do
			local action = actions[k]
			if not IsEmpty(action) and tonumber(action.indexId) == tonumber(indexId) then
				return action
			end
		end
	end
	return nil
end

function ActionManager:makePrepareAction()
	local delay = DelayAction.new()
	delay:setActionId("pre01")
	delay:setIndexId(1)
	delay:setBeginCondition(0, 0)
	delay:setEndCondition(0, 1)

	local ready = ReadyAction.new()
	ready:setActionId("pre01")
	ready:setIndexId(2)
	ready:setBeginCondition(1, -1)
	ready:setEndCondition(0, -1)

	ready:setEndParent(delay)
	delay:addChild(ready)

	local actionMap = Map:new()
	actionMap:add(1, delay)
	actionMap:add(2, delay)
	self.actionMaps:add("pre01", actionMap)
end

function ActionManager:initActionMaster(action, skillHeroOnlyIds)
	local masterType = action.actionMasterType
	local isOne = IsMember(masterType, ActionMasterType)
	if not isOne then
		local jsonActionList = CopyTable(self.actionsData[action.actionId])
		if not IsEmpty(jsonActionList) then
			if not IsEmptyTable(action.actionMaster) then
				local index = tonumber(action.actionMaster[1])
				local target = jsonActionList[index]
				if not IsEmpty(target) then
					action.actionMasterType = tonumber(target.actionMasterType)
					action.isGroup = (tonumber(target.isGroup) == 1) and true or false
					action.isClone = (tonumber(target.isClone) == 1) and true or false
					action.actionMaster = split(target.actionMaster, ",")
					action.teamSide = FightControler.getTeamSide(action.camp, action.actionMasterType)
					action.otherSide = 3 - action.teamSide
					action.destIndexId = index
				end
			end
		end
	end
	masterType = action.actionMasterType
	isOne = IsMember(masterType, ActionMasterType)
	if isOne then
		local fighters = {}
		local trigger = skillHeroOnlyIds -- FightControler.getSkillOnlyIds(action.skillId, action.camp)
		action.skillOnlyIds = trigger or {}
--		plog(skillHeroOnlyIds, "skillHeroOnlyIds")
		if masterType == ActionMasterType.SkillHeros then
			fighters = trigger
		elseif masterType == ActionMasterType.NoSkillHeros then
			fighters = FightControler.getNoSkillOnlyIds(trigger, action.camp)
		elseif masterType == ActionMasterType.TeammateHeros then
			fighters = FightControler.getAllOnlyIds(action.camp, ActionMasterType.TeammateHeros)
		elseif masterType == ActionMasterType.EnemyHeros then	-- 在友方回合里，对敌方执行行为
			fighters = FightControler.getAllOnlyIds(action.camp, ActionMasterType.EnemyHeros)
		end
		if IsEmpty(fighters) then 
			return
		end
		if action.isGroup then
			for _, id in ipairs(fighters) do
				action:addOnlyId(id)
			end
		else
			local lastOnlyIds = {}
			local target = nil
			local index = 1
			local onlyOne = nil			
			target = tostring(action.actionMaster[1])
			index = tonumber(action.actionMaster[2]) or 1
			for n = 1, #fighters do
				local id = fighters[n]
				local heroId = FightControler.getHeroIdByOnlyId(action.teamSide, id)
				if heroId == target then
					table.insert(lastOnlyIds, id)
				end
			end
			onlyOne = lastOnlyIds[index]
			if not onlyOne then
				onlyOne = RandomMember(fighters)
			end
			action:addOnlyId(onlyOne)
		end
	end
end

function ActionManager:makeSingleAction(configRowData, extension, custom)
	if not configRowData.actionType or not configRowData.actionExtra then
		error("行为表导出异常!")
		return
	end
	local action = nil
	local actionType = tonumber(configRowData.actionType)
	local actionExtra = Trim(configRowData.actionExtra)
	local actionId = Trim(extension.actionId)
	local indexId = tonumber(configRowData.indexId)

	local reportData = extension.reportData
	local fightHeroIds = extension.fightHeroIds
	local camp = extension.camp
	local skillHeroOnlyIds = extension.skillHeroOnlyIds

	if actionType == ActionType.SkillEffect then
		action = EffectAction.new()

	elseif actionType == ActionType.AnimationCtrl then
		action = AnimationAction.new()

	elseif actionType == ActionType.BoneAttach then
		action = EffectAction.new(true)

	elseif actionType == ActionType.ActorEffect then
		local extra = tonumber(actionExtra)
		if extra == ActorEffectExtra.Hide then
			action = AlphaAction.new(false)
		elseif extra == ActorEffectExtra.Show then
			action = AlphaAction.new(true)
		elseif extra == ActorEffectExtra.Scale then
			action = ScaleActorAction.new()
		end

	elseif actionType == ActionType.ActorCtrl then
		action = ControlActorAction.new()

	elseif actionType == ActionType.HertNum then
		local extra = tonumber(actionExtra)
		action = ShowNumberAction.new()

	elseif actionType == ActionType.Special then
		local extra = tonumber(actionExtra)
		if extra == 1 then
			action = DelayAction.new()
		elseif extra == 2 then
			action = BlackScreenAction.new()
		end
	end

	if IsEmpty(action) then
		error("行为配置表出错! action为空 =>  actionId: %s, indexId: %s", actionId, indexId)
		return
	end
	
	action.actionId = actionId
	action.indexId = indexId
	action.destIndexId = indexId
	action.actionType = actionType
	action.actionExtra = actionExtra
	action.custom = custom

	action.actionMasterType = tonumber(configRowData.actionMasterType)
	action.isGroup = (tonumber(configRowData.isGroup) == 1) and true or false
	action.isClone = (tonumber(configRowData.isClone) == 1) and true or false
	action.actionMaster = split(configRowData.actionMaster, ",")
	if action.actionType == ActionType.ActorCtrl and action.actionExtra == ActorCtrlExtra.Create then
		action.isClone = true
	end

	local scaleSize = tonumber(configRowData.scaleSize) or 1
	action.scaleSize = (scaleSize <= 0)  and 1 or scaleSize
	
	action.startPositionType = tonumber(configRowData.startPositionType)
	action.startPosition = TranPosition(configRowData.startPosition)
	action.aimPositionType = tonumber(configRowData.aimPositionType)
	action.aimPosition = TranPosition(configRowData.aimPosition)

	action.skillId = reportData.skillId
	action.round = tonumber(reportData.round) or 0
	action.teamId = tonumber(reportData.teamId) or 0
	action.damage = tonumber(reportData.damage) or 0
	action.restoration = tonumber(reportData.restoration) or 0
	action.dodge = reportData.dodge and true or false
--	plog("@teamId = %d, skillId = %s, damage = %d, restoration = %d, dodge = %s", action.teamId, action.skillId, action.damage, action.restoration, tostring(action.dodge))
	action.isOver = reportData.isOver
	action.fightHeroIds = fightHeroIds
	action.viewParent = FightControler.getViewParent()
	action.camp = camp
	action.teamSide = FightControler.getTeamSide(action.camp, action.actionMasterType)
	action.otherSide = 3 - action.teamSide

	self:initActionMaster(action, skillHeroOnlyIds)
	
	-- 开始时机
	local startTimeArr = split(configRowData.startTime, ",");		
	action.beginCond = {[1] = tonumber(startTimeArr[1]) or 0, [2] = tonumber(startTimeArr[2]) or 0}
	if action.beginCond[2] > 0 then
		action.beginCond[2] = action.beginCond[2] / 1000
	end
	
	-- 结束时机
	local overTimeArr = split(configRowData.overTime , ",");
	if #overTimeArr == 1 then
		local cond = tonumber(overTimeArr[1])
		if cond and cond < 0 then
			action.endCond = {[1] = -1, [2] = 0}
		else
			action.endCond = {[1] = 0, [2] = tonumber(overTimeArr[1]) or 0}
		end
	else
		action.endCond = {[1] = math.abs(tonumber(overTimeArr[1])) or 0, [2] = tonumber(overTimeArr[2]) or 0}
	end
	if action.endCond[2] > 0 then
		action.endCond[2] = action.endCond[2] / 1000
	end
	action.lastTime = action.endCond[2]

	return action
end

-- 格式化action
function ActionManager:formatAction(skillId, action, params)
	local ret = false
	if IsEmpty(params) then
		local skillConfigData = self.skillsData[tostring(skillId)]
		if IsEmpty(skillConfigData) then
			return ret
		end
		params = split(tostring(skillConfigData["#params"]), ";")
--		plog(params, "params2:" .. skillId .. ", " .. tostring(action.actionId))
	end

	for key, val in pairs(action) do
		local str = tostring(val) or ""
		local numL = string.find(str, "{") or 0
		if numL > 0 then
			local numR = string.find(str, "}") or 0
			local len = numR - numL - 1
			local num = 0
			if len > 0 then
				num = tonumber(string.sub(str, numL+1, numR-1)) or 0
			end
			if num > 0 then
				local param = params[num]
				if param ~= nil then
					action[key] = param
					ret = true
				end
			end
--			plog("找到了[%s][%s][%s]= %d, %d, %d, len = %d", skillId, tostring(key), str, numL, numR, num, len)
		end
	end
	return ret
end

-- 制作Actions Map
function ActionManager:makeActions(battleData)
	local reports = battleData.reports
 	if IsEmpty(reports) then
 		error("服务器的战报数据出错!!!")
		return
	end
	for m = 1, reports.size do
		local reportData = reports[m]
		local skillId = reportData.skillId
 		if IsEmpty(self.skillsData[skillId]) then
 			error("技能表不存在此技能ID : %s", tostring(skillId))
			break
 		end

		local skillHeroOnlyIds = reportData.skillHeroOnlyIds
		local skillConfigData = self.skillsData[skillId]
		if IsEmpty(skillConfigData) then
			break
		end
		local fightHeroIds = split(skillConfigData.hero_id, ",")		-- 上阵英雄id
--		plog(fightHeroIds, "FightHeroIds")
		local camp = Camp.Neutrality											-- 默认阵营中立
		if battleData.homeTeam.teamId == reportData.teamId then
			camp = Camp.Teammate
		else
			camp = Camp.Enemy
		end
--		plog(reportData, "##Team:"..camp..", ReportData="..reportData.skillId)
		
		local actionIds = split(skillConfigData["#actionId"], ",")
		local actionId = tostring(actionIds[1])
		if reportData.dodge and not IsEmpty(actionIds[2]) then
			actionId = tostring(actionIds[2])
--			plog("闪避： %d", camp)
		end
		local params = split(tostring(skillConfigData["#params"]), ";")
		
		local jsonActionList = CopyTable(self.actionsData[actionId])
		if IsEmpty(jsonActionList) or #jsonActionList <= 0 then
			error("行为配置表的数据不存在!  actionId: %s", actionId)
			break
		end

--		if not IsEmpty(params) then
--			plog(params, "params1:" .. skillId .. ", " .. actionId)
--			plog(jsonActionList, "JsonActionList: " .. actionId)
--		end

		local isFormat = false
		for _, action in ipairs(jsonActionList) do
			if self:formatAction(skillId, action, params) then
				isFormat = true
--				plog(action, "目标1 = ".. skillId .. ", ".. actionId .. ", ".. tostring(action.indexId))
			end
		end

		local actionMap = Map:new()
		local beginMap = Map:new()
		local endMap = Map:new()
		for n = 1 , #jsonActionList do
			local configRowData = jsonActionList[n]
			local indexId = tonumber(configRowData.indexId) or 0
--			plog(configRowData, "-------->ConfigRowData分解ID: " .. indexId)
--			if FightConfigDataChecker.checkExceptionData(configRowData) then
			do
				local extension = {}
				extension.actionId = actionId
				extension.reportData = reportData
				extension.fightHeroIds = fightHeroIds
				extension.skillHeroOnlyIds = skillHeroOnlyIds
				extension.camp = camp
				
				local action = self:makeSingleAction(configRowData, extension, false)
				if not IsEmpty(action) then
					local beginTime1 = action.beginCond[1]
					if beginTime1 ~= 0 then
						if not beginMap:containsKey(beginTime1) then
							beginMap:add(beginTime1, List:new())
						end
						beginMap[beginTime1]:add(indexId)
					end
					
					local endCond = action.endCond
					if not IsEmpty(endCond) then
						if not endMap:containsKey(endCond[1]) then
							endMap:add(endCond[1] , List:new())
						end
						endMap[endCond[1]]:add(indexId)
					end
					-- 添加到当前的同组行为
					actionMap:add(action.indexId, action)
				end
			end
		end -- for n = 1 , #jsonActionList do do end
		
		-- 1.关联开始依赖
		if beginMap:size() > 0 then
			local keys = beginMap:keys()
			for k = 1, #keys do
				local key = keys[k]
				local action = actionMap[key]
				if not IsEmpty(beginMap[key]) then
					for _, idx in ipairs(beginMap[key]) do
						local child = actionMap[idx]
						if not IsEmpty(child) then
							if not IsEmpty(action) and child ~= action then 
								action:addChild(child)
								child:setBeginParent(action)
							else
								self:clearActionMap(actionMap, idx)
							end
						end
					end
				end
			end
		end
		
		-- 2.关联结束依赖
		if endMap:size() > 0 then
			local keys = endMap:keys()
			for k = 1, #keys do
				local key = keys[k]
				local action = actionMap[key]
				if not IsEmpty(action) and endMap[key] ~= nil then
					for _, idx in ipairs(endMap[key]) do
						local child = actionMap[idx]
						if not IsEmpty(child) and child ~= action then
							child:setEndParent(action)
						end
					end
				end
			end
		end

		if actionMap:size() > 0 then
			local action = actionMap[actionMap:keys()[1]]
			local teamSide = FightControler.getTeamSide(action.camp, ActionMasterType.SkillHeros) -- 默认友方
			local key = FightControler.getActionMapKey(action.actionId, teamSide, action.round)
			self.actionMaps:add(key, actionMap)
		end
	end -- for m = 1, reports.size do end
--	plog("self.actionMaps.size = %d", self.actionMaps:size())
--	plog(self.actionMaps, "self.actionMaps")
end

function ActionManager:insertNewAction(actionType, actionExtra, actionParent)
	if actionParent.custom then
		return
	end
	local configData = {}
	local actionId = actionParent.actionId
	local jsonActionList = CopyTable(self.actionsData[actionId])
	if IsEmpty(jsonActionList) then
		return
	end
	for n = 1, #jsonActionList do
		local configRowData = jsonActionList[n]
		if tonumber(configRowData.indexId) == actionParent.indexId then
			configData = configRowData
			if self:formatAction(actionParent.skillId, configData) then
--				plog(configData, "目标2 = "..configData.skillId)
			end
		end
	end
	if IsEmpty(configData) then
		return
	end
	configData.actionType = actionType
	configData.actionExtra = actionExtra

	local reportData = {}
	reportData.skillId = actionParent.skillId
	reportData.round = actionParent.round
	reportData.teamId =actionParent.teamId
	reportData.damage = actionParent.damage
	reportData.restoration = actionParent.restoration
	reportData.dodge = actionParent.dodge
	
	local extension = {}
	extension.actionId = actionId
	extension.reportData = reportData
	extension.fightHeroIds = actionParent.fightHeroIds
	extension.camp = actionParent.camp

	local action = self:makeSingleAction(configData, extension, true)
	if not IsEmpty(action) then
		action.beginCond = {actionParent.indexId, 0}
		if actionType == ActionType.HertNum then
--			plog("[HertNum] skillId = %s, damage = %d, restoration = %d", action.skillId, action.damage, action.restoration)
			if actionExtra == FightNumExtra.Retrieve then
				FightControler.updateHP(action.teamSide, action.restoration, action.actionExtra)
			else
				FightControler.updateHP(action.teamSide, action.damage, action.actionExtra)
			end
		end
	
		self:insertActionMap(action)
	end
end

function ActionManager:insertActionMap(action, actionParent)
	if IsEmpty(action) then return end
	local teamSide = FightControler.getTeamSide(action.camp, ActionMasterType.SkillHeros) -- 默认友方
	local mapkey = FightControler.getActionMapKey(action.actionId, teamSide, action.round)
	local actionMap = self.actionMaps[mapkey]
	if not IsEmpty(actionMap) then
--		local indexId = 0
--		local keys = actionMap:keys()
--		for k = 1, #keys do
--			local key = keys[k]
--			local act = actionMap[key]
--			if act.indexId > indexId then
--				indexId = act.indexId
--			end
--		end
--		indexId = indexId + 1

		self.insertIndex = self.insertIndex - 1
		action.indexId = self.insertIndex
		actionMap:add(indexId, action)
		if actionParent ~= nil then
			actionParent:addChild(action)
			action:setBeginParent(actionParent)
		else
			self:beginAction(action)
		end
		self.curCount = self.curCount + 1
	end
end

function ActionManager:clearActionMap(actionMap, indexId)
	if IsEmpty(actionMap) then
		return
	end
	actionMap:remove(indexId)
	local keys = actionMap:keys()
	for k = #keys, 1, -1 do
		local key = keys[k]
		local action = actionMap[key]
		if action.beginCond[1] == indexId then
			actionMap:remove(k)
		end
	end
end

function ActionManager:stopAllActions()
	if self.beganQueue.size > 0 then
		for n = #self.beganQueue, 1, -1 do
			local action = self.beganQueue[n]
			self:endAction(action)
		end
	end
end

function ActionManager:checkActionMaps()
	if self.isOver then
		return
	end
	if self.curMapIndex > self.actionMaps:size() then
		self.isOver = true
		FightControler.gameOver()
		return
	end
	if self.curCount <= 0 then
		self.beganQueue:clear()
		self.endedQueue:clear()
		self.curMapIndex = self.curMapIndex + 1
		local keys = self.actionMaps:keys()

		local key = keys[self.curMapIndex]
		if not IsEmpty(key) then
			local actions = self.actionMaps[key]
			self.curCount = actions:size()
			if self.curCount > 0 then
				local order = actions:keys()
				for n = 1, #order do
					local action = actions[order[n]]
					if action.beginCond[1] == 0 and not action:isBegan() then
						self:beginAction(action)
					end
				end
			end
		end
	end
end

function ActionManager:beginAction(action)
	if IsEmpty(action) or action:isBegan() then
		return
	end
--	plog("[Began] Camp: %d, SkillId: %s, ActionId: %s, IndexId: %d, Time: %.2f, Type: %s, Extra: %s", action.camp, action.skillId, action.actionId, action.indexId, action:getBeginTime(), tostring(action.actionType), tostring(action.actionExtra))
	action:onBegin()
	self.beganQueue:add(action)
	self.curRound = action.round
	FightControler.updateRound(self.curRound)
	self:addNumAction(action)
end

function ActionManager:endAction(action)
	if IsEmpty(action) then -- 不能加action:isEnded()， 考虑到外部中断
		return
	end
	if not IsMember(action, self.beganQueue) then
		return
	end
	self.curCount = self.curCount - 1
	if action.endCond[1] >= 0 then -- 当为-1时，属于跨回合Action
		action:onEnd()
--		plog("[Ended] Camp: %d, SkillId: %s, ActionId: %s, IndexId: %d, Time: %.2f", action.camp, action.skillId, action.actionId, action.indexId, action:getDeltaTime())
	end
	self.beganQueue:remove(action)
	self.endedQueue:add(action)
end

function ActionManager:addNumAction(action)
	local parentMT = (not action:getBeginParent() or action:getBeginParent().actionMasterType) or 0
	if action.dodge and action.actionMasterType == ActionMasterType.EnemyHeros
	and parentMT ~= ActionMasterType.EnemyHeros then
		self:insertNewAction(ActionType.HertNum, FightNumExtra.Miss, action)
	elseif action.actionType == ActionType.AnimationCtrl and action.actionExtra == AnimationName.HIT then
		self:insertNewAction(ActionType.HertNum, FightNumExtra.Hert, action)
	elseif action.actionType == ActionType.SkillEffect  and action.actionExtra == "tongyong_huixue" then
		self:insertNewAction(ActionType.HertNum, FightNumExtra.Retrieve, action)
	end
end

function ActionManager:checkActionStart(father, child)
	-- 开始条件: 依赖分解行为
	if not IsEmpty(child.beginCond) and not child:isBegan() then
		if child.beginCond[2] == -1 then			-- 结束依赖
			if father ~= nil and father:isEnded() then
				self:beginAction(child)
			end
		elseif child.beginCond[2] == 0 then		-- 开始依赖
			if father ~= nil and father:isBegan() then
				self:beginAction(child)
			end
		elseif child.beginCond[2] > 0 then			-- 时间依赖
			if Time.timeSinceLevelLoad - father:getBeginTime() >= child.beginCond[2] then
				self:beginAction(child)
			end
		end
	end
end

function ActionManager:checkActionOver(action)
	if action:isEnded() then	-- 外部中断
		self:endAction(action)
	end
	if action.endCond[1] > 0 then						-- 1. 结束条件: 依赖分解行为
		local parent = action:getEndParent()
		if not parent and action.beginCond[1] ~= 0 then
			error(string.format("结束的条件依赖出错： Team[%d], ActionId: %s, IndexId: %d", action.camp, action.actionId, action.indexId))
			return
		end
		if action.endCond[2] == -1 then				-- 结束依赖
			if not parent or parent:isEnded() then
				self:endAction(action)
			end
		elseif action.endCond[2] == 0 then		-- 开始依赖
			if not parent or parent:isBegan() then
				self:endAction(action)
			end
		elseif action.endCond[2] > 0 then			-- 时间依赖
			if Time.timeSinceLevelLoad - parent:getBeginTime() >= action.endCond[2] then
				self:endAction(action)
			end
		end
	else																-- 2. 结束条件: 持续时间
		if Time.timeSinceLevelLoad - action:getBeginTime() >= action.endCond[2] and not action:isEnded() then
			self:endAction(action)
--			plog(self.beganQueue, "self.beganQueue")
--			plog(self.endedQueue, "self.endedQueue")
		end
	end
end

function ActionManager:checkActionChildren(action)
	if IsEmpty(action) then
		return
	end
	local children = action:getChildren()
	if IsEmpty(children) or children.size<=0 then
		return
	end
	for _, child in ipairs(children) do
		self:checkActionStart(action, child)
	end
end

function ActionManager:Update()
	self:checkActionMaps()
	if self.beganQueue.size > 0 then
		for n=#self.beganQueue, 1, -1 do
			local action = self.beganQueue[n]
			self:checkActionOver(action)
			self:checkActionChildren(action)
		end
	end
	if self.endedQueue.size > 0 then
		for n=#self.endedQueue, 1, -1 do
			local action = self.endedQueue[n]
			self:checkActionChildren(action)
		end
	end
end
