--[[
-- Desc: 角色
-- Author: kenny
-- Date: 2016
--]]
Actor = class()

local s_prefab_names = {"daojian", "daojian2", "daojian3", "gongjianshou", "gushou", "renma", "renma2", "renmagong", "yueqi"}
local s_sortingOrder = 0
local s_prefabs = {}
local s_heros = List.new()

function Actor:ctor(res)
	self.skills = {}
	self.teamSide = TeamSide.Left	-- 默认站位左边(Client)
	self.filename = ""
	self.savePos = Vector3.zero
	self.dead = false
	self.loopCount = 0
end

function Actor:create(teamSide, mapId, pos, onlyId)
	s_sortingOrder = s_sortingOrder + 1
	self.mapId = mapId
	self.onlyId = onlyId
	self.heroId = FightControler.getHeroIdByOnlyId(teamSide, onlyId)
	self.savePos = pos
	self.teamSide = teamSide
	self.prefabName = "wangzhaojun"
	self.parentObj = FightControler.getCampObject(teamSide)
--	plog("self.onlyId = %s", self.onlyId)
--	plog("self.heroId = %s", self.heroId)
	local herosProp = Config.getHerosData()
	local heroProp = herosProp[tostring(self.heroId)]
	if not IsEmpty(heroProp) then
		local prefabName = heroProp["#prefab_name"]
		if not IsEmpty(prefabName) then
			self.prefabName = prefabName
		end
	end

	local prefab = CreateActorPrefab(self.prefabName)
	if not prefab then 
		error("Not Found PrefabName = %s", self.prefabName)
		prefab = CreateActorPrefab("wangzhaojun")
	end
	self.hero = CreateObject(prefab)
	self.hero.name = self.prefabName
	self.hero.transform.parent = self.parentObj.transform
	self.hero.transform.localPosition = pos
	self.hero.transform.localRotation = Quaternion.Euler(Vector3.zero)
	self.hero.transform.localScale = Vector3.one
--	self.hero.layer = LayerMask.NameToLayer(CameraLayer.Default)

	self.body = GetGameObject(self.hero.transform, "body")
	self.skeletonAnimation = GetComponent(self.body.transform, "SkeletonAnimation")
	if not self.skeletonAnimation then
		plog("Error: Actor's SkeletonAnimation is not exist, HeroName = ", self.hero.name)
		return
	end
	
	self.spineRenderer = self.skeletonAnimation:GetComponent(MeshRenderer.GetClassType())
	self.spineRenderer.sortingLayerName = LayerName.Actor
	self.spineRenderer.sortingOrder = s_sortingOrder
	
	self.shadow = CreateObject(Util.LoadPrefab("Effects/shadow", AssetBundleInfo.assetBundles))
	self.shadow.transform.parent = self.body.transform
	self.shadow.transform.localPosition = Vector3(self.body.transform.localPosition.x, self.body.transform.localPosition.y, 0)
	self.shadow.transform.localRotation = Quaternion.Euler(Vector3.zero)
	self.shadow.transform.localScale = Vector3.one
--	self.shadowRenderer = GetPathComponent(self.shadow.transform, "Sprite", "UI2DSprite")
--	self.shadowRenderer.sortingLayerName = LayerName.Shadow
--	self.shadowRenderer.sortingOrder = 0
	ViewUtil.sortLayerOrder(self.shadow, LayerName.Shadow)

	self:playAnimation(AnimationName.IDLE)

--	local shader = Util.findShader("Particles/Additive")
--	plog(shader, "shader")
--	local renderer = self.skeletonAnimation:GetComponent(Renderer.GetClassType())
--	renderer.material.shader = shader
end

--function Actor:getActorZOrder(teamSide)
--	local ret = s_sortingOrder
--	if teamSide == TeamSide.Right then
--		ret = -ret
--	end
--	return ret
--end

function Actor:getActorObject()
	return self.hero
end

function Actor:getActorRender()
	return self.skeletonAnimation
end

function Actor:getPositionSource()
	return self.savePos
end

function Actor:getPosition()
	return self.hero.transform.localPosition
end

function Actor:showShadow(opacity)
	enabled = (opacity and opacity > 0) and true or false
	if not enabled then
		NGUITools.SetActive(self.shadow, false)
	else
		NGUITools.SetActive(self.shadow, true)
	end
end

-- 外部接口
function Actor:showAlpha(opacity)
	local render = self.skeletonAnimation
	if render ~= nil then
		render.skeleton.a = opacity
		self:showShadow(opacity)
	end
end

function Actor:showColorNormal()
	Util.SetSpineColor(self.skeletonAnimation, CreateColorF(1, 1, 1, self.skeletonAnimation.skeleton.a))
end

function Actor:showColorRed()
	Util.SetSpineColor(self.skeletonAnimation, CreateColorF(1, 53/255, 53/255, self.skeletonAnimation.skeleton.a))
end

function Actor:playAnimation(animationName)
	if self.dead then
		return
	end
	Util.StopSpineAnimation(self.skeletonAnimation)
	if animationName == AnimationName.RUN or animationName == AnimationName.IDLE then
		self.skeletonAnimation.loop = true
	else
		self.skeletonAnimation.loop = false
	end
	if name ~= AnimationName.HIT then
		self:showColorNormal()
	end

	local function OnComplete(skeletonAnimation, state, trackIndex, loopCount)
		self.loopCount = loopCount
		--[[local name = self.skeletonAnimation.AnimationName
		if name == AnimationName.HIT then
			if skeletonAnimation then
				Util.StopSpineAnimation(skeletonAnimation)
				skeletonAnimation.loop = true
				skeletonAnimation.AnimationName = "idle"
			end
			self:showColorNormal()
		end--]]
	end
	Util.AddSpineCompleteForLua(self.skeletonAnimation, OnComplete)
	self.skeletonAnimation.AnimationName = animationName
end

function Actor:playAnimationWithAction(animationName, action)
	if not self.skeletonAnimation or self.dead then
		return
	end
	Util.StopSpineAnimation(self.skeletonAnimation)
	if animationName == AnimationName.RUN or animationName == AnimationName.IDLE then
		self.skeletonAnimation.loop = true
	else
		self.skeletonAnimation.loop = false
	end
	if name ~= AnimationName.HIT then
		self:showColorNormal()
	end

	local function OnComplete(skeletonAnimation, state, trackIndex, loopCount)
		self.loopCount = loopCount
		if not self.skeletonAnimation.loop then
			if action.isActorEnd then
				FightControler.getActionManager():endAction(action)
			end
		end
	end
	Util.AddSpineCompleteForLua(self.skeletonAnimation, OnComplete)
	self.skeletonAnimation.AnimationName = animationName
end

function Actor:OnStart()
	plog("OnStart")
end

function Actor:OnEnd()
	plog("OnEnd")
end

function Actor:OnComplete()
	plog("OnComplete")
end

function Actor:pauseAnimation(isPause)
	if self.dead then
		return
	end
	if isPause then
		self.skeletonAnimation.timeScale = 0
	else
		self.skeletonAnimation.timeScale = 1
	end
end

function Actor:hert()
	if self.dead then
		return
	end
end

function Actor:die()
	self.dead = true
end
 
function Actor:destroy()
	if self.hero ~= nil then
		GameObject.Destroy(self.hero)
	end
	self:die()
end

function PreloadActors(skinName)
	local _, prefabName, _ = GetActorPrefab(skinName)
	if prefabName ~= nil then
		Util.PreloadAssetAsync("Actors/"..prefabName, "Objects", function(obj)
			if obj ~= nil then
				obj.name = prefabName
				s_heros:add(obj)
			end
		end)
	end
end

function PreloadActorObjects()
	for _, name in ipairs(s_prefab_names) do
		Util.PreloadAssetAsync("Actors/"..name, "Objects", function(obj)
			if obj ~= nil then
				obj.name = name
				s_heros:add(obj)
			end
		end)
	end
end

function InitActorPrefabs()
	for _, name in ipairs(s_prefab_names) do
		Util.LoadAssetAsync("Actors/"..name, function(prefab)
			if prefab ~= nil then
				if s_prefabs[name] ~= nil then
					DestroyObject(s_prefabs[name])
					s_prefabs[name] = nil
				end
				s_prefabs[name] = prefab
			end
		end)
	end
end

function InitActorObjects()
	local heros = TeamData.GetHerosByIndex(TeamData.GetCurPageIndex())
	local heroProps = Config.getHerosData()
	for i = 1, #heros do
		local hero = heros[i]
		local heroProp = heroProps[tostring(hero.heroId)]
		if not IsEmpty(heroProp) then
			local skinName = heroProp["#prefab_name"]
			PreloadActors(skinName)
		end
	end
end

function GetActorObject(skinName)
	local _, prefabName, _ = GetActorPrefab(skinName)
	for i = 1, s_heros.size do
		local obj = s_heros[i]
		if obj ~= nil and obj.name == prefabName then
			local go = s_heros:pop(i)
			go:SetActive(true)
			local anim = GetPathComponent(go.transform, "body", "SkeletonAnimation")
			anim.initialSkinName = skinName
			return go
		end
	end
end

function GetActorPrefab(skinName)
	for _, name in ipairs(s_prefab_names) do
		local prefab = s_prefabs[name] or Util.LoadPrefab("Actors/" .. name, AssetBundleInfo.assetBundles)
		if prefab ~= nil then
			local anim = GetPathComponent(prefab.transform, "body", "SkeletonAnimation")
			local skins = anim.Skeleton.data.skins
			for i = 0, skins.Count - 1 do
				local skin = skins.Items[i].name
				if skinName == skin then
					return prefab, name, anim
				end
			end
		end
	end
	return nil
end

function CreateActorPrefab(skinName)
	local prefab, name, anim = GetActorPrefab(skinName)
	if prefab ~= nil then
		anim.initialSkinName = skinName
		return prefab, name
	end
	return nil
end

function RefreshActor(go, skinName)
	local anim = GetPathComponent(go.transform, "body", "SkeletonAnimation")
	local skins = anim.Skeleton.data.skins
	for i = 0, skins.Count - 1 do
		local skin = skins.Items[i].name
		if skinName == skin then
			anim.initialSkinName = skinName
			anim:Initialize(true)
		end
	end
end

function GeneratePositions(standType, teamSide, skillActorNum)
	local ret = {}
	local standNum = 0
	local standType = tonumber(standType) or 1
	local homeNum, awayNum = FightControler.GetTeamNums()
	local skillActorNum = tonumber(skillActorNum) or 1
	if skillActorNum <= 0 then skillActorNum = 1 end

	local idleAeraPosList ={}
	if standType == StandType.HeroArea then
		idleAeraPosList = Config.getPositionsData()["1"]		-- 克隆区
		if teamSide == TeamSide.Left then
			standNum = homeNum
		else
			standNum = awayNum
		end
	elseif standType == StandType.CloneArea then
		idleAeraPosList = Config.getPositionsData()["2"]		-- 英雄区
		standNum = skillActorNum
	else
		error("StandType[%s] is wrong!", standType)
	end

	if not IsEmptyTable(idleAeraPosList) then
		for i = 1, #idleAeraPosList do
			if tonumber(idleAeraPosList[i].number) == standNum then
				local posX = tonumber(idleAeraPosList[i].posX) or 0
				local posY = tonumber(idleAeraPosList[i].posY) or 0
				local vPos = Vector3.New(posX, posY, 0)
				local idx = tonumber(idleAeraPosList[i].posID) or 0
				ret[idx] = vPos
			end
		end
	else
		error("站位表数据异常!")
	end
	if IsEmptyTable(ret) then
		error("布置人数不存在: standType[%s], standNum[%s]!", standType, standNum)
	end
	return ret
end
