--[[
-- Desc: 战斗单例
-- Author: kenny
-- Date: 2016
--]]
require "Game/Fight/View/FightView"
require "Game/Fight/Logic/Actor"
require "Game/Fight/Logic/FightConfigDataChecker"
require "Game/Fight/Logic/ActionDefine"

FightControler = {}

local this = FightControler
this.isFighting = false

UpdateRandomSeed()		-- 更新随机种子

function FightControler.New()
	return this
end

function FightControler.OnCreate(obj, name)
	ViewUtil.closeAllViews()

	this.root = ViewUtil.getRootView()
	this.view = FightView.new()
	this.view.name = name
	this.gameObject = obj
	this.transform = obj.transform
	HangObject(obj, this.root)

	this.luaBehaviour = this.transform:GetComponent('LuaBehaviour')
	this.view:Awake(this.gameObject)
    this.initListener()

	this.battleData = nil
	this.allActorsMap = Map.new()
	this.curRound = 0

    this.hideGuideModule();
end

--[[

]]
function FightControler.hideGuideModule ()
    local guide = nil;

    guide = FindObject("UIRoot/GuideMain(Clone)/Container");
    if (nil ~= guide) then
        guide:SetActive(false);
    end
end

function FightControler.Awake()
end

function FightControler.Start()
end

function FightControler.SetTeamNums(homeNum, awayNum)
	if homeNum < 1 then homeNum = 1 end
	if homeNum > 25 then homeNum = 25 end
	if awayNum < 1 then awayNum = 1 end
	if awayNum > 25 then awayNum = 25 end
	this.actorNums = {tonumber(homeNum) or 1, tonumber(awayNum) or 1}
end
function FightControler.GetTeamNums()
	local home = this.actorNums[1] or 1
	local away = this.actorNums[2] or 1
	return home, away
end

if not Global.isLocal then
function FightControler.readyBattle()
	local battleData = {}
	local battleTeamData_self = {}
	local battleTeamData_enemy = {}
	local battleEndReportData = {}
	local battleDungeonsData = {}
	battleData.reports = List:new()
	battleData.homeTeam = battleTeamData_self
	battleData.awayTeam = battleTeamData_enemy

	for i = 1, #this.reports do
		local reportData = {}
		local report = this.reports[i] or {}
		reportData.round = tonumber(report.round)
		reportData.teamId = tonumber(report.teamId)
		reportData.damage = tonumber(report.damage)
		reportData.restoration = tonumber(report.restoration)
		reportData.skillId = tostring(report.skillId)
		reportData.dodge = report.dodge and true or false
		reportData.isOver = report.isOver and true or false
		local heroIds = split(tostring(report.heroReqId), ",")
		TrimTable(heroIds)
		reportData.skillHeroOnlyIds = heroIds
		battleData.reports:add(reportData)
--		plog(heroIds, "skillHeroOnlyIds")
	end

	battleData.endReport = battleEndReportData

	battleData.dungeons = battleDungeonsData

	battleTeamData_self.teamId = this.homeTeam.teamId
	battleTeamData_self.teamName = this.homeTeam.teamName
	battleTeamData_self.teamAbility = this.homeTeam.teamAbility
	battleTeamData_self.onlyIds = {}
	battleTeamData_self.heroIds = {}
	local heros = this.homeTeam.heros
	if not IsEmpty(heros) then
		for i = 1, #heros do
			battleTeamData_self.onlyIds[i] = heros[i].id
			battleTeamData_self.heroIds[i] = heros[i].heroId
		end
	end
--	plog(battleTeamData_self.onlyIds, "homeTeam.onlyIds")
--	plog(battleTeamData_self.heroIds, "homeTeam.heroIds")

	battleTeamData_enemy.teamId = this.awayTeam.teamId
	battleTeamData_enemy.teamName = this.awayTeam.teamName
	battleTeamData_enemy.teamAbility = this.awayTeam.teamAbility
	battleTeamData_enemy.onlyIds = {}
	battleTeamData_enemy.heroIds = {}
	local heros = this.awayTeam.heros
	if not IsEmpty(heros) then
		for i = 1, #heros do
			battleTeamData_enemy.onlyIds[i] = heros[i].id
			battleTeamData_enemy.heroIds[i] = heros[i].heroId
		end
	end
--	plog(battleTeamData_enemy.onlyIds, "awayTeam.onlyIds")
--	plog(battleTeamData_enemy.heroIds, "awayTeam.heroIds")

	this.battleData = battleData
	this.startBattle(this.battleData)
end

else

function FightControler.readyBattle() -- 模拟服务器数据
	local battleData = {}
	local battleTeamData_self = {}
	local battleTeamData_enemy = {}
	local battleEndReportData = {}
	local battleDungeonsData = {}
	battleData.reports = List:new()
	battleData.homeTeam = battleTeamData_self
	battleData.awayTeam = battleTeamData_enemy

	for i = 1, 10 do -- test
		local battleReportData = {}
		battleReportData.round = GetIntPart(i/2 + 0.5)
		if i%2 ~= 0 then
			battleReportData.teamId = 1001
		else
			battleReportData.teamId = 1002
		end
		battleReportData.damage = 25
		battleReportData.restoration = 12
		battleReportData.skillId = "150101"
		battleReportData.dodge = false
		battleReportData.isOver = false

		local onlyIds = {"1001", "1002", "1003"}
		if camp == Camp.Enemy then
			onlyIds = {"2001", "2002", "2003"}
		end
		battleReportData.skillHeroOnlyIds = onlyIds

		battleData.reports:add(battleReportData)
	end

	battleData.endReport = battleEndReportData
	
	battleData.dungeons = battleDungeonsData

	battleTeamData_self.teamId = 1001;
	battleTeamData_self.teamName = "己方";
	battleTeamData_self.teamAbility = {xg=0, fy=0, sb=0, wz=0, power=1000}
	battleTeamData_self.onlyIds = 
	{"1001","1002","1003","1004","1005","1006","1007","1008","1009","1010","1011","1012","1013","1014","1015","1016","1017","1018","1019","1020","1021","1022","1023","1024","1025"}
	battleTeamData_self.heroIds = 
	{"11001","11001","11001","11001","11001","11001","11001","11001","11001","11001","11001","11001","11001","11001","11001","11001","11001","11001","11001","11001","11001","11001","11001","11001","11001"};

	battleTeamData_enemy.teamId = 1002;
	battleTeamData_enemy.teamName = "敌方";
	battleTeamData_enemy.teamAbility = {xg=0, fy=0, sb=0, wz=0, power=1000}
	battleTeamData_enemy.onlyIds = 
	{"2001","2002","2003","2004","2005","2006","2007","2008","2009","2010","2011","2012","2013","2014","2015","2016","2017","2018","2019","2020","2021","2022","2023","2024","2025"}
	battleTeamData_enemy.heroIds = {"11001","11001","11001","11001","11001","11001","11001","11001","11001","11001","11001","11001","11001","11001","11001","11001","11001","11001","11001","11001","11001","11001","11001","11001","11001"};

	battleEndReportData.isWin = true;
	battleEndReportData.awards = "??????";

	battleDungeonsData.dungeonsId = 00000;

	this.battleData = battleData
	this.startBattle(this.battleData)
end
end

--开始战斗
function FightControler.startBattle(battleData)
	this.isFighting = true

	--初始化战斗UI
	ViewUtil.initData(this, battleData)

	-- 创建己方和敌方英雄们
	this.createDefaultActors(battleData)

	-- 组织战斗回合Action
	this.actManager = ActionManager:new(battleData)
	this.actManager:init(battleData)

	-- 开始播放战斗回合指令
	UpdateHelper.addUpdate(this.Update)
end

-- 摆放己方和敌方英雄们
function FightControler.createDefaultActors(battleData)
--	plog("Camera.Length = " ..Camera.allCameras.Length)
--	plog(Camera.main:ScreenToWorldPoint(Vector3(0, 0, -150)), "Camera.main")
--	plog(Camera.main:ScreenToViewportPoint(Vector3(0, 0, -150)), "Camera.main")
	FightControler.SetTeamNums(#battleData.homeTeam.onlyIds, #battleData.awayTeam.onlyIds)

	this.actorParentObj = GameObject.New()
	this.heroParentObj = GameObject.New()
	this.enemyParentObj = GameObject.New()

	this.actorParentObj.name = "ActorObjects"
	this.heroParentObj.name = "Teammate"
	this.enemyParentObj.name = "Enemy"

	this.actorParentObj.transform.parent = this.transform
 	this.heroParentObj.transform.parent = this.actorParentObj.transform
 	this.enemyParentObj.transform.parent = this.actorParentObj.transform
	this.actorParentObj.transform.localPosition = Vector3(0, 0, -100)
 	this.heroParentObj.transform.localPosition = Vector3(0, 0, 0)
 	this.enemyParentObj.transform.localPosition = Vector3(0, 0, 0)

 	this.actorParentObj.transform.localScale = Vector3.one
	this.enemyParentObj.transform.localScale = Vector3.one
	this.heroParentObj.transform.localScale = Vector3.one

	this.enemyParentObj.transform.localRotation = Quaternion.Euler(Vector3(0, 180, 0))
	local onlyIDs = battleData.homeTeam.onlyIds
	local lenHeros = #onlyIDs
	local idleAeraPosDic = Map.new()
	local idleAeraPosList = GeneratePositions(StandType.HeroArea, TeamSide.Left)
--	plog(idleAeraPosList, "英雄站位坐标")
	for i = 1, #idleAeraPosList do
		idleAeraPosDic:put(i, idleAeraPosList[i])
	end
	for i = 1, lenHeros do
		this.createActor(0, TeamSide.Left, 0, onlyIDs[i], idleAeraPosDic[i])
	end
 
	onlyIDs = battleData.awayTeam.onlyIds
	lenHeros = #onlyIDs
	idleAeraPosDic:clear()
	idleAeraPosList = GeneratePositions(StandType.HeroArea, TeamSide.Right)
--	plog(idleAeraPosList, "敌方站位坐标")
	for i = 1, #idleAeraPosList do
		idleAeraPosDic:put(i, idleAeraPosList[i])
	end
	for i = 1, lenHeros do
		this.createActor(0, TeamSide.Right, 0, onlyIDs[i], idleAeraPosDic[i])
	end
end

function FightControler.getFightView()
    return this.view
end

function FightControler.getViewParent()
    return  this.transform.parent
end

function FightControler.getActionManager()
    return this.actManager
end

function FightControler.getActorMap()
	return this.allActorsMap
end

function FightControler.getActionMapKey(actionId, camp, round)
	actionId = tostring(actionId)
	if IsEmpty(actionId) then
		actionId = 0
	end
	round = tonumber(round)
	if not round or round < 0 then
		round = 0
	end
	local mapKey = actionId..'-'..camp..'-'..round
	return mapKey
end

function FightControler.getActorMapId(skillId, teamSide, indexId, onlyId)
	skillId = tostring(skillId)
	if IsEmpty(skillId) then
		skillId = 0
	end
	indexId = tonumber(indexId)
	if not indexId or indexId < 0 then
		indexId = 0
	end
	local mapId = skillId..'-'..teamSide..'-'..indexId..'-'..onlyId
	return mapId
end

-- 左边友方，右边敌方
function FightControler.getTeamSide(actionCamp, actionMasterType)
	if actionCamp == Camp.Teammate then
		if actionMasterType == ActionMasterType.EnemyHeros then
			return TeamSide.Right
		else
			return TeamSide.Left
		end
	elseif actionCamp == Camp.Enemy then
		if actionMasterType == ActionMasterType.EnemyHeros then
			return TeamSide.Left
		else
			return TeamSide.Right
		end
	end
end

function FightControler.getCampObject(teamSide)
	if teamSide == Camp.Teammate then
		return this.heroParentObj
	else
		return this.enemyParentObj
	end
end

function FightControler.getActorByOnlyId(skillId, teamSide, indexId, onlyId)
	local actorMap = FightControler.getActorMap()
	local mapId = this.getActorMapId(skillId, teamSide, indexId, onlyId)
	return actorMap[mapId]
end

function FightControler.getHeroIdByOnlyId(teamSide, onlyId)
	local ret = onlyId
	local battleTeamData = nil
	if teamSide == TeamSide.Left then
		battleTeamData = this.battleData.homeTeam
	elseif teamSide == TeamSide.Right then
		battleTeamData = this.battleData.awayTeam
	end
	if battleTeamData ~= nil then
		local onlyIDs = battleTeamData.onlyIds
		local heroIDs = battleTeamData.heroIds
--		plog(onlyIDs, "onlyIDs")
--		plog(heroIDs, "heroIDs")
--		plog("onlyId = %s", onlyId)
		onlyId = tonumber(onlyId)
		local bool, key = IsMember(onlyId, onlyIDs)
		if bool then
			ret = heroIDs[key] or ret
		end
	end
	return ret
end

function FightControler.getOnlyIdByHeroId(heroId, teamSide) -- 暂时不需要用，由后台告知唯一ID
	local ret = heroId
	local battleTeamData = nil
	if teamSide == TeamSide.Left then
		battleTeamData = this.battleData.homeTeam
	elseif teamSide == TeamSide.Right then
		battleTeamData = this.battleData.awayTeam
	end
	if battleTeamData ~= nil then
		local onlyIDs = battleTeamData.onlyIds
		local heroIDs = battleTeamData.heroIds
		local bool, key = IsMember(heroId, heroIDs)
		if bool then
			ret = onlyIDs[key] or ret
		end
	end
	return ret
end

function FightControler.getSkillOnlyIds(skillId, camp) -- test
--	local onlyIds = {}
--	 local skillsData = Config.getSkillsData()
--	 if not IsEmpty(skillsData) and not IsEmpty(skillId) then
--		local data = skillsData[skillId]
--		if not IsEmpty(data) then
--			local heroIds = split(data[1].hero_id, ",")
--			for _, id in ipairs(heroIds) do
--				if camp == Camp.Teammate then
--				elseif camp == Camp.Enemy then
--					onlyIds = {"2001", "2002", "2003"}
--				end
--			end
--		end
--	 end

	local onlyIds = {"1001", "1002", "1003"}
	if camp == Camp.Enemy then
		onlyIds = {"2001", "2002", "2003"}
	end
	return onlyIds
end

function FightControler.getNoSkillOnlyIds(skillOnlyIds, camp)
	local ret = {}
	local onlyIdList = nil
	if camp == Camp.Teammate then
		onlyIdList = this.battleData.homeTeam.onlyIds
	elseif camp == Camp.Enemy then
		onlyIdList = this.battleData.awayTeam.onlyIds
	end
	if not IsEmpty(onlyIdList) then
		for i = 1, #onlyIdList do
			local onlyId = onlyIdList[i]
			local bool = IsMember(onlyId, skillOnlyIds)
			if not bool then
				table.insert(ret, onlyId)
			end
		end
	end
	return ret
end

-- 根据当前阵营和主体阵营获取英雄onlyIds
function FightControler.getAllOnlyIds(actionCamp, masterCamp)
	local ret = {}
	local onlyIdList = nil

	if actionCamp == Camp.Teammate then
		if masterCamp == ActionMasterType.TeammateHeros then
			return this.battleData.homeTeam.onlyIds
		elseif masterCamp == ActionMasterType.EnemyHeros then
			return this.battleData.awayTeam.onlyIds
		end
	else
		if masterCamp == ActionMasterType.TeammateHeros then
			return this.battleData.awayTeam.onlyIds
		elseif masterCamp == ActionMasterType.EnemyHeros then
			return this.battleData.homeTeam.onlyIds
		end
	end

	return ret
end

-- For action
function FightControler.initActionActorList(action)
	if not action:isActionMasterType() then
		return
	end
	local skillId = action.skillId
	local teamSide  = action.teamSide
	local indexId  = action.indexId
	
	if not action.isClone then
		skillId = 0
		indexId = 0
	end
	
	local actorMap = FightControler.getActorMap()
	if actorMap ~= nil then
		for _, onlyId in ipairs(action.onlyIdList) do
			if onlyId ~= nil then
				local mapId = this.getActorMapId(skillId, teamSide, indexId, onlyId)
--				plog("#initActionActorList mapId = %s", mapId)
				local actor = actorMap[mapId]
				if actor ~= nil then
					action:addActor(actor)
				end
			end
		end
	end
end

function FightControler.createActor(skillId, teamSide, indexId, onlyId, pos) -- indexId == 0：表示非配置创建的本体
	local actorMap = FightControler.getActorMap()
	local mapId = this.getActorMapId(skillId, teamSide, indexId, onlyId)
--	plog("#createActor mapId = %s", mapId)
	if actorMap[mapId] ~= nil then
		FightControler.removeActor(skillId, teamSide, indexId, onlyId)
	end
	if pos ~= nil then
		pos.z = 0
		local actor = Actor.new()
		actor:create(teamSide, mapId, pos, onlyId)
		actorMap:put(mapId, actor)
		return actor
	end
	return nil
end

function FightControler.removeActor(skillId, teamSide, indexId, onlyId)
	local actorMap = FightControler.getActorMap()
	local mapId = this.getActorMapId(skillId, teamSide, indexId, onlyId)
--	plog("#removeActor mapId = %s", mapId)
	local actor = actorMap[mapId]
	if actor and actor.destroy then
		actor:destroy()
		actorMap:remove(mapId)
	end
	actorMap[mapId] = nil
end

function FightControler.pauseFighting()
--	Time.timeScale = 0
end

function FightControler.resumeFighting()
--	Time.timeScale = 1
end

function FightControler.isFighting()
	return this.isFighting
end

function FightControler.stopAllActions()
	if this.actManager then
		this.actManager:stopAllActions()
	end
end

--战斗结束
function FightControler.gameOver()
	UpdateHelper.removeUpdate(this.Update)
	this.stopAllActions()
	
	this.isFighting = false
	this.rewards = nil
	local result = false

	if not IsEmpty(this.endReport) then
		result = this.endReport.isWin
		this.rewards = this.endReport.awards
	end
	if result then
		ViewUtil.openView(ModuleName.WIN, true, nil, nil)
	else
		ViewUtil.openView(ModuleName.FAIL, true, nil, nil)
	end
	plog(">>>>>>>>>>>>>>>>>>>>>>>>>>>> GameOver! [%s]", tostring(result))
end

function FightControler.updateRound(round)
	local round = tonumber(round) or this.curRound
	if this.curRound ~= round then
		this.curRound = round
		this.view:updateRound(this.curRound)
	end
end

function FightControler.updateHP(side, num, extra)
	if not side or not num then return end
	this.view:updateHP(side, num, extra)
end

function FightControler.Update()
	if this.actManager then
		this.actManager:Update()
	end
end

function FightControler.OnDestroy()
	UpdateHelper.removeUpdate(this.Update)
	this.root = nil
	this.view = nil
	this.gameObject = nil
	this.transform = nil
	this.luaBehaviour = nil

	this.battleData = nil
	this.allActorsMap = nil
	this.curRound = 0
	this.actManager = nil

	this.homeTeam = nil
	this.awayTeam = nil
	this.reports = nil
	this.endReport = nil
	this.battleInfo = nil
end

function FightControler.initListener()
end

function FightControler.listNotificationInterests()
    local list = {""}
    return unpack(list)
end

function FightControler.handleNotification(notification)
	print("notification = " .. notification:getName())
	print(notification:getBody())
end

function FightControler.accept(data)
	-- 友方玩家信息
	this.homeTeam = data.homeTeam
	-- 敌方玩家信息
	this.awayTeam = data.awayTeam
	-- 战斗战报信息
	this.reports = data.reports
	-- 战斗结束战报
	this.endReport = data.endReport
	-- 战斗副本信息
	this.battleInfo = data.battleInfo
	
--	plog(data, "dataF")
--	plog(this.homeTeam, "this.homeTeam")
--	plog(this.awayTeam, "this.awayTeam")

	this.readyBattle()
end
