
FriendNoticeView = class(BaseView)

function FriendNoticeView:ctor()
end  

function FriendNoticeView:Awake(obj)
	self.gameObject = obj
	self.transform = obj.transform
	self.shadeOpacity = true
	self.herosConf = Config.getHerosData()
	self:initPanel()
end

function FriendNoticeView:Start()
end

function FriendNoticeView:initPanel()
 	self.top = GetPathComponent(self.transform, "Mode/Top", "UIWidget")
 	self.bottom = GetPathComponent(self.transform, "Mode/Bottom", "UIWidget")
	self.top:SetAnchor(self.root.transform)
	self.bottom:SetAnchor(self.root.transform)
		
	local topPath = "Mode/Top/Panel"

	local centerPath = "Mode/Center/Panel"
	self.btnBack = GetGameObject(self.transform, centerPath.."/BG/BtnBack")
	self.checkBox = GetGameObject(self.transform, centerPath.."/BG/CheckBox")
	self.selectFB = GetGameObject(self.transform, centerPath.."/BG/CheckBox/SelectFB")
	self.selectFB:SetActive(not PlayerData.GetRefuseFriendNotice())
	
	self.scvBG = GetGameObject(self.transform, centerPath.."/ScrollViewBG")
	self.scvObj = GetGameObject(self.scvBG.transform, "ScrollView")
	self.gridObj = GetGameObject(self.scvObj.transform, "Panel/Grid")
	self.grid = GetPathComponent(self.scvObj.transform, "Panel/Grid", "UIGrid")
	
	self.btnAgreeAll = GetGameObject(self.transform, centerPath.."/BtnAgreeAll")
	self.btnDenyAll = GetGameObject(self.transform, centerPath.."/BtnDenyAll")

	ViewUtil.adaptUISize(self.scvBG, true, 60, 60, 0, 0)
	ViewUtil.adaptUISize(self.scvObj, false, 80, 150, 20, 20)

	local bottomPath = "Mode/Bottom/Panel"

    local function doClick(obj)
		if obj == self.btnBack then
			ViewUtil.openView(ModuleName.Friend)
			self:closeView(false)
		elseif obj == self.btnAgreeAll then
			local list = self.grid:GetChildList()
			for i = 0, list.Count - 1 do
				local bar = list[i].gameObject
				local lb = GetComponent(bar.transform, "LuaBehaviour")
				if lb ~= nil then
					local src = lb.sourceLuaObj
					local info = src.info
--					Network.SendMoData(ServerID.LeagueAllowJoin, {info.teamId})
					Network.SendMoData(ServerID.FriendComfirm, {info.friendTeamId, 1})
				end
				DestroyObject(bar)
			end
			ViewUtil.openView(ModuleName.Friend)
			self:closeView(false)
		elseif obj == self.btnDenyAll then
			local list = self.grid:GetChildList()
			for i = 0, list.Count - 1 do
				local bar = list[i].gameObject
				local lb = GetComponent(bar.transform, "LuaBehaviour")
				local src = lb.sourceLuaObj
				local info = src.info
--				Network.SendMoData(ServerID.LeagueDenyJoin, {info.teamId})
				Network.SendMoData(ServerID.FriendComfirm, {info.friendTeamId, 2})
				DestroyObject(bar)
			end
			ViewUtil.openView(ModuleName.Friend)
			self:closeView(false)
		elseif obj == self.checkBox then
			local refuse = not PlayerData.GetRefuseFriendNotice()
			PlayerData.SetRefuseFriendNotice(refuse)
			self.selectFB:SetActive(not refuse)
			if refuse then
				Network.SendMoData(ServerID.FriendReject, {0}) -- 拒绝好友申请
			else
				Network.SendMoData(ServerID.FriendReject, {1}) -- 同意好友申请
			end
		end
    end
	-- UIEventListener.Get(self.btnBack).onClick = doClick
	-- UIEventListener.Get(self.btnAgreeAll).onClick = doClick
	-- UIEventListener.Get(self.btnDenyAll).onClick = doClick
	-- UIEventListener.Get(self.checkBox).onClick = doClick
	ViewUtil.AddClickEvent(self.btnBack, doClick)
	ViewUtil.AddClickEvent(self.btnAgreeAll, doClick)
	ViewUtil.AddClickEvent(self.btnDenyAll, doClick)
	ViewUtil.AddClickEvent(self.checkBox, doClick)
end

function FriendNoticeView:initData(data)
	if IsEmpty(data) then return end
	local list = data.notifyList
	if IsEmpty(list) then return end

	DestroyChildren(self.gridObj)
--	if PlayerData.GetRefuseFriendNotice() then return end

	local function _callback(obj)
		local lb = GetComponent(obj, "LuaBehaviour")
		local src = lb.sourceLuaObj
		local sw = src.sw
		local friendId = src.info.friendTeamId

		local function _call(session, tbl)
			local mo = GetProtoTable("DefaultData", session)
			plog(mo, sw.." friend")
			
			if mo.code == 80061 then
				ViewUtil.showMsg("好友已达上限，无法添加")
			elseif mo.code == 80062 then
				ViewUtil.showMsg("对方好友已达上限，无法添加")
			elseif mo.code == 0 then
				local bar = tbl.obj
				if bar ~= nil then
					DestroyObject(bar)
					ResetGrid(self.gridObj)
				end
				ViewUtil.showMsg("添加好友成功")
			end
		end
		if sw == "Agree" then
			Network.SendMoData(ServerID.FriendComfirm, {friendId, 1}, _call, {obj = src.obj})
			-- ViewUtil.showMsg("添加好友成功")
		else
			Network.SendMoData(ServerID.FriendComfirm, {friendId, 2}, _call, {obj = src.obj})
		end
	end
	
	local limit = #list
	if limit > 20 then limit = 20 end
	for i = 1, limit do
		local info = list[i]
		if not IsEmpty(info) then
			local name = tostring(info.friendName)
			if not IsEmptyString(name) then
				local bar = CreateObjectByPath("UI/BarFriendNotice", AssetBundleInfo.assetBundles)
				bar.name = name
				HangObject(bar, self.gridObj)
				BindLuaScript({info = info}, bar)
				plog("@FriendNotice name = %s", bar.name)
			
				local head = GetGameObject(bar.transform, "Head")
				local conf = self.herosConf[tostring(info.friendLogo)]
				if not IsEmptyTable(conf) then
					local baseHero = BaseHero.new()
					baseHero:setData(conf)
					local item = baseHero:createHero("UI/Hero")
					HangObject(item, head)
				end

				local btnAgree = GetGameObject(bar.transform, "BtnAgree")
				local btnDeny = GetGameObject(bar.transform, "BtnDeny")
				BindLuaScript({sw = "Agree", obj = bar, info = info}, btnAgree)
				BindLuaScript({sw = "Deny", obj = bar, info = info}, btnDeny)

				-- UIEventListener.Get(btnAgree).onClick = _callback
				-- UIEventListener.Get(btnDeny).onClick = _callback
				ViewUtil.AddClickEvent(btnAgree, _callback)
				ViewUtil.AddClickEvent(btnDeny, _callback)
			
				local lblName = GetPathComponent(bar.transform, "Name", "UILabel")
				lblName.text = info.friendName
			end
		end
	end
	
	ResetGrid(self.gridObj)
	ResetScrollView(self.scvObj)

	PlayerData.playerData.status.friend = false
	Facade:sendNotification(MVCConst.SHOWREDICON, {})
end
