--觉醒详细面板
--2016-10-13 10:43:33 library

local const_wakeupLvl = "觉醒等级";
local const_abilityUp = "战力提升";
local const_wakeupCfm = "确认觉醒";

local const_wakeupDesc = "\n觉醒可以提高英雄的战力，觉醒需要消耗材料";
local const_lackOfGold = "金币不足";
local const_maxLvl = "已达到最大等级";
local const_lackOfProp = "道具数量不足";
local const_wiseupSuccess = "觉醒成功";
local const_needGold = "消耗 {1} 金币";

HUWakeupPane = class();

--[[

]]
function HUWakeupPane:ctor ()
    self.gameObject = nil;
    self.transform = nil;
end

--[[

]]
function HUWakeupPane:Awake (_go)
    self.gameObject = _go;
    self.transform = self.gameObject.transform;

    self.heroData = nil;
    self.heroId = nil;
    self.id = nil;
    self.lvl = nil;
    self.wiseup = nil;--觉醒等级
    self.strengthen = nil;
    self.synDataFunc = nil;
    self.closeFunc = nil;
    self.costGold = nil;

    self:initPanel();
    self:initListener();
end

--[[

]]
function HUWakeupPane:initPanel ()
    self.hero = GetGameObject(self.transform, "heroInfoPane/Hero");
    self.heroBG = GetGameObject(self.hero.transform, "BG");
    self.heroSprite = GetGameObject(self.hero.transform, "Sprite");

    self.sLvlWord = GetGameObject(self.transform, "heroInfoPane/sLvl/sLvlWord");
    self.scurLvl = GetGameObject(self.transform, "heroInfoPane/sLvl/curLvl");
    self.snextLvl = GetGameObject(self.transform, "heroInfoPane/sLvl/nextLvl");

    self.aLvlWord = GetGameObject(self.transform, "heroInfoPane/abilityAdd/sLvlWord");
    self.acurLvl = GetGameObject(self.transform, "heroInfoPane/abilityAdd/curLvl");
    self.anextLvl = GetGameObject(self.transform, "heroInfoPane/abilityAdd/nextLvl");

    self.propListPane = GetGameObject(self.transform, "propListPane");
    self.propGrid = GetGameObject(self.propListPane.transform, "grid");

    self.wakeupBtn = GetGameObject(self.transform, "strongBtn");
    self.wakeupBtnText = GetGameObject(self.wakeupBtn.transform, "text");

    self.desc = GetGameObject(self.transform, "mnyNeedInfo/desc");
    
    GetComponent(self.sLvlWord, "UILabel").text = const_wakeupLvl;
    GetComponent(self.aLvlWord, "UILabel").text = const_abilityUp;
    GetComponent(self.wakeupBtnText, "UILabel").text = const_wakeupCfm;
    GetComponent(self.desc, "UILabel").text = const_wakeupDesc;
end

--[[

]]
function HUWakeupPane:initListener ()
    -- UIEventListener.Get(self.wakeupBtn).onClick = function () self:wakeupHandle(); end;
    ViewUtil.AddClickEvent(self.wakeupBtn, function () self:wakeupHandle(); end)
end

--[[
    _heroId number
    _id number 英雄的唯一ID
    _lvl number 升级等级
    _wiseup number 觉醒等级
    _strengthen number 强化等级
    _cb function 同步数据的回调
    _closeFunc function 关闭面板的回调
]]
function HUWakeupPane:setData (_heroId, _id, _lvl, _wiseup, _strengthen, _synDataFunc, _closeFunc)
    self.heroId = _heroId;
    self.id = _id;
    self.lvl = _lvl;
    self.wiseup = _wiseup;
    self.strengthen = _strengthen;
    
    self.synDataFunc = _synDataFunc;
    self.closeFunc = _closeFunc;

    self:showHero();
    self:addProp();
    self:panelInfoRefresh();
end

--[[
    显示英雄
]]
function HUWakeupPane:showHero ()
    local hCfg, hQua, pName = nil, nil, nil;
    local path = "heroInfoPane/Hero/";

    hCfg = HeroDataProxy.getHeroCfgById(self.heroId);
    hQua = HeroDataProxy.getHeroQuality(hCfg);
    pName = HeroDataProxy.getHeroAtlasName(hCfg);

    GetPathComponent(self.transform, path .. "BG", "UISprite").spriteName = "class" .. hQua;
    GetPathComponent(self.transform, path .. "Sprite", "UISprite").spriteName = pName;
end

--[[
    
]]
function HUWakeupPane:wakeupHandle ()
    local mapData, buf = Map.new(), nil;
    local myGold = PlayerData.getMyGold();
    local props = HeroDataProxy.getHeroWiseupProps(self.wiseup);
    local pNum, isLackOfProp = 0, false;

    Util.EnableButton2D(self.wakeupBtn, 3, false);
    if (myGold < self.costGold) then
        ViewUtil.showMsg(const_lackOfGold);
        Util.EnableButton2D(self.wakeupBtn, 0, true);
        return ;
    end

    if (self.wiseup >= HeroDataProxy.maxWakeLvl) then
        ViewUtil.showMsg(const_maxLvl);
        return ;
    end

    for i = 1, #props do
        local pNum = PropDataProxy.getPropNumByPropId(props[i][1]);
        if (pNum < tonumber(props[i][2])) then
            ViewUtil.showMsg(const_lackOfProp);
            isLackOfProp = true;
            break ;
        end
    end

    if (isLackOfProp) then
        return ;
    end
    mapData:put("id", self.id);
    mapData:put("level", self.lvl);

    buf = Network.reqParamEncode(mapData);

    NetManager:SendData(self, buf, ServerID.HU_WakeUp, ModuleName.HeroUpgrade, doNthFunc, nil, doNthFunc);

    ViewUtil.showMsg(const_wiseupSuccess);
    Util.EnableButton2D(self.wakeupBtn, 0, true);
    self:synData();

    if (nil ~= self.synDataFunc) then
        self.synDataFunc();
    end

    if (nil ~= self.closeFunc) then
        self.closeFunc();
    end
end

--[[
    同步数据
]]
function HUWakeupPane:synData ()
    local hData = nil;

    hData = TeamData.getMyHeroDataById(self.id);

    if (nil ~= hData) then
        hData.wiseup = self.wiseup + 1;
    end

    self:addProp();
    self:panelInfoRefresh();
end

--[[

]]
function HUWakeupPane:addProp ()
    local luaBhvi, spPane = nil, nil;--LuaBehaviour, HUShowPropsPane

    luaBhvi = GetComponent(self.propGrid, "LuaBehaviour");
    if (nil ~= luaBhvi) then
        spPane = luaBhvi.sourceLuaObj;
    end

    if (nil == spPane) then
        spPane = HUShowPropsPane.new();
        spPane:Awake(self.propGrid);
        PanelManager:BindPanelScript(spPane, self.propGrid, doNthFunc, doNthFunc, doNthFunc, doNthFunc);
    end
    
    spPane:setData(HeroDataProxy.getHeroWiseupProps(self.wiseup));
end

--[[

]]
function HUWakeupPane:panelInfoRefresh ()
    local curAbility, nextAbility, deltaAbility = 0, 0;
    local curWakeLvl, totalWakeLvl = 0, 0;
    local curWiseVal, nextWiseVal = self.wiseup, HeroDataProxy.getHeroWiseByLvl(self.lvl + 1);
    
    curAbility = HeroDataProxy.getHeroAbility(self.heroId, self.lvl, self.wiseup, self.strengthen);
    if (curWiseVal < nextWiseVal) then
        nextAbility = HeroDataProxy.getHeroAbility(self.heroId, self.lvl, nextWiseVal, self.strengthen);
    else
        nextAbility = curAbility;
    end

    self.costGold = HeroDataProxy.getHeroWakeupGold(nextWiseVal);
    GetComponent(self.desc, "UILabel").text = string.gsub(const_needGold, "{1}", LogicUtil.getShowNum(self.costGold)) .. const_wakeupDesc;
    deltaAbility = nextAbility - curAbility;
    deltaAbility = (deltaAbility >= 0 and deltaAbility or 0);

    GetComponent(self.scurLvl, "UILabel").text = "" .. (curWiseVal + 1);
    GetComponent(self.snextLvl, "UILabel").text = "" .. (nextWiseVal + 1);
    GetComponent(self.acurLvl, "UILabel").text = "" .. curAbility;
    GetComponent(self.anextLvl, "UILabel").text = "" .. nextAbility;
end