
LeagueApplyView = class(BaseView)

function LeagueApplyView:ctor()
end  

function LeagueApplyView:Awake(obj)
	self.gameObject = obj
	self.transform = obj.transform
	self:initPanel()
end

function LeagueApplyView:Start(data)
	-- self:initData(data)
end

function LeagueApplyView:initPanel()
 	self.top = GetPathComponent(self.transform, "Mode/Top", "UIWidget")
 	self.bottom = GetPathComponent(self.transform, "Mode/Bottom", "UIWidget")
	self.top:SetAnchor(self.root.transform)
	self.bottom:SetAnchor(self.root.transform)
	
	local topPath = "Mode/Top/Panel"

	local centerPath = "Mode/Center/Panel"
	self.scvBG = GetGameObject(self.transform, centerPath.."/ScrollViewBG")
	self.scvObj = GetGameObject(self.scvBG.transform, "ScrollView")
	self.grid = GetGameObject(self.scvObj.transform, "Panel/Grid")

	-- ViewUtil.adaptUISize(self.scvBG, true, 80, 60, 0, 0)
	-- ViewUtil.adaptUISize(self.scvObj, true, 154, 70, 16, 16)

	local bottomPath = "Mode/Bottom/Panel"
	self.btnBack = GetGameObject(self.transform, bottomPath.."/BtnBack")
	self.btnCreate = GetGameObject(self.transform, bottomPath.."/BtnCreate")

    local function doClick(obj)
		
		if obj == self.btnBack then
			ViewUtil.openView(ModuleName.MAIN)

		elseif obj == self.btnCreate then
			self:createLeagueDialog()
		end
    end

	ViewUtil.AddClickEvent(self.btnBack, doClick)
	ViewUtil.AddClickEvent(self.btnCreate, doClick)
end

function LeagueApplyView:createLeagueDialog()
	local dialog = CreateObjectByPath("UI/LeagueCreate", AssetBundleInfo.assetBundles)
	HangObject(ViewUtil.CreateShadeView(), dialog)
	HangObject(dialog, self.gameObject)
	ViewUtil.sortLayerOrder(dialog, LayerName.Dialog)
	
	local centerPath = "Mode/Center/Panel"
	local btnCancel = GetGameObject(dialog.transform, centerPath.."/BtnCancel")
	local btnConfirm = GetGameObject(dialog.transform, centerPath.."/BtnConfirm")
	local lblName = GetPathComponent(dialog.transform, centerPath.."/LeagueName/Input", "UILabel")

    local function doClick(obj)
		
		if obj == btnCancel then
			DestroyObject(dialog)
		elseif obj == btnConfirm then
			local txt = lblName.text
			if IsEmpty(txt) or txt == GetWord("InputTips") then
				ViewUtil.showMsg(GetWord("InputError"))
				return
			end
			if TeamData.GetGem() < 500 then
				ViewUtil.showMsg("钻石不足，需要500钻石！")
				return
			end
			local function _OnCallbackData(session)
				local serviceId = tonumber(session.serviceId)
				local mo = GetProtoTable("DefaultData", session)

				if mo.code == 0 then
					ViewUtil.showMsg(GetWord("CreateSuccess"))
					ViewUtil.closeView(ModuleName.LeagueApply)
					ViewUtil.goto(ModuleName.LeagueJoined)
				elseif mo.code == 80080 then
					ViewUtil.showMsg("您已加入其他军团！")
				else
					ViewUtil.showMsg(GetWord(mo.code))
				end
			end
			Network.SendMoData(ServerID.LeagueCreate, {txt}, _OnCallbackData)
		end
    end

	ViewUtil.AddClickEvent(btnCancel, doClick)
	ViewUtil.AddClickEvent(btnConfirm, doClick)
end

function LeagueApplyView:initData(data)
	if IsEmpty(data) then return end
	
	local function _callback(obj)
		local function _call(session, params)
			local serviceId = tonumber(session.serviceId)
			if serviceId == ServerID.LeagueApplyJoin then
				local mo = GetProtoTable("DefaultData", session)
				if mo.code == 0 then
					ViewUtil.showMsg(GetWord("ApplySuccess"))
					local obj = params.obj
					local lbl = GetPathComponent(obj.transform, "Title", "UILabel")
					lbl.text = GetWord("Applying")
--					local btn = GetComponent(obj.transform, "UIButton")
					Util.EnableButton(obj, ButtonState.Disabled, false)
				elseif mo.code == 80086 then
					ViewUtil.showMsg("距离上次退出军团不足4小时，无法申请！")
				end
			end
		end

		local lb = GetComponent(obj.transform, "LuaBehaviour")
		if not IsEmpty(lb) then
			local luaClass = lb.sourceLuaObj
			local leagueId = tonumber(luaClass.leagueId) or 0
			if leagueId and leagueId > 0 then
				Network.SendMoData(ServerID.LeagueApplyJoin, {leagueId}, _call, {obj = obj})
			end
		end
	end

	local all = data.allLeague
	local sortList = {}
	sortList = all
	local tempData = nil
	for i = 1, #sortList - 1 do
		for j = i + 1, #sortList do
			if sortList[i].totalPower < sortList[j].totalPower then
				tempData = sortList[i]
				sortList[i] = sortList[j]
				sortList[j] = tempData
			end
		end
	end

	for i = 1, #sortList do
		local army = sortList[i]
		local bar = CreateObjectByPath("UI/LeagueApply", AssetBundleInfo.assetBundles)
		HangObject(bar, self.grid)
		bar.name = "Army"..i
		local btnReq = GetGameObject(bar.transform, "BtnReq")
		BindLuaScript(army, btnReq)
		-- UIEventListener.Get(btnReq).onClick = _callback
		ViewUtil.AddClickEvent(btnReq, _callback)

		if all[i].status == 1 then
			GetPathComponent(btnReq.transform, "Title", "UILabel").text = GetWord("Applying")
			Util.EnableButton(btnReq, ButtonState.Disabled, false)
		end

		local name = GetPathComponent(bar.transform, "Name", "UILabel")
		local member = GetPathComponent(bar.transform, "Member", "UILabel")
		local fightValue = GetPathComponent(bar.transform, "FightValue", "UILabel")
		local leader = GetPathComponent(bar.transform, "Leader", "UILabel")
		local requirement = GetPathComponent(bar.transform, "Require", "UILabel")
		local badgeSpr = GetPathComponent(bar.transform, "Badge", "UISprite")

		name.text = army.leagueName
		member.text = army.count.."/30"
		fightValue.text = army.totalPower
		leader.text = army.teamName
		requirement.text = army.limitPower
		badgeSpr.spriteName = "LeagueBadge_"..army.logoId
	end

	ResetGrid(self.grid)
	ResetScrollView(self.scvObj)
end
