
LeagueNoticeView = class(BaseView)

function LeagueNoticeView:ctor()
end  

function LeagueNoticeView:Awake(obj)
	self.gameObject = obj
	self.transform = obj.transform
	self.shadeOpacity = true
	self:initPanel()
end

function LeagueNoticeView:Start(data)
	-- self:initData(data)
end

function LeagueNoticeView:initPanel()
 	self.top = GetPathComponent(self.transform, "Mode/Top", "UIWidget")
 	self.bottom = GetPathComponent(self.transform, "Mode/Bottom", "UIWidget")
	self.top:SetAnchor(self.root.transform)
	self.bottom:SetAnchor(self.root.transform)
		
	local topPath = "Mode/Top/Panel"
	self.btnExit = GetGameObject(self.transform, topPath.."/BtnExit")

	local centerPath = "Mode/Center/Panel"
	self.scvBG = GetGameObject(self.transform, centerPath.."/ScrollViewBG")
	self.scvObj = GetGameObject(self.scvBG.transform, "ScrollView")
	self.gridObj = GetGameObject(self.scvObj.transform, "Panel/Grid")
	self.grid = GetPathComponent(self.scvObj.transform, "Panel/Grid", "UIGrid")
	
	self.btnAgreeAll = GetGameObject(self.transform, centerPath.."/BtnAgreeAll")
	self.btnDenyAll = GetGameObject(self.transform, centerPath.."/BtnDenyAll")

	ViewUtil.adaptUISize(self.scvBG, true, 100, 60, 0, 0)
	ViewUtil.adaptUISize(self.scvObj, true, 174, 150, 16, 16)

	local bottomPath = "Mode/Bottom/Panel"

    local function doClick(obj)
		
		if obj == self.btnExit then
			self:closeView(true)
		elseif obj == self.btnAgreeAll then
			local list = self.grid:GetChildList()
			for i = 0, list.Count - 1 do
				local obj = list[i].gameObject
				local lb = GetComponent(obj.transform, "LuaBehaviour")
				local src = lb.sourceLuaObj
				local info = src.info
				Network.SendMoData(ServerID.LeagueAllowJoin, {info.teamId})
				DestroyObject(obj)
			end
		elseif obj == self.btnDenyAll then
			local list = self.grid:GetChildList()
			for i = 0, list.Count - 1 do
				local obj = list[i].gameObject
				local lb = GetComponent(obj.transform, "LuaBehaviour")
				local src = lb.sourceLuaObj
				local info = src.info
				Network.SendMoData(ServerID.LeagueDenyJoin, {info.teamId})
				DestroyObject(obj)
			end
		end
    end
    
	ViewUtil.AddClickEvent(self.btnExit, doClick)
	ViewUtil.AddClickEvent(self.btnAgreeAll, doClick)
	ViewUtil.AddClickEvent(self.btnDenyAll, doClick)
end

function LeagueNoticeView:initData(data)
	if IsEmpty(data) then return end

	local function _callback(obj)
		local lb = GetComponent(obj, "LuaBehaviour")
		local src = lb.sourceLuaObj
		local sw = src.sw
		local teamId = src.info.teamId

		local function _call(session, tbl)
			local serviceId = tonumber(session.serviceId)
			if serviceId == ServerID.LeagueAllowJoin then
				local mo = GetProtoTable("DefaultData", session)
				if mo.code == 0 then
					ViewUtil.showMsg("已同意加入军团！")
					local bar = tbl.obj
					if bar ~= nil then
						DestroyObject(bar)
						ResetScrollView(self.scvObj)
					end
				elseif mo.code == 80080 then
					ViewUtil.showMsg("对方已加入军团！")
					local bar = tbl.obj
					if bar ~= nil then
						DestroyObject(bar)
						ResetScrollView(self.scvObj)
					end
				else
					ViewUtil.showMsg(GetWord(mo.code))
				end
			elseif serviceId == ServerID.LeagueDenyJoin then
				local mo = GetProtoTable("DefaultData", session)
				if mo.code == 0 then
					ViewUtil.showMsg("已拒绝加入军团！")
					local bar = tbl.obj
					if bar ~= nil then
						DestroyObject(bar)
						ResetScrollView(self.scvObj)
					end
				else
					ViewUtil.showMsg(GetWord(mo.code))
				end
			end
		end
		if sw == "Agree" then
			Network.SendMoData(ServerID.LeagueAllowJoin, {teamId}, _call, {obj = src.obj})
		else
			Network.SendMoData(ServerID.LeagueDenyJoin, {teamId}, _call, {obj = src.obj})
		end
	end

	local list = data.teamListData
	for i = 1, #list do
		local info = list[i]
		if not IsEmpty(info) then
			local bar = CreateObjectByPath("UI/LeagueNotice", AssetBundleInfo.assetBundles)
			HangObject(bar, self.gridObj)
			bar.name = info.teamId
			BindLuaScript({info = info}, bar)	
				
			local btnAgree = GetGameObject(bar.transform, "BtnAgree")
			local btnDeny = GetGameObject(bar.transform, "BtnDeny")
			BindLuaScript({sw = "Agree", obj = bar, info = info}, btnAgree)
			BindLuaScript({sw = "Deny", obj = bar, info = info}, btnDeny)

			-- UIEventListener.Get(btnAgree).onClick = _callback
			-- UIEventListener.Get(btnDeny).onClick = _callback
			ViewUtil.AddClickEvent(btnAgree, _callback)
			ViewUtil.AddClickEvent(btnDeny, _callback)

			local name = GetPathComponent(bar.transform, "Name", "UILabel")
			local fightValue = GetPathComponent(bar.transform, "FightValue", "UILabel")
			name.text = info.name
			fightValue.text = info.power
		end
	end
	
	ResetGrid(self.gridObj)
	ResetScrollView(self.scvObj)
end

