
LeaguePVERewardBoard = class();

--[[

]]
function LeaguePVERewardBoard:ctor ()
    self.gameObject = nil
    self.transform = nil
end

--[[

]]
function LeaguePVERewardBoard:Awake (_go)
    self.gameObject = _go
    self.transform = self.gameObject.transform

    self:initPanel()
end

--[[

]]
function LeaguePVERewardBoard:initPanel ()
    local topPath = "Mode/Top/Panel"
    self.btnClose = GetGameObject(self.transform, topPath.."/BtnClose")
    self.rewardApply = GetGameObject(self.transform, topPath.."/RewardApply")
    self.record = GetGameObject(self.transform, topPath.."/Record")

    self.panel_1 = GetGameObject(self.transform, "Mode/Center/Panel_1")
    self.uiGrid_1 = GetGameObject(self.panel_1.transform, "ScrollViewBG/ScrollView/Panel/Grid")
    -- self.rewardApplyRow = GetGameObject(self.uiGrid_1.transform, "RewardApplyRow")
    self.panel_2 = GetGameObject(self.transform, "Mode/Center/Panel_2")
    self.uiGrid_2 = GetGameObject(self.panel_2.transform, "ScrollViewBG/ScrollView/Panel/Grid")
    self.recordRow = GetGameObject(self.uiGrid_2.transform, "RecordRow")

    local bottomPath = "Mode/Bottom/Panel"

    local function _callback(obj)
        if obj == self.btnClose then
            DestroyObject(self.gameObject)
        elseif obj == self.rewardApply then
            self:SlideTab(1)
        elseif obj == self.record then
            self:SlideTab(2)
        end
    end
    ViewUtil.AddClickEvent(self.btnClose, _callback)
    ViewUtil.AddClickEvent(self.rewardApply, _callback)
    ViewUtil.AddClickEvent(self.record, _callback)

    self:SlideTab(1)
end

function LeaguePVERewardBoard:SlideTab(_tp)
    if _tp == 1 then
        Util.EnableButton(self.rewardApply, ButtonState.Disabled, false)
        Util.EnableButton(self.record, ButtonState.Normal, true)
        self.panel_1:SetActive(true)
        self.panel_2:SetActive(false)
        self:CreateBars(1)
    elseif _tp == 2 then
        Util.EnableButton(self.record, ButtonState.Disabled, false)
        Util.EnableButton(self.rewardApply, ButtonState.Normal, true)
        self.panel_1:SetActive(false)
        self.panel_2:SetActive(true)
        self:CreateBars(2)
    end
end

function LeaguePVERewardBoard:CreateBars(_tp)
    if _tp == 1 then
        local mapData, buf = nil, nil;

        mapData = Map.new();
        buf = Network.reqParamEncode(mapData);

        NetManager:SendData(self, buf, ServerID.LeaguePVEReward, nil, function (sd) self:ShowRewardPane(sd) end, nil, doNthFunc)
    elseif _tp == 2 then
        local mapData, buf = nil, nil;

        mapData = Map.new();
        buf = Network.reqParamEncode(mapData);

        NetManager:SendData(self, buf, ServerID.LeaguePVERecord, nil, function (sd)
            local lpverMain = GetProtoTable("LeagueHuntGiftLogMainData", sd)
            local recordList = lpverMain.giftLogList

            for i = 1, #recordList do
                local info = recordList[i]

                local tempObj = CreateObject(self.recordRow)
                tempObj:SetActive(true)
                HangObject(tempObj, self.uiGrid_2)

                local propItem = CreateObject(Util.LoadPrefab("UI/PropItem", AssetBundleInfo.assetBundles));
                local propItemShow = PropShowItem.new();
                propItemShow:Awake(propItem, info.propId);
                propItemShow:Start(true);
                PanelManager:BindPanelScript(propItemShow, propItem, doNthFunc, doNthFunc, doNthFunc, doNthFunc);
                HangObject(propItem, tempObj);
                ViewUtil.sortLayerOrder(propItem, LayerName.Dialog)
                propItem.transform.localPosition = Vector3.New(-195, 0, 0)

                GetPathComponent(tempObj.transform, "Name", "UILabel").text = info.teamName
                local pCfg = PropDataProxy.getPropCfgById(info.propId)
                GetPathComponent(tempObj.transform, "Prop", "UILabel").text = PropDataProxy.getPropName(pCfg)
                GetPathComponent(tempObj.transform, "Time", "UILabel").text = Time2String(info.time / 1000)
            end
            ResetGrid(self.uiGrid_2)
        end, nil, doNthFunc)
    end
end

function LeaguePVERewardBoard:ShowRewardPane(sd)
    local lpveraMain = GetProtoTable("LeagueHuntGiftMainData", sd)
    self.applyList = lpveraMain.giftList
    self.myApply = lpveraMain.myApply

    self.applyRows = {}
    self.btnApplys = {}
    self.teamCounts = {}

    if self.myApply == 0 then
        self:NotApplyYet()
    else
        self:HadApplyed(self.myApply)
    end
end

function LeaguePVERewardBoard:NotApplyYet()
    DestroyChildren(self.uiGrid_1)

    for i = 1, #self.applyList do
        local info = self.applyList[i]

        -- self.applyRows[i] = CreateObject(self.rewardApplyRow)
        self.applyRows[i] = CreateObject(Util.LoadPrefab("UI/LeaguePVERewardApplyRow", AssetBundleInfo.assetBundles))
        self.applyRows[i].name = "RewardApplyRow_"..i
        self.applyRows[i]:SetActive(true)
        HangObject(self.applyRows[i], self.uiGrid_1)

        local propItem = CreateObject(Util.LoadPrefab("UI/PropItem", AssetBundleInfo.assetBundles));
        local propItemShow = PropShowItem.new();
        propItemShow:Awake(propItem, info.propId, info.num);
        propItemShow:Start(true);
        PanelManager:BindPanelScript(propItemShow, propItem, doNthFunc, doNthFunc, doNthFunc, doNthFunc);
        HangObject(propItem, self.applyRows[i]);
        ViewUtil.sortLayerOrder(propItem, LayerName.Dialog)
        propItem.transform.localPosition = Vector3.New(-195, 0, 0)

        GetPathComponent(self.applyRows[i].transform, "Name", "UILabel").text = PropDataProxy.getPropName(PropDataProxy.getPropCfgById(info.propId))

        if self.teamCounts[i] == nil then
            self.teamCounts[i] = info.teamCount
        end
        GetPathComponent(self.applyRows[i].transform, "ApplyNum", "UILabel").text = "已有"..self.teamCounts[i].."人申请"
        
        self.btnApplys[i] = GetGameObject(self.applyRows[i].transform, "BtnApply")
        GetPathComponent(self.btnApplys[i].transform, "Title", "UILabel").text = "申请"

        local function Handler(session)
            local serviceId = tonumber(session.serviceId)
            if serviceId == ServerID.LeaguePVERewardApply then
                local mo = GetProtoTable("DefaultData", session)
                if mo.code ~= 0 then
                    ViewUtil.showMsg(GetWord(mo.code))
                    return
                end
                ViewUtil.showMsg("申请成功！")
                self.teamCounts[i] = self.teamCounts[i] + 1
                self:HadApplyed(info.propId)
            end
        end

        local function _callback(obj)
            if obj == self.btnApplys[i] then
                local mapData = Map.new();
                mapData:put("propId", info.propId)
                local buf = Network.reqParamEncode(mapData);

                NetManager:SendData(self, buf, ServerID.LeaguePVERewardApply, nil, function (sd) Handler(sd) end, nil, doNthFunc)
            end
        end
        ViewUtil.AddClickEvent(self.btnApplys[i], _callback)
    end
    ResetGrid(self.uiGrid_1)
end

function LeaguePVERewardBoard:HadApplyed(_applyId)
    DestroyChildren(self.uiGrid_1)
    for i = 1, #self.applyList do
        local info = self.applyList[i]

        -- self.applyRows[i] = CreateObject(self.rewardApplyRow)
        self.applyRows[i] = CreateObject(Util.LoadPrefab("UI/LeaguePVERewardApplyRow", AssetBundleInfo.assetBundles))
        self.applyRows[i].name = "RewardApplyRow_"..i
        self.applyRows[i]:SetActive(true)
        HangObject(self.applyRows[i], self.uiGrid_1)

        local propItem = CreateObject(Util.LoadPrefab("UI/PropItem", AssetBundleInfo.assetBundles));
        local propItemShow = PropShowItem.new();
        propItemShow:Awake(propItem, info.propId, info.num);
        propItemShow:Start(true);
        PanelManager:BindPanelScript(propItemShow, propItem, doNthFunc, doNthFunc, doNthFunc, doNthFunc);
        HangObject(propItem, self.applyRows[i]);
        ViewUtil.sortLayerOrder(propItem, LayerName.Dialog)
        propItem.transform.localPosition = Vector3.New(-195, 0, 0)

        GetPathComponent(self.applyRows[i].transform, "Name", "UILabel").text = PropDataProxy.getPropName(PropDataProxy.getPropCfgById(info.propId))

        if self.teamCounts[i] == nil then
            self.teamCounts[i] = info.teamCount
        end
        GetPathComponent(self.applyRows[i].transform, "ApplyNum", "UILabel").text = "已有"..self.teamCounts[i].."人申请"
        
        self.btnApplys[i] = GetGameObject(self.applyRows[i].transform, "BtnApply")
        if _applyId == info.propId then
            GetPathComponent(self.btnApplys[i].transform, "Title", "UILabel").text = "取消申请"
        else
            self.btnApplys[i]:SetActive(false)
        end

        local function Handler(session)
            local serviceId = tonumber(session.serviceId)
            if serviceId == ServerID.LeaguePVERewardCancel then
                local mo = GetProtoTable("DefaultData", session)
                if mo.code ~= 0 then
                    ViewUtil.showMsg(GetWord(mo.code))
                    return
                end
                ViewUtil.showMsg("已取消申请！")
                self.teamCounts[i] = self.teamCounts[i] - 1
                self:NotApplyYet()
            end
        end

        local function _callback(obj)
            if obj == self.btnApplys[i] then
                local function DonateOKFun()
                    local mapData = Map.new();
                    local buf = Network.reqParamEncode(mapData);

                    NetManager:SendData(self, buf, ServerID.LeaguePVERewardCancel, nil, function (sd) Handler(sd) end, nil, doNthFunc)
                end

                ViewUtil.comCfmPanel("退出申请需要重新排队，请确认是否退出。", function () DonateOKFun(); end);
            end
        end
        ViewUtil.AddClickEvent(self.btnApplys[i], _callback)
    end
    ResetGrid(self.uiGrid_1)
end