
LeaguePVEView = class(BaseView)

function LeaguePVEView:ctor()
end  

function LeaguePVEView:Awake(obj)
	self.gameObject = obj
	self.transform = obj.transform
	self.herosConf = Config.getHerosData()
	self:initPanel()
end

function LeaguePVEView:Start(data)
	-- self:initData(data)
end

function LeaguePVEView:initPanel()
 	self.top = GetPathComponent(self.transform, "Mode/Top", "UIWidget")
 	self.bottom = GetPathComponent(self.transform, "Mode/Bottom", "UIWidget")
	self.top:SetAnchor(self.root.transform)
	self.bottom:SetAnchor(self.root.transform)
	
	local topPath = "Mode/Top/Panel"

	local centerPath = "Mode/Center/Panel"
	self.bgSpr = GetPathComponent(self.transform, centerPath.."/mainBG/Sprite", "UISprite")
	self.btnRank = GetGameObject(self.transform, centerPath.."/BtnRank")
	self.btnReward = GetGameObject(self.transform, centerPath.."/BtnReward")
	self.btnRule = GetGameObject(self.transform, centerPath.."/BtnRule")
	self.btnChest = GetGameObject(self.transform, centerPath.."/BtnChest")
	self.btnApply = GetGameObject(self.transform, centerPath.."/BtnApply")
	self.huntEnd = GetGameObject(self.transform, centerPath.."/Finish")
	self.btnReset = GetGameObject(self.transform, centerPath.."/BtnReset")

	local bottomPath = "Mode/Bottom/Panel"
	self.btnBack = GetGameObject(self.transform, bottomPath.."/BtnBack")

    local function doClick(obj)		
		if obj == self.btnBack then
			ViewUtil.openView(ModuleName.LeagueJoined)
		elseif obj == self.btnRank then
			self:createRankBoard()
		elseif obj == self.btnReward then
			self:createRewardBoard()
		elseif obj == self.btnRule then
			self:createRuleBoard()
		elseif obj == self.btnChest then
			self:createChestBoard()
		elseif obj == self.btnApply then
			ViewUtil.openView(ModuleName.LeaguePVEMap)
		elseif obj == self.btnReset then
			Network.SendMoData(ServerID.LeaguePVEReset, {}, function(session, params)
				local serviceId = tonumber(session.serviceId)
				local mo = GetProtoTable("DefaultData", session)
				if mo.code == 0 then
					self.huntEnd:SetActive(false)
					self.btnReset:SetActive(false)
					self.btnApply:SetActive(true)
					self.bgSpr.color = Color.New(1, 1, 1, 1)
					ViewUtil.showMsg("刷新成功！")
				else
					ViewUtil.showMsg("刷新失败！")
				end
			end, {})
		end
    end

	ViewUtil.AddClickEvent(self.btnBack, doClick)
	ViewUtil.AddClickEvent(self.btnRank, doClick)
	ViewUtil.AddClickEvent(self.btnReward, doClick)
	ViewUtil.AddClickEvent(self.btnRule, doClick)
	ViewUtil.AddClickEvent(self.btnChest, doClick)
	ViewUtil.AddClickEvent(self.btnApply, doClick)
	ViewUtil.AddClickEvent(self.btnReset, doClick)
end

function LeaguePVEView:initData(data)
	if IsEmpty(data) then return end

	local huntStatus = data.code
	local refreshSteatus = data.msg
	if huntStatus == 0 then
		self.btnApply:SetActive(true)
	else
		self.huntEnd:SetActive(true)
		self.bgSpr.color = Color.New(0, 0, 0, 1)

		if refreshSteatus == "0" and LeagueData.IsCaptain() then
			self.btnReset:SetActive(true)
		end
	end
end

function LeaguePVEView:createRankBoard()
	local dialog = CreateObjectByPath("UI/LeaguePVERank", AssetBundleInfo.assetBundles)
	HangObject(ViewUtil.CreateShadeView(), dialog)
	HangObject(dialog, self.gameObject)
	ViewUtil.sortLayerOrder(dialog, LayerName.Dialog)

	local boardScr = LeaguePVERankBoard.new()
    PanelManager:BindPanelScript(boardScr, dialog, doNthFunc, doNthFunc, doNthFunc, doNthFunc)
    boardScr:Awake(dialog)
end

function LeaguePVEView:createRewardBoard()
	local dialog = CreateObjectByPath("UI/LeaguePVEReward", AssetBundleInfo.assetBundles)
	HangObject(ViewUtil.CreateShadeView(), dialog)
	HangObject(dialog, self.gameObject)
	ViewUtil.sortLayerOrder(dialog, LayerName.Dialog)

	local boardScr = LeaguePVERewardBoard.new()
    PanelManager:BindPanelScript(boardScr, dialog, doNthFunc, doNthFunc, doNthFunc, doNthFunc)
    boardScr:Awake(dialog)
end

function LeaguePVEView:createRuleBoard()
	local dialog = CreateObjectByPath("UI/LeaguePVERule", AssetBundleInfo.assetBundles)
	HangObject(ViewUtil.CreateShadeView(), dialog)
	HangObject(dialog, self.gameObject)
	ViewUtil.sortLayerOrder(dialog, LayerName.Dialog)

	local centerPath = "Mode/Center/Panel"
	local ruleLbl = GetPathComponent(dialog.transform, centerPath.."/ScrollViewBG/ScrollView/Panel/Label", "UILabel")

	local bottomPath = "Mode/Bottom/Panel"
	local btnExit = GetGameObject(dialog.transform, bottomPath.."/BtnExit")

	local function _callback(obj)
		if obj == btnExit then
			DestroyObject(dialog)
		end
	end

	ViewUtil.AddClickEvent(btnExit, _callback)
end

function LeaguePVEView:createChestBoard()
	local dialog = CreateObjectByPath("UI/LeaguePVEChest", AssetBundleInfo.assetBundles)
	HangObject(ViewUtil.CreateShadeView(), dialog)
	HangObject(dialog, self.gameObject)
	ViewUtil.sortLayerOrder(dialog, LayerName.Dialog)

	local topPath = "Mode/Top/Panel"
	local btnClose = GetGameObject(dialog.transform, topPath.."/BtnClose")

	local centerPath = "Mode/Center/Panel"
	local gridObj = GetGameObject(dialog.transform, centerPath.."/ScrollViewBG/ScrollView/Panel/Grid")
	local log = GetGameObject(gridObj.transform, "Label")
	local boxsObj = {}
	for i = 1, 3 do
		boxsObj[i] = GetGameObject(dialog.transform, centerPath.."/Chest_"..i)
	end

	local bottomPath = "Mode/Bottom/Panel"

	local function _callback(obj)
		if obj == btnClose then
			DestroyObject(dialog)
		end
	end
	ViewUtil.AddClickEvent(btnClose, _callback)

	for i = 1, 3 do
		local function _doClick(obj)
			if obj == boxsObj[i] then
				Network.SendMoData(ServerID.LeaguePVEOpenBox, {i}, function(session, params)
					local serviceId = tonumber(session.serviceId)
					local mo = GetProtoTable("DefaultData", session)
					if (0 ~= mo.code) then
				        ViewUtil.showMsg(GetWord(mo.code));
				        return ;
	    			end
	    			ViewUtil.showMsg("领取成功！");
	    			self.counts[i] = self.counts[i] - 1
	    			if self.counts[i] == 0 then
	    				Util.EnableButton(boxsObj[i], ButtonState.Disabled, false)
						GetGameObject(boxsObj[i].transform, "Status/Open"):SetActive(false)
						GetGameObject(boxsObj[i].transform, "Status/Close"):SetActive(true)
						GetPathComponent(boxsObj[i].transform, "Sprite", "UISprite").spriteName = "League_ChestOpen"
					end
					GetPathComponent(boxsObj[i].transform, "Count", "UILabel").text = self.counts[i].."/"..self.maxCounts[i]
				end, {})
			end
		end
		ViewUtil.AddClickEvent(boxsObj[i], _doClick)
	end

	Network.SendMoData(ServerID.LeaguePVEChest, {}, function(session, params)
		local serviceId = tonumber(session.serviceId)
		local mo = GetProtoTable("LeagueHuntBoxMainData", session)
		
		for i = 1, #mo.logs do
			local info = mo.logs[i]
			local logTemp = CreateObject(log)
			logTemp:SetActive(true)
			logTemp.name = "Log_"..i
			HangObject(logTemp, gridObj)
			GetComponent(logTemp.transform, "UILabel").text = info.name.."开启狩猎宝箱，获得"..info.num.."个"..PropDataProxy.getPropName(PropDataProxy.getPropCfgById (info.pid))
		end
		ResetGrid(gridObj)

		self.counts = {}
		self.maxCounts = {}
		
		for i = 1, #mo.boxs do
			local info = mo.boxs[i]

			GetPathComponent(boxsObj[i].transform, "Time", "UILabel").text = Date2String(info.time/1000)

			self.counts[i] = info.maxCount - info.count
			self.maxCounts[i] = info.maxCount
			GetPathComponent(boxsObj[i].transform, "Count", "UILabel").text = self.counts[i].."/"..self.maxCounts[i]
			GetGameObject(boxsObj[i].transform, "Status"):SetActive(true)
			if self.counts[i] == 0 then
				Util.EnableButton(boxsObj[i], ButtonState.Disabled, false)
				GetGameObject(boxsObj[i].transform, "Status/Open"):SetActive(false)
				GetGameObject(boxsObj[i].transform, "Status/Close"):SetActive(true)
				GetPathComponent(boxsObj[i].transform, "Sprite", "UISprite").spriteName = "League_ChestOpen"
			end
		end
	end, {})
end