
EnterView = class(BaseView)

function EnterView:ctor()
end  

function EnterView:Awake(obj)
	self.gameObject = obj
	self.transform = obj.transform

	self:initPanel()
end

function EnterView:Start(data)
	self:initData(data)
end

function EnterView:initPanel()
 	self.top = GetPathComponent(self.transform, "Mode/Top", "UIWidget")
 	self.bottom = GetPathComponent(self.transform, "Mode/Bottom", "UIWidget")
	self.top:SetAnchor(self.root.transform)
	self.bottom:SetAnchor(self.root.transform)
	
	local bottomPath = "Mode/Bottom/Panel"
 	self.enter = GetGameObject(self.transform, bottomPath.."/Enter")
 	self.title = GetPathComponent(self.transform, bottomPath.."/SelectArea/Title", "UILabel")
 	self.areaName = GetPathComponent(self.transform, bottomPath.."/SelectArea/AreaName", "UILabel")
 	self.btnPop = GetGameObject(self.transform, bottomPath.."/SelectArea/BtnPop")

    local function doClick(obj)
		if obj == self.enter then
			if self.serverTip ~= "" then
				-- self:SetServerCache()
				Util.EnableButton(self.enter, ButtonState.Disabled, false);
				Util.EnableButton(self.btnPop, ButtonState.Disabled, false);
				ViewUtil.comAnnouncePanel(self.serverTip, function() EnterControler.Login(true) end)
			else
				-- self:SetServerCache()
				EnterControler.Login(true)
			end
		elseif obj == self.btnPop then
			self:CreateSelectServerPanel()
		end
    end

	ViewUtil.AddClickEvent(self.enter, doClick)
	ViewUtil.AddClickEvent(self.btnPop, doClick)
end

function EnterView:initData(data)
	if IsEmpty(data) then return end

	self.serverList = List:new()

	for i = 1, #data do
		self.serverList:add(data[i])
	end

	--首次登录无法取得PlayerData.GetRoleTeamId() by iorinie
	-- error("玩家ID： "..PlayerData.GetRoleTeamId())
	self.serverName = SimpleCache.getCacheValWithoutID(SimpleCache.serverName1)
	-- error("初始名字： "..self.serverName)
	self.serverIp = SimpleCache.getCacheValWithoutID(SimpleCache.serverIp1)
	-- error("初始地址： "..self.serverIp)
	self.serverId = SimpleCache.getCacheValWithoutID(SimpleCache.serverId1)
	-- error("初始ID： "..self.serverId)
	
	if self.serverName == "1" or self.serverIp == "1" or self.serverId == "1" then
		self.title.text = "区服选取"
		self.serverName = self.serverList[1].serverName
		self.serverId = self.serverList[1].sid
		self.areaName.text = self.serverId.."-"..self.serverName
		self.serverIp = self.serverList[1].serverIp
		self:SetAddress(self.serverIp)
		self.serverTip = self.serverList[1].tips
	else
		self.title.text = "上次登录"
		self.areaName.text = self.serverId.."-"..self.serverName
		self:SetAddress(self.serverIp)

		for i = 1, #self.serverList do
			if self.serverIp == self.serverList[i].serverIp then
				self.serverTip = self.serverList[i].tips
			end
		end
	end
end

function EnterView:SetAddress(_address)
	if Global.isRelease then
	    GameDefine.ServerIp = tostring(_address)
	    GameDefine.ServerPort = tonumber(8038)
	else
		GameDefine.ServerIp = tostring(_address)
	    GameDefine.ServerPort = tonumber(8038)
	end
end

function EnterView:CreateSelectServerPanel()
	local selectServer = Util.LoadPrefab("UI/SelectServer", AssetBundleInfo.assetBundles)
	selectServerObj = CreateObject(selectServer)

	local sMask = ViewUtil.showScreenMask();
    GetComponent(sMask, "UIPanel").depth = 0;
    ViewUtil.sortLayerOrder(sMask, LayerName.Dialog);

    HangObject(selectServerObj, self.gameObject);
	selectServerObj.transform.localPosition = Vector3.New(0, 0, 0);
	ViewUtil.sortLayerOrder(selectServerObj, LayerName.Top);

    local centerPath = "Mode/Center/Panel"
    local recentServer_1 = GetGameObject(selectServerObj.transform, centerPath.."/RecentServer/RecentServerTab_1")
    local recentServer_2 = GetGameObject(selectServerObj.transform, centerPath.."/RecentServer/RecentServerTab_2")
    local rs_1Lbl = GetPathComponent(recentServer_1.transform, "Label", "UILabel") --recent server 1 label
    local rs_2Lbl = GetPathComponent(recentServer_2.transform, "Label", "UILabel")
    local serverTab = GetGameObject(selectServerObj.transform, centerPath.."/ServerTab")
    local scrollViewObj = GetGameObject(selectServerObj.transform, centerPath.."/ScrollViewBG/ScrollView")

    local bottomPath = "Mode/Bottom/Panel"
    local btnClose = GetGameObject(selectServerObj.transform, bottomPath.."/BtnClose")
    
    local gridObj = GetGameObject(scrollViewObj.transform, "Panel/Grid")

    local sc_1Name = SimpleCache.getCacheValWithoutID(SimpleCache.serverName1) --server cache 1 name
    local sc_1Id = SimpleCache.getCacheValWithoutID(SimpleCache.serverId1)
    local sc_1Ip = SimpleCache.getCacheValWithoutID(SimpleCache.serverIp1)
    local sc_2Name = SimpleCache.getCacheValWithoutID(SimpleCache.serverName2)
    local sc_2Id = SimpleCache.getCacheValWithoutID(SimpleCache.serverId2)
    local sc_2Ip = SimpleCache.getCacheValWithoutID(SimpleCache.serverIp2)

    if sc_1Name == "1" then
    	rs_1Lbl.text = "无上次登录"
    else
    	rs_1Lbl.text = sc_1Id.."-"..sc_1Name
    end

    if sc_2Name == "1" then
    	rs_2Lbl.text = "无上次登录"
    else
    	rs_2Lbl.text = sc_2Id.."-"..sc_2Name
    end

    local function doClick(obj)
    	if obj == btnClose then
    		DestroyObject(selectServerObj)
    		DestroyObject(sMask)

    	elseif obj == recentServer_1 then
    		if sc_1Name == "1" then
    			ViewUtil.showMsg("请选择下方服务器!")
    			return
    		end

    		self.serverName = sc_1Name
    		self.serverId = sc_1Id
    		self.serverIp = sc_1Ip
    		self.title.text = "本次登录"
    		self.areaName.text = self.serverId.."-"..self.serverName
    		self:SetAddress(self.serverIp)
    		
    		for i = 1, #self.serverList do
				if self.serverIp == self.serverList[i].serverIp then
					self.serverTip = self.serverList[i].tips
				end
			end

    		DestroyObject(selectServerObj)
    		DestroyObject(sMask)

    	elseif obj == recentServer_2 then
    		if sc_2Name == "1" then
    			ViewUtil.showMsg("请选择下方服务器!")
    			return
    		end

			self.serverName = sc_2Name
    		self.serverId = sc_2Id
    		self.serverIp = sc_2Ip
    		self.title.text = "本次登录"
    		self.areaName.text = self.serverId.."-"..self.serverName
    		self:SetAddress(self.serverIp)
    		
    		for i = 1, #self.serverList do
				if self.serverIp == self.serverList[i].serverIp then
					self.serverTip = self.serverList[i].tips
				end
			end

    		DestroyObject(selectServerObj)
    		DestroyObject(sMask)
    	end
    end
    ViewUtil.AddClickEvent(btnClose, doClick)
    ViewUtil.AddClickEvent(recentServer_1, doClick)
    ViewUtil.AddClickEvent(recentServer_2, doClick)

    local serverTabObjs = {}
    local positionX = 0
    local positionY = 0

    for i = 1, #self.serverList do
    	serverTabObjs[i] = CreateObject(serverTab)
    	serverTabObjs[i]:SetActive(true)
    	HangObject(serverTabObjs[i], gridObj)

    	serverTabObjs[i].transform.localPosition = Vector3.New(positionX, positionY, 0)
    	positionX = positionX + 260
    	if math.mod(i, 2) ~= 1 then
    		positionX = 0
    		positionY = positionY - 80
    	end

    	GetPathComponent(serverTabObjs[i].transform, "Label", "UILabel").text = self.serverList[i].sid.."-"..self.serverList[i].serverName

    	local serverStatSpr = GetPathComponent(serverTabObjs[i].transform, "Sprite", "UISprite")
    	local serverNew = GetGameObject(serverTabObjs[i].transform, "New")

    	if self.serverList[i].status == 0 then
    		serverStatSpr.spriteName = "Server_Gray"
    	elseif self.serverList[i].status == 1 then
    		serverStatSpr.spriteName = "Server_Green"
    	elseif self.serverList[i].status == 2 then
    		serverStatSpr.spriteName = "Server_Red"
    	elseif self.serverList[i].status == 10 then
    		serverNew:SetActive(true)
    	end

    	function _callback(obj)
    		if obj == serverTabObjs[i] then
    			self.serverName = self.serverList[i].serverName
    			self.serverId = self.serverList[i].sid
    			self.serverIp = self.serverList[i].serverIp
    			self.title.text = "本次登录"
    			self.areaName.text = self.serverId.."-"..self.serverName
    			self:SetAddress(self.serverIp)
    			self.serverTip = self.serverList[i].tips

    			DestroyObject(selectServerObj)
    			DestroyObject(sMask)
    		end
    	end
    	ViewUtil.AddClickEvent(serverTabObjs[i], _callback)
    end
    -- ResetGrid(gridObj)
    -- ResetScrollView(scrollViewObj)
end

function EnterView:SetServerCache()
	-- error("设置名字: "..self.serverName)
	-- error("设置地址: "..self.serverIp)
	-- error("设置ID: "..self.serverId)

	--取缓存时，没有的不是空值，默认值是1。 by iorinie
	local serverIp1 = SimpleCache.getCacheValWithoutID(SimpleCache.serverIp1)
	local serverIp2 = SimpleCache.getCacheValWithoutID(SimpleCache.serverIp2)
	local serverIp3 = SimpleCache.getCacheValWithoutID(SimpleCache.serverIp3)

	local serverName1 = SimpleCache.getCacheValWithoutID(SimpleCache.serverName1)
	local serverName2 = SimpleCache.getCacheValWithoutID(SimpleCache.serverName2)
	local serverName3 = SimpleCache.getCacheValWithoutID(SimpleCache.serverName3)

	local serverId1 = SimpleCache.getCacheValWithoutID(SimpleCache.serverId1)
	local serverId2 = SimpleCache.getCacheValWithoutID(SimpleCache.serverId2)
	local serverId3 = SimpleCache.getCacheValWithoutID(SimpleCache.serverId3)

	if GameDefine.ServerIp == serverIp1 then
		return
	elseif GameDefine.ServerIp == serverIp2 then
		SimpleCache.setCacheWithoutID(SimpleCache.serverIp2, serverIp1)
		SimpleCache.setCacheWithoutID(SimpleCache.serverName2, serverName1)
		SimpleCache.setCacheWithoutID(SimpleCache.serverId2, serverId1)

		SimpleCache.setCacheWithoutID(SimpleCache.serverIp1, serverIp2)
		SimpleCache.setCacheWithoutID(SimpleCache.serverName1, serverName2)
		SimpleCache.setCacheWithoutID(SimpleCache.serverId1, serverId2)
	else
		SimpleCache.setCacheWithoutID(SimpleCache.serverIp3, serverIp2)
		SimpleCache.setCacheWithoutID(SimpleCache.serverName3, serverName2)
		SimpleCache.setCacheWithoutID(SimpleCache.serverId3, serverId2)

		SimpleCache.setCacheWithoutID(SimpleCache.serverIp2, serverIp1)
		SimpleCache.setCacheWithoutID(SimpleCache.serverName2, serverName1)
		SimpleCache.setCacheWithoutID(SimpleCache.serverId2, serverId1)

		SimpleCache.setCacheWithoutID(SimpleCache.serverIp1, self.serverIp)
		SimpleCache.setCacheWithoutID(SimpleCache.serverName1, self.serverName)
		SimpleCache.setCacheWithoutID(SimpleCache.serverId1, self.serverId)
	end
end

function EnterView:ShowAnnouncement(_data)
	if IsEmpty(_data) then return end

    if _data.tips ~= "" then
	   ViewUtil.comAnnouncePanel(_data.tips)
    end
end