
LoginView = class(BaseView)

function LoginView:ctor()
end  

function LoginView:Awake(obj)
	self.gameObject = obj
	self.transform = obj.transform
	self:initPanel()
	
	AudioConst.SetEffectStatus(SimpleCache.getCacheVal(SimpleCache.soundBtn))
	AudioConst.SetBgmStatus(SimpleCache.getCacheVal(SimpleCache.soundBG))
	SimpleAudio.PlayMusic("main")
end

function LoginView:Start(data)
	self:initData(data)
end

function LoginView:initPanel()
 	self.top = GetPathComponent(self.transform, "Mode/Top", "UIWidget")
 	self.bottom = GetPathComponent(self.transform, "Mode/Bottom", "UIWidget")
	self.top:SetAnchor(self.root.transform)
	self.bottom:SetAnchor(self.root.transform)
	
	local bottomPath = "Mode/Bottom/Panel"
 	self.login = GetGameObject(self.transform, bottomPath.."/Login")
 	self.register = GetGameObject(self.transform, bottomPath.."/Register")
	
	if Global.isSDKLogin then
		self.register:SetActive(false)
	else
		self.register:SetActive(true)
	end

    local function doClick(obj)
		if obj == self.login then
			Access.DoLogin()

		elseif obj == self.register then
			ViewUtil.openView(ModuleName.REG, true)
			
--			Access.DoPay(1, 10, 1, "物品名称", "物品描述")
		end
    end
	ViewUtil.AddClickEvent(self.login, doClick)
	ViewUtil.AddClickEvent(self.register, doClick)
end

function LoginView:LostFocus()
end

function LoginView:createLoginDialog()
	local loginObj = CreateObjectByPath("UI/LoginDialog", AssetBundleInfo.assetBundles)
	local shade = ViewUtil.CreateShadeView(function()
--		DestroyObject(loginObj)
	end)
	HangObject(shade, loginObj)
	HangObject(loginObj, self.gameObject)
	ViewUtil.sortLayerOrder(loginObj, LayerName.Dialog)
	
	local centerPath = "Mode/Center/Panel"
	local btnBack = GetGameObject(loginObj.transform, centerPath.."/BtnBack")
	local account = GetPathComponent(loginObj.transform, centerPath.."/InputAccount", "UIInput")
	local pwd = GetPathComponent(loginObj.transform, centerPath.."/InputPassword", "UIInput")
 	local confirm = GetGameObject(loginObj.transform, centerPath.."/BtnConfirm")
	
	local saveInfo = PlayerData.loadAccount()
	if not IsEmptyTable(saveInfo) then
		account.value = saveInfo.acc
		pwd.value = saveInfo.pwd
	end

    local function doClick(obj)
		if obj == btnBack then
			CancelDelayCall(self.delayLogin)
			DestroyObject(loginObj)
		elseif obj == confirm then
			Util.EnableButton(obj, ButtonState.Disabled, false)
			CancelDelayCall(self.delayLogin)
			self.delayLogin = DelayCall(function()
				Util.EnableButton(obj, ButtonState.Normal, true)
			end, 3)

			local account = account.value
			local pwd = pwd.value

			local uri = tostring(self:GetLoginURI(account, pwd))
			RequestURL(uri, function(data, args)
				local info = cjson.decode(data)
				if IsEmpty(info) then
					ViewUtil.showMsg("登录服务器拒绝访问!")
					return
				end
			
				local ret = tonumber(info.ret)
				local msg = tostring(info.msg)
				local userId = info.userId
				local key = info.key
				local timestamp = info.time
				if ret == 0 then
					PlayerData.setLoginAccount(account, pwd)
					PlayerData.setLoginData(userId, key, timestamp)
					ViewUtil.openView(ModuleName.ENTER)
					--sdk登录不走这里 by iorinie
					self:GetAnnouncementInfo()
					-- self:GetServerInfo()
					ViewUtil.closeView(ModuleName.LOGIN)
					CancelDelayCall(self.delayLogin)
					self:closeView()
				else
					ViewUtil.showMsg(msg)
				end
				if obj ~= nil then
					Util.EnableButton(obj, ButtonState.Normal, true)
				end

			end)
		end
	end
	-- UIEventListener.Get(btnBack).onClick = doClick
	-- UIEventListener.Get(confirm).onClick = doClick
	ViewUtil.AddClickEvent(btnBack, doClick)
	ViewUtil.AddClickEvent(confirm, doClick)
end

function LoginView:GetLoginURI(acount, pwd)
	if IsEmpty(acount) or IsEmpty(pwd) then
		ViewUtil.showMsg("账号或密码输入非法")
		return "";
	end

	local ret = nil
	if Global.isRelease then
		ret = string.format(
			"http://will.aboilgame.com/will/login?accountName=%s&password=%s&source=%s&system=%s",
			EncodeURI(acount), EncodeURI(pwd), "ftx", "Android")
	else
		ret = string.format(
			"http://192.168.10.177/login_xgame.php?accountName=%s&password=%s&source=%s&system=%s",
			EncodeURI(acount), EncodeURI(pwd), "ftx", "Android")
	end
	return ret
end

function LoginView:initData(data)
--	local function _onDrag(tbl)
--		local curPos = UICamera.currentTouch.delta
--		local lastPos = UICamera.currentTouch.totalDelta
--		plog(curPos, "delta")
--		plog(lastPos, "totalDelta")
----		plog(Util.ScreenToWorld(Vector3.New(curPos.x, curPos.y, 0)))
----	local dir = Util.GetTouchDirection()
----	plog("Dir = ", dir)
--	end
--	ViewUtil.AddEventTrigger(self.login, EventConst.onDrag, _onDrag, {})

	if IsEmpty(data) then return end
end

function LoginView:Update()
end

--获取Enter界面需要选服的服务器信息，已移至EnterControler。 by iorinie
-- function LoginView:GetServerInfo()
-- 	local uri = ""
-- 	if Global.isRelease then
-- 		uri = tostring(string.format("http://121.10.118.50:8080/xgame/server/select?type=0", nil, nil, "ftx", "Android"))
-- 	else
-- 		uri = tostring(string.format("http://192.168.10.154:8080/xgame/server/select?type=0", nil, nil, "ftx", "Android"))
-- 	end
-- 	RequestURL(uri, function(data, args)
-- 		local info = cjson.decode(data)
-- 		if IsEmpty(info) then
-- 			ViewUtil.showMsg("无服务器信息!")
-- 			return
-- 		end

-- 		local ctrl = ModuleManager.GetCtrl(ModuleName.ENTER)
-- 		ctrl.data = info
-- 		ctrl.Start()
-- 	end)
-- end

--获取全服公告信息 by iorinie
function LoginView:GetAnnouncementInfo()
	local uri = ""
	if Global.isRelease then
		uri = tostring(string.format("http://121.10.118.50:8080/xgame/server/tips?type=0", nil, nil, "ftx", "Android"))
	else
		uri = tostring(string.format("http://192.168.10.154:8080/xgame/server/tips?type=0", nil, nil, "ftx", "Android"))
	end
	RequestURL(uri, function(data, args)
		local info = cjson.decode(data)
		if IsEmpty(info) then
			ViewUtil.showMsg("无全服公告!")
			return
		end
		local ctrl = ModuleManager.GetCtrl(ModuleName.ENTER)
		ctrl.view:ShowAnnouncement(info)
	end)
end