HeroClass = class(BaseHero)

function HeroClass:ctor()
	self.heroObj = nil
	self.onlyId = 0					-- 唯一ID
	self.heroId = 0					-- 英雄ID
	self.prefabName = nil
	self.heroName = nil
	self.placeId = 0
	self.data = nil
	self.propAtk = 0
	self.propDef = 0
	self.propDodge = 0
	self.propKing = 0
	self.level = 0
	self.wiseup = 0
	self.strengthen = 0
	self.soldiers = 0
end

function HeroClass:init(data, placeId)
	self.placeId = placeId or self.placeId
	if IsEmpty(data) then return end
	self.data = CopyTable(data)
	self.onlyId = tostring(data.id)
	self.heroId = tostring(data.heroId)

	local herosConf = Config.getHerosData()
	local conf = herosConf[self.heroId]
	self.conf = conf
	if IsEmptyTable(conf) then
		return
	end
	self.prefabName = conf["#prefab_name"]
	self.quality = conf["quality"]
	self.heroName = conf["hero_name"]

	if tonumber(self.heroId) <= TeamData.maxRoleHeroId then			--主角信息需要特殊处理
		self.skillIds = {}
		local skillIdsStr = ""

		local totalAtk = 0
      	local totalDef = 0
      	local totalDodge = 0
      	local totalKing = 0

		local equipsData = Config.getEquipsData()

		local equiList = TeamData.GetRoleEquiList()
		if equiList ~= nil then
			local nonWeaponId = List:new()
			for k, v in pairs(equiList) do
				if v.type == 1 and v.status == 1 then
					skillIdsStr = skillIdsStr..":"..tostring(equipsData[tostring(v.equId)].skill_id)
				end

				if v.type ~= 1 then
      				nonWeaponId:add(v.equId)
      			end

      			totalAtk = totalAtk + equipsData[tostring(v.equId)].xg
      			totalDef = totalDef + equipsData[tostring(v.equId)].fy
      			totalDodge = totalDodge + equipsData[tostring(v.equId)].sb
      			totalKing = totalKing + equipsData[tostring(v.equId)].wz
      		end
      		for i = 1, #nonWeaponId do
      			skillIdsStr = skillIdsStr..":"..tostring(equipsData[tostring(nonWeaponId[i])].skill_id)
      		end
      	end

      	if skillIdsStr ~= "" then
      		skillIdsStr = (string.gsub(skillIdsStr, ":", "", 1))
      		table.insert(self.skillIds, skillIdsStr)
      	end

		self.propAtk = totalAtk				-- 先攻
		self.propDef = totalDef				-- 防御
		self.propDodge = totalDodge		-- 闪避
		self.propKing = totalKing			-- 王者
	else
		self.skillIds = split(conf["skill_id"], ",")
		self.propAtk = tonumber(conf["xg"]) or 0					-- 先攻
		self.propDef = tonumber(conf["fy"]) or 0					-- 防御
		self.propDodge = tonumber(conf["sb"])	 or 0				-- 闪避
		self.propKing = tonumber(conf["wz"]	) or 0				-- 王者
	end

	self.level = tonumber(data.level) or 1							-- 等级
	self.wiseup = tonumber(data.wiseup) or 0					-- 觉醒
	self.strengthen = tonumber(data.strengthen) or 0		-- 强化
	self.power = HeroDataProxy.getHeroAbility(self.heroId, self.level, self.wiseup, self.strengthen)
end

function HeroClass:refresh()
	local allHeros = TeamData.GetAllHeros()
	for _, hero in pairs(allHeros) do
		if tostring(hero.id) == tostring(self.onlyId) then
--			self.level = tonumber(hero.level) or 1								-- 等级
--			self.wiseup = tonumber(hero.wiseup) or 0						-- 觉醒
--			self.strengthen = tonumber(hero.strengthen) or 0		-- 强化
----			self.soldiers = tonumber(hero.soldiers) or 0
--			self.power = HeroDataProxy.getHeroAbility(self.heroId, self.level, self.wiseup, self.strengthen)
			self:init(hero)
			self:refreshHero()
			return
		end
	end
end

function HeroClass:getPlaceId()
	return self.placeId
end

function HeroClass:setPlaceId(placeId)
	self.placeId = placeId
end

function HeroClass:getData()
	return self.data
end

function HeroClass:createHero()
	local hero = self.super.createHero(self, "UI/Hero")
	ViewUtil.addDragScrollView(hero)
	self.heroObj = hero
	return hero
end

function HeroClass:createHeroInstead()
	local hero = self.super.createHero(self, "UI/HeroInstead")
	hero.name = "NoName"
	ViewUtil.addDragScrollView(hero)
	ViewUtil.sortLayerOrder(hero, LayerName.UI)
	self.heroObj = hero
	return hero
end

function HeroClass:createHeroCond(gateId)
	if not gateId then return end
	local hero = self.super.createHero(self, "UI/HeroOpenCond")
	hero.name = "NoUsed"
	local cond = GetPathComponent(hero.transform, "Cond", "UILabel")
	cond.text = "第"..gateId.."关"
	ViewUtil.addDragScrollView(hero)
	ViewUtil.sortLayerOrder(hero, LayerName.UI)
	self.heroObj = hero
	return hero
end

function HeroClass:createHeroEmpty()
	local hero = self.super.createHero(self, "UI/HeroEmpty")
	hero.name = "Empty"
	ViewUtil.addDragScrollView(hero)
	ViewUtil.sortLayerOrder(hero, LayerName.UI)
	self.heroObj = hero
	return hero
end

function HeroClass:bindHeroObj(hero)
	self.heroObj = hero
end
